@@ -1,17 +1,17 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2000 by Brian V. Smith
+ * Parts Copyright (c) 1989-2007 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
@@ -25,30 +25,45 @@
 #include "u_search.h"
 #include "u_undo.h"
 #include "w_canvas.h"
+#include "w_drawprim.h"
 #include "w_mousefun.h"
+#include "w_msgpanel.h"
 #include "w_modepanel.h"
 #include "w_zoom.h"
 
-static void	init_join();
-static void	init_split();
-static void	join_lines();
-static void	join_splines();
-static void	split_line();
-static void	split_spline();
-static void	cancel_join();
-static void	join_line2();
-static void	join_spline2();
-static Boolean	connect_line_points();
-static Boolean	connect_spline_points();
+#include "e_addpt.h"
+#include "f_util.h"
+#include "u_free.h"
+#include "u_markers.h"
+#include "u_redraw.h"
+#include "w_cursor.h"
+
+static void	init_join(F_line *obj, int type, int x, int y, F_point *p, F_point *q);
+static void	init_split(F_line *obj, int type, int x, int y, int px, int py);
+static void	join_lines(F_line *line, F_point *prev_point, F_point *selected_point);
+static void	join_splines(F_spline *spline, F_point *prev_point, F_point *selected_point);
+static void	split_line(int px, int py);
+static void	split_spline(int px, int py);
+static void	cancel_join(void);
+static void	join_line2(F_line *obj, int type, int x, int y, F_point *p, F_point *q);
+static void	join_spline2(F_line *obj, int type, int x, int y, F_point *p, F_point *q);
+static Boolean	connect_line_points(F_line *line1, Boolean first1, F_line *line2, Boolean first2, F_line *new_line);
+static Boolean	connect_spline_points(F_spline *spline1, Boolean first1, F_spline *spline2, Boolean first2, F_spline *new_spline);
+
+
+void draw_join_marker (F_point *point);
+void split_polygon (F_line *poly, F_line *line, F_point *point);
+void split_cspline (F_spline *cspline, F_spline *spline, F_point *point, F_sfactor *csf);
 
 void
-join_split_selected()
+join_split_selected(void)
 {
     set_mousefun("Join Lines/Splines", "Split Line/Spline", "", 
 			LOC_OBJ, LOC_OBJ, LOC_OBJ);
     draw_mousefun_canvas();
     canvas_kbd_proc = null_proc;
     canvas_locmove_proc = null_proc;
+    canvas_ref_proc = null_proc;
     init_searchproc_left(init_join);
     init_searchproc_middle(init_split);
     canvas_leftbut_proc = point_search_left;		/* point search for join */
@@ -59,19 +74,15 @@ join_split_selected()
     /* (the markers are originally set this way from the mode panel, but
        we need to set them again after the previous join/split */
     update_markers(M_POLYLINE | M_SPLINE_O | M_SPLINE_C);
+    reset_action_on();
 }
 
 static void
-init_join(obj, type, x, y, p, q)
-    F_line	   *obj;
-    int		    type;
-    int		    x, y;
-    F_point	   *p, *q;
+init_join(F_line *obj, int type, int x, int y, F_point *p, F_point *q)
 {
     switch (type) {
       case O_POLYLINE:
 	cur_l = (F_line *) obj;
-	canvas_middlebut_proc = null_proc;
 	join_lines(cur_l, p, q);
 	break;
       case O_SPLINE:
@@ -84,15 +95,16 @@ init_join(obj, type, x, y, p, q)
 }
 
 static void
-init_split(obj, type, x, y, px, py)
-    F_line	   *obj;
-    int		    type;
-    int		    x, y;
-    int		    px, py;
+init_split(F_line *obj, int type, int x, int y, int px, int py)
 {
     switch (type) {
       case O_POLYLINE:
 	cur_l = (F_line *) obj;
+	if (cur_l->type == T_PICTURE) {
+	    put_msg("Can't remove points from picture");
+	    beep();
+	    return;
+	}
 	split_line(px, py);
 	break;
       case O_SPLINE:
@@ -110,9 +122,7 @@ static F_point	*sel_point;
 static Boolean	 first1, first2;
 
 static void
-join_lines(line, prev_point, selected_point)
-    F_line	   *line;
-    F_point	   *prev_point, *selected_point;
+join_lines(F_line *line, F_point *prev_point, F_point *selected_point)
 {
 	/* can only join lines (not boxes or polygons) */
 	if (line->type != T_POLYLINE)
@@ -136,9 +146,7 @@ join_lines(line, prev_point, selected_point)
 }
 
 static void
-join_splines(spline, prev_point, selected_point)
-    F_spline	   *spline;
-    F_point	   *prev_point, *selected_point;
+join_splines(F_spline *spline, F_point *prev_point, F_point *selected_point)
 {
 	/* can only join open splines */
 	if (spline->type & 1)
@@ -164,8 +172,7 @@ join_splines(spline, prev_point, selected_point)
 	update_markers(M_SPLINE_O);
 }
 
-draw_join_marker(point)
-    F_point	   *point;
+void draw_join_marker(F_point *point)
 {
 	int x=ZOOMX(point->x), y=ZOOMY(point->y);
 	pw_vector(canvas_win, x-10, y-10, x-10, y+10, INV_PAINT,1,PANEL_LINE,0.0,MAGENTA);
@@ -174,8 +181,7 @@ draw_join_marker(point)
 	pw_vector(canvas_win, x+10, y-10, x-10, y-10, INV_PAINT,1,PANEL_LINE,0.0,MAGENTA);
 }
 
-erase_join_marker(point)
-    F_point	   *point;
+void erase_join_marker(F_point *point)
 {
     int x=ZOOMX(point->x), y=ZOOMY(point->y);
     if (point) {
@@ -189,7 +195,7 @@ erase_join_marker(point)
 
 
 static void
-cancel_join()
+cancel_join(void)
 {
     join_split_selected();
     /* erase any marker */
@@ -198,11 +204,7 @@ cancel_join()
 }
 
 static void
-join_line2(obj, type, x, y, p, q)
-    F_line	   *obj;
-    int		    type;
-    int		    x, y;
-    F_point	   *p, *q;
+join_line2(F_line *obj, int type, int x, int y, F_point *p, F_point *q)
 {
 	F_line	   *line;
 	F_point	   *lastp;
@@ -214,9 +216,12 @@ join_line2(obj, type, x, y, p, q)
 		return;
 	line = (F_line *) obj;
 	first2 = (p == NULL);
+	/* if user clicked on same point twice return */
+	if (line == line1 && first1 == first2)
+	    return;
 	new_l = copy_line(line1);
 	/* if user clicked on both endpoints of a line, make it a POLYGON */
-	if (line == line1) {
+	if (line == line1 && first1 != first2) {
 	    new_l->type = T_POLYGON;
 	    lastp = last_point(new_l->points);
 	    append_point(new_l->points->x, new_l->points->y, &lastp);
@@ -246,11 +251,7 @@ join_line2(obj, type, x, y, p, q)
 }
 
 static void
-join_spline2(obj, type, x, y, p, q)
-    F_line	   *obj;
-    int		    type;
-    int		    x, y;
-    F_point	   *p, *q;
+join_spline2(F_line *obj, int type, int x, int y, F_point *p, F_point *q)
 {
 	F_spline   *spline;
 
@@ -302,9 +303,7 @@ join_spline2(obj, type, x, y, p, q)
 */
 
 static Boolean
-connect_line_points(line1, first1, line2, first2, new_line)
-    F_line	   *line1, *line2, *new_line;
-    Boolean	    first1, first2;
+connect_line_points(F_line *line1, Boolean first1, F_line *line2, Boolean first2, F_line *new_line)
 {
 	F_point    *p1,*p2;
 
@@ -355,9 +354,7 @@ connect_line_points(line1, first1, line2, first2, new_line)
 */
 
 static Boolean
-connect_spline_points(spline1, first1, spline2, first2, new_spline)
-    F_spline	   *spline1, *spline2, *new_spline;
-    Boolean	    first1, first2;
+connect_spline_points(F_spline *spline1, Boolean first1, F_spline *spline2, Boolean first2, F_spline *new_spline)
 {
 	F_point    *p1, *p2;
 	F_sfactor  *sf1, *sf2;
@@ -433,8 +430,7 @@ connect_spline_points(spline1, first1, spline2, first2, new_spline)
 /**************************  split line or spline  *******************************/
 
 static void
-split_line(px, py)
-    int		    px, py;
+split_line(int px, int py)
 {
     F_point	   *p;
     F_line	   *new_l1, *new_l2, *save_l;
@@ -445,7 +441,7 @@ split_line(px, py)
     if (cur_l->points->next == NULL)
 	return;				/* A single point line - that would be tough to split */
 
-    if (cur_l->type != T_POLYGON && cur_l->type != T_BOX && cur_l->type != T_ARC_BOX) {
+    if (cur_l->type != T_POLYGON && cur_l->type != T_BOX && cur_l->type != T_ARCBOX) {
 	if (left_point == NULL) 	/* selected_point is the first point */
 	    return;
 	else if (right_point == NULL)	/* last point */
@@ -460,7 +456,7 @@ split_line(px, py)
     /* remove original line from the objects */
     delete_line(cur_l);
 
-    if (cur_l->type == T_POLYGON || cur_l->type == T_BOX || cur_l->type == T_ARC_BOX) {
+    if (cur_l->type == T_POLYGON || cur_l->type == T_BOX || cur_l->type == T_ARCBOX) {
 	/* change polygon or box to polyline */
 
 	/* search original for point */
@@ -508,9 +504,7 @@ split_line(px, py)
 
 /* turn polygon or box "poly" into polyline "line" by breaking at point "point" */
 
-split_polygon(poly, line, point)
-    F_line	   *poly, *line;
-    F_point	   *point;
+void split_polygon(F_line *poly, F_line *line, F_point *point)
 {
     F_point	   *splitp, *p, *q;
 
@@ -546,8 +540,7 @@ split_polygon(poly, line, point)
 
 
 static void
-split_spline(px, py)
-    int		    px, py;
+split_spline(int px, int py)
 {
     F_spline	   *new_spl1, *new_spl2, *save_spl;
     F_point	   *p, *cp, *left_point, *right_point;
@@ -643,10 +636,7 @@ split_spline(px, py)
 
 /* turn closed spline "cspline" open spline "spline" by breaking at point "point" */
 
-split_cspline(cspline, spline, point, csf)
-    F_spline	   *cspline, *spline;
-    F_point	   *point;
-    F_sfactor	   *csf;
+void split_cspline(F_spline *cspline, F_spline *spline, F_point *point, F_sfactor *csf)
 {
     F_point	   *splitp, *p;
     F_sfactor	   *sf;
