@@ -1,17 +1,17 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2000 by Brian V. Smith
+ * Parts Copyright (c) 1989-2007 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
@@ -33,45 +33,54 @@
 #include "w_mousefun.h"
 #include "w_msgpanel.h"
 
+#include "f_util.h"
+#include "u_geom.h"
+#include "u_redraw.h"
+#include "w_cursor.h"
+
 /* local routine declarations */
 
 static F_point *moved_point;
 
-static void	fix_movedcompoundpoint();
-static void	cancel_compound();
+static void	fix_movedcompoundpoint(int x, int y);
+static void	cancel_compound(void);
 
-static Boolean	init_ellipsepointmoving();
-static void	init_arcpointmoving();
-static void	init_linepointmoving();
-static void	init_splinepointmoving();
-static void	init_compoundpointmoving();
+static Boolean	init_ellipsepointmoving(void);
+static void	init_arcpointmoving(void);
+static void	init_linepointmoving(void);
+static void	init_splinepointmoving(void);
+static void	init_compoundpointmoving(void);
 
-static void	relocate_arcpoint();
-static void	relocate_ellipsepoint();
-static void	relocate_linepoint();
-static void	relocate_splinepoint();
+static void	relocate_arcpoint(F_arc *arc, int x, int y, int movedpoint_num);
+static void	relocate_ellipsepoint(F_ellipse *ellipse, int x, int y, int point_num);
+static void	relocate_linepoint(F_line *line, int x, int y, F_point *moved_point, F_point *left_point);
+static void	relocate_splinepoint(F_spline *s, int x, int y, F_point *moved_point);
 
-static Boolean	init_move_point();
-static void	init_arb_move_point();
-static void	init_stretch_move_point();
+static Boolean	init_move_point(F_line *obj, int type, int x, int y, F_point *p, F_point *q);
+static void	init_arb_move_point(F_line *obj, int type, int x, int y, F_point *p, F_point *q);
+static void	init_stretch_move_point(F_line *obj, int type, int x, int y, F_point *p, F_point *q);
 
-static void	fix_movedarcpoint();
-static void	fix_movedellipsepoint();
-static void	fix_movedsplinepoint();
-static void	fix_box();
-static void	fix_movedlinepoint();
+static void	fix_movedarcpoint(int x, int y);
+static void	fix_movedellipsepoint(int x, int y);
+static void	fix_movedsplinepoint(int x, int y);
+static void	fix_box(int x, int y);
+static void	fix_movedlinepoint(int x, int y);
 
-static void	cancel_movedarcpoint();
-static void	cancel_movedellipsepoint();
-static void	cancel_movedsplinepoint();
-static void	cancel_movept_box();
-static void	cancel_movedlinepoint();
+static void	cancel_movedarcpoint(void);
+static void	cancel_movedellipsepoint(void);
+static void	cancel_movedsplinepoint(void);
+static void	cancel_movept_box(void);
+static void	cancel_movedlinepoint(void);
 
-move_point_selected()
+
+void assign_newboxpoint (F_line *b, int x1, int y1, int x2, int y2);
+
+void move_point_selected(void)
 {
     set_mousefun("move point", "horiz/vert move", "", LOC_OBJ, LOC_OBJ, LOC_OBJ);
     canvas_kbd_proc = null_proc;
     canvas_locmove_proc = null_proc;
+    canvas_ref_proc = null_proc;
     init_searchproc_left(init_arb_move_point);
     init_searchproc_middle(init_stretch_move_point);
     canvas_leftbut_proc = point_search_left;
@@ -79,13 +88,11 @@ move_point_selected()
     canvas_rightbut_proc = null_proc;
     set_cursor(pick9_cursor);
     force_anglegeom();
+    reset_action_on();
 }
 
 static void
-init_arb_move_point(obj, type, x, y, p, q)
-    F_line	   *obj;
-    int		    type, x, y;
-    F_point	   *p, *q;
+init_arb_move_point(F_line *obj, int type, int x, int y, F_point *p, F_point *q)
 {
     constrained = MOVE_ARB;
     if (!init_move_point(obj, type, x, y, p, q))
@@ -96,10 +103,7 @@ init_arb_move_point(obj, type, x, y, p, q)
 }
 
 static void
-init_stretch_move_point(obj, type, x, y, p, q)
-    F_line	   *obj;
-    int		    type, x, y;
-    F_point	   *p, *q;
+init_stretch_move_point(F_line *obj, int type, int x, int y, F_point *p, F_point *q)
 {
     constrained = MOVE_HORIZ_VERT;
     if (!init_move_point(obj, type, x, y, p, q))
@@ -111,25 +115,22 @@ init_stretch_move_point(obj, type, x, y, p, q)
 }
 
 static Boolean
-init_move_point(obj, type, x, y, p, q)
-    F_line	   *obj;
-    int		    type, x, y;
-    F_point	   *p, *q;
+init_move_point(F_line *obj, int type, int x, int y, F_point *p, F_point *q)
 {
     left_point = p;
     moved_point = q;
     switch (type) {
-    case O_POLYLINE:
+      case O_POLYLINE:
 	cur_l = (F_line *) obj;
 	right_point = q->next;
 	init_linepointmoving();
 	break;
-    case O_SPLINE:
+      case O_SPLINE:
 	cur_s = (F_spline *) obj;
 	right_point = q->next;
 	init_splinepointmoving();
 	break;
-    case O_ELLIPSE:
+      case O_ELLIPSE:
 	force_noanglegeom();
 	/* dirty trick - arcpoint_num is stored in p */
 	movedpoint_num = (int) p;
@@ -137,14 +138,14 @@ init_move_point(obj, type, x, y, p, q)
 	if (!init_ellipsepointmoving()) /* selected center, ignore */
 	    return False;
 	break;
-    case O_ARC:
+      case O_ARC:
 	force_noanglegeom();
 	/* dirty trick - arcpoint_num is stored in p */
 	movedpoint_num = (int) p;
 	cur_a = (F_arc *) obj;
 	init_arcpointmoving();
 	break;
-    case O_COMPOUND:
+      case O_COMPOUND:
 	force_noanglegeom();
 	/* dirty trick - posn of corner is stored in p and q */
 	cur_x = (int) p;
@@ -159,7 +160,7 @@ init_move_point(obj, type, x, y, p, q)
 }
 
 static void
-wrapup_movepoint()
+wrapup_movepoint(void)
 {
     reset_action_on();
     move_point_selected();
@@ -169,7 +170,7 @@ wrapup_movepoint()
 /*************************  ellipse  *******************************/
 
 static		Boolean
-init_ellipsepointmoving()
+init_ellipsepointmoving(void)
 {
     double	    dx, dy, l;
 
@@ -208,19 +209,19 @@ init_ellipsepointmoving()
     switch (cur_e->type) {
       case T_ELLIPSE_BY_RAD:
 	canvas_locmove_proc = constrained_resizing_ebr;
-	elastic_ebr();
+	canvas_ref_proc = elastic_ebr;
 	break;
       case T_CIRCLE_BY_RAD:
 	canvas_locmove_proc = resizing_cbr;
-	elastic_cbr();
+	canvas_ref_proc = elastic_cbr;
 	break;
       case T_ELLIPSE_BY_DIA:
 	canvas_locmove_proc = constrained_resizing_ebd;
-	elastic_ebd();
+	canvas_ref_proc = elastic_ebd;
 	break;
       case T_CIRCLE_BY_DIA:
 	canvas_locmove_proc = resizing_cbd;
-	elastic_cbd();
+	canvas_ref_proc = elastic_cbd;
 	break;
     }
     /* show current radius(ii) */
@@ -234,7 +235,33 @@ init_ellipsepointmoving()
 }
 
 static void
-cancel_movedellipsepoint()
+cancel_movedellipsepoint(void)
+{
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
+    /* erase elastic version */
+    switch (cur_e->type) {
+      case T_ELLIPSE_BY_RAD:
+	elastic_ebr();
+	break;
+      case T_CIRCLE_BY_RAD:
+	elastic_cbr();
+	break;
+      case T_ELLIPSE_BY_DIA:
+	elastic_ebd();
+	break;
+      case T_CIRCLE_BY_DIA:
+	elastic_cbd();
+	break;
+    }
+    /* redraw original ellipse */
+    redisplay_ellipse(cur_e);
+    /* turn back on all relevant markers */
+    update_markers(new_objmask);
+    wrapup_movepoint();
+}
+
+static void
+fix_movedellipsepoint(int x, int y)
 {
     switch (cur_e->type) {
       case T_ELLIPSE_BY_RAD:
@@ -250,29 +277,7 @@ cancel_movedellipsepoint()
 	elastic_cbd();
 	break;
     }
-    /* turn back on all relevant markers */
-    update_markers(new_objmask);
-    wrapup_movepoint();
-}
-
-static void
-fix_movedellipsepoint(x, y)
-    int		    x, y;
-{
-    switch (cur_e->type) {
-    case T_ELLIPSE_BY_RAD:
-	elastic_ebr();
-	break;
-    case T_CIRCLE_BY_RAD:
-	elastic_cbr();
-	break;
-    case T_ELLIPSE_BY_DIA:
-	elastic_ebd();
-	break;
-    case T_CIRCLE_BY_DIA:
-	elastic_cbd();
-	break;
-    }
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     adjust_box_pos(x, y, from_x, from_y, &cur_x, &cur_y);
     new_e = copy_ellipse(cur_e);
     relocate_ellipsepoint(new_e, cur_x, cur_y, movedpoint_num);
@@ -287,9 +292,7 @@ fix_movedellipsepoint(x, y)
 }
 
 static void
-relocate_ellipsepoint(ellipse, x, y, point_num)
-    F_ellipse	   *ellipse;
-    int		    x, y, point_num;
+relocate_ellipsepoint(F_ellipse *ellipse, int x, int y, int point_num)
 {
     double	    dx, dy;
 
@@ -313,23 +316,23 @@ relocate_ellipsepoint(ellipse, x, y, point_num)
     }
     cur_angle = ellipse->angle;
     switch (ellipse->type) {
-    case T_ELLIPSE_BY_RAD:
+      case T_ELLIPSE_BY_RAD:
 	ellipse->radiuses.x = abs(x - fix_x);
 	ellipse->radiuses.y = abs(y - fix_y);
 	break;
-    case T_CIRCLE_BY_RAD:
+      case T_CIRCLE_BY_RAD:
 	dx = fix_x - x;
 	dy = fix_y - y;
 	ellipse->radiuses.x = round(sqrt(dx * dx + dy * dy));
 	ellipse->radiuses.y = ellipse->radiuses.x;
 	break;
-    case T_ELLIPSE_BY_DIA:
+      case T_ELLIPSE_BY_DIA:
 	ellipse->center.x = (fix_x + x) / 2;
 	ellipse->center.y = (fix_y + y) / 2;
 	ellipse->radiuses.x = abs(ellipse->center.x - fix_x);
 	ellipse->radiuses.y = abs(ellipse->center.y - fix_y);
 	break;
-    case T_CIRCLE_BY_DIA:
+      case T_CIRCLE_BY_DIA:
 	dx = ellipse->center.x = round((fix_x + x) / 2);
 	dy = ellipse->center.y = round((fix_y + y) / 2);
 	dx -= x;
@@ -344,7 +347,7 @@ relocate_ellipsepoint(ellipse, x, y, point_num)
 /***************************  arc  *********************************/
 
 static void
-init_arcpointmoving()
+init_arcpointmoving(void)
 {
     set_action_on();
     /* turn off all markers */
@@ -353,6 +356,7 @@ init_arcpointmoving()
     cur_y = cur_a->point[movedpoint_num].y;
     set_cursor(crosshair_cursor);
     canvas_locmove_proc = reshaping_arc;
+    canvas_ref_proc = elastic_arclink;
     canvas_leftbut_proc = fix_movedarcpoint;
     canvas_rightbut_proc = cancel_movedarcpoint;
     elastic_arclink();
@@ -361,8 +365,9 @@ init_arcpointmoving()
 }
 
 static void
-cancel_movedarcpoint()
+cancel_movedarcpoint(void)
 {
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     elastic_arclink();
     /* erase last lengths if appres.showlengths is true */
     erase_lengths();
@@ -372,9 +377,9 @@ cancel_movedarcpoint()
 }
 
 static void
-fix_movedarcpoint(x, y)
-    int		    x, y;
+fix_movedarcpoint(int x, int y)
 {
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     elastic_arclink();
     /* erase last lengths if appres.showlengths is true */
     erase_lengths();
@@ -393,9 +398,7 @@ fix_movedarcpoint(x, y)
 }
 
 static void
-relocate_arcpoint(arc, x, y, movedpoint_num)
-    F_arc	   *arc;
-    int		    x, y, movedpoint_num;
+relocate_arcpoint(F_arc *arc, int x, int y, int movedpoint_num)
 {
     float	    xx, yy;
     F_pos	    p[3];
@@ -406,6 +409,7 @@ relocate_arcpoint(arc, x, y, movedpoint_num)
     p[movedpoint_num].x = x;
     p[movedpoint_num].y = y;
     if (compute_arccenter(p[0], p[1], p[2], &xx, &yy)) {
+
 	arc->point[movedpoint_num].x = x;
 	arc->point[movedpoint_num].y = y;
 	arc->center.x = xx;
@@ -417,7 +421,7 @@ relocate_arcpoint(arc, x, y, movedpoint_num)
 /**************************  spline  *******************************/
 
 static void
-init_splinepointmoving()
+init_splinepointmoving(void)
 {
     set_action_on();
     /* turn off all markers */
@@ -436,22 +440,27 @@ init_splinepointmoving()
             }
 	    if (latexline_mode || latexarrow_mode) {
                 canvas_locmove_proc = latex_line;
+		canvas_ref_proc = elastic_line;
                 cur_latexcursor = crosshair_cursor;
             } else if (mountain_mode || manhattan_mode) {
                 canvas_locmove_proc = constrainedangle_line;
+		canvas_ref_proc = elastic_line;
             } else {
                 /* freehand line */
                 canvas_locmove_proc = reshaping_line;
+		canvas_ref_proc = elastic_linelink;
             }
 	} else {
             /* linelink, always freehand */
 	    force_noanglegeom();
 	    canvas_locmove_proc = reshaping_line;
+	    canvas_ref_proc = elastic_linelink;
 	}
     } else {
 	/* must be closed spline */
 	force_noanglegeom();
 	canvas_locmove_proc = reshaping_line;
+	canvas_ref_proc = elastic_linelink;
 	if (left_point == NULL) {
 	    for (left_point = right_point;
 		 left_point->next != NULL;
@@ -469,8 +478,9 @@ init_splinepointmoving()
 }
 
 static void
-cancel_movedsplinepoint()
+cancel_movedsplinepoint(void)
 {
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     elastic_linelink();
     /* erase last lengths if appres.showlengths is true */
     erase_lengths();
@@ -480,10 +490,10 @@ cancel_movedsplinepoint()
 }
 
 static void
-fix_movedsplinepoint(x, y)
-    int		    x, y;
+fix_movedsplinepoint(int x, int y)
 {
     (*canvas_locmove_proc) (x, y);
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     elastic_linelink();
     /* erase last lengths if appres.showlengths is true */
     erase_lengths();
@@ -503,10 +513,7 @@ fix_movedsplinepoint(x, y)
 }
 
 static void
-relocate_splinepoint(s, x, y, moved_point)
-    F_spline	   *s;
-    int		    x, y;
-    F_point	   *moved_point;
+relocate_splinepoint(F_spline *s, int x, int y, F_point *moved_point)
 {
     moved_point->x = x;
     moved_point->y = y;
@@ -516,9 +523,10 @@ relocate_splinepoint(s, x, y, moved_point)
 /***************************  compound	********************************/
 
 static void
-init_compoundpointmoving()
+init_compoundpointmoving(void)
 {
     double	    dx, dy, l;
+    F_line	   *line, *dum;
 
     set_action_on();
     if (cur_x == cur_c->nwcorner.x)
@@ -535,6 +543,16 @@ init_compoundpointmoving()
     update_markers(0);
     set_cursor(crosshair_cursor);
     elastic_box(fix_x, fix_y, cur_x, cur_y);
+    /* is this a dimension line? */
+    if (dimline_components(cur_c, &line, &dum, &dum, &dum) && line) {
+	/* yes, constrain the resizing in the direction of the line */
+	if (line->points->x == line->points->next->x)
+	    /* vertical line */
+	    constrained = BOX_VSTRETCH;
+	else
+	    /* horizontal line */
+	    constrained = BOX_HSTRETCH;
+    }
     if (constrained) {
 	dx = cur_x - fix_x;
 	dy = cur_y - fix_y;
@@ -542,7 +560,8 @@ init_compoundpointmoving()
 	cosa = fabs(dx / l);
 	sina = fabs(dy / l);
     }
-    canvas_locmove_proc = constrained_resizing_box;
+    canvas_locmove_proc = constrained_resizing_scale_box;
+    canvas_ref_proc = elastic_fixedbox;
     canvas_leftbut_proc = fix_movedcompoundpoint;
     canvas_rightbut_proc = cancel_compound;
     /* show current length(s) */
@@ -550,8 +569,9 @@ init_compoundpointmoving()
 }
 
 static void
-cancel_compound()
+cancel_compound(void)
 {
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     elastic_box(fix_x, fix_y, cur_x, cur_y);
     /* erase last lengths if appres.showlengths is true */
     erase_lengths();
@@ -561,11 +581,11 @@ cancel_compound()
 }
 
 static void
-fix_movedcompoundpoint(x, y)
-    int		    x, y;
+fix_movedcompoundpoint(int x, int y)
 {
     float	    scalex, scaley;
 
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     elastic_box(fix_x, fix_y, cur_x, cur_y);
     /* erase last lengths if appres.showlengths is true */
     erase_lengths();
@@ -599,7 +619,7 @@ fix_movedcompoundpoint(x, y)
 /***************************  line  ********************************/
 
 static void
-init_linepointmoving()
+init_linepointmoving(void)
 {
     F_point	   *p;
 
@@ -610,18 +630,19 @@ init_linepointmoving()
     from_y = cur_y = moved_point->y;
     set_cursor(crosshair_cursor);
     switch (cur_l->type) {
-    case T_POLYGON:
+      case T_POLYGON:
 	if (left_point == NULL)
 	    for (left_point = right_point, p = left_point->next;
 		 p->next != NULL;
 		 left_point = p, p = p->next);
         force_noanglegeom();
 	canvas_locmove_proc = reshaping_line;
+	canvas_ref_proc = elastic_linelink;
 	break;
 
-    case T_BOX:
-    case T_ARC_BOX:
-    case T_PICTURE:
+      case T_BOX:
+      case T_ARCBOX:
+      case T_PICTURE:
 	if (right_point->next == NULL) {	/* point 4 */
 	    fix_x = cur_l->points->next->x;
 	    fix_y = cur_l->points->next->y;
@@ -640,15 +661,17 @@ init_linepointmoving()
 	}
 
         force_noanglegeom();
-	elastic_box(fix_x, fix_y, cur_x, cur_y);
+	if (cur_l->thickness != 1)
+	    elastic_box(fix_x, fix_y, cur_x, cur_y);
 	canvas_locmove_proc = constrained_resizing_box;
+	canvas_ref_proc = elastic_fixedbox;
 	canvas_leftbut_proc = fix_box;
 	canvas_rightbut_proc = cancel_movept_box;
 	/* show current length(s) */
 	(canvas_locmove_proc)(cur_x, cur_y);
 	return;
 
-    case T_POLYLINE:
+      case T_POLYLINE:
 	if (left_point == NULL || right_point == NULL) {
 	    if (left_point != NULL) {
 		fix_x = left_point->x;
@@ -659,21 +682,24 @@ init_linepointmoving()
 	    }
             if (latexline_mode || latexarrow_mode) {
                 canvas_locmove_proc = latex_line;
+		canvas_ref_proc = elastic_line;
 		cur_latexcursor = crosshair_cursor;
             } else if (mountain_mode || manhattan_mode) {
                 canvas_locmove_proc = constrainedangle_line;
+		canvas_ref_proc = elastic_line;
             } else {
-		/* freehand line */
+		/* unconstrained or freehand line */
 		canvas_locmove_proc = reshaping_line;
+		canvas_ref_proc = elastic_linelink;
 	    }
 	} else {
 	    /* linelink, always freehand */
             force_noanglegeom();
 	    canvas_locmove_proc = reshaping_line;
+	    canvas_ref_proc = elastic_linelink;
 	}
 	break;
     }
-    elastic_linelink();
     canvas_leftbut_proc = fix_movedlinepoint;
     canvas_rightbut_proc = cancel_movedlinepoint;
     /* show current length(s) */
@@ -681,23 +707,27 @@ init_linepointmoving()
 }
 
 static void
-cancel_movept_box()
+cancel_movept_box(void)
 {
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
+    /* erase the elastic box */
     elastic_box(fix_x, fix_y, cur_x, cur_y);
     /* erase last lengths if appres.showlengths is true */
-    erase_lengths();
+    erase_box_lengths();
+    /* redraw original box */
+    redisplay_line(cur_l);
     /* turn back on all relevant markers */
     update_markers(new_objmask);
     wrapup_movepoint();
 }
 
 static void
-fix_box(x, y)
-    int		    x, y;
+fix_box(int x, int y)
 {
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     elastic_box(fix_x, fix_y, cur_x, cur_y);
     /* erase last lengths if appres.showlengths is true */
-    erase_lengths();
+    erase_box_lengths();
     adjust_box_pos(x, y, from_x, from_y, &x, &y);
     new_l = copy_line(cur_l);
     if (new_l->type == T_PICTURE) {
@@ -717,9 +747,7 @@ fix_box(x, y)
     wrapup_movepoint();
 }
 
-assign_newboxpoint(b, x1, y1, x2, y2)
-    F_line	   *b;
-    int		    x1, y1, x2, y2;
+void assign_newboxpoint(F_line *b, int x1, int y1, int x2, int y2)
 {
     F_point	   *p;
 
@@ -751,21 +779,25 @@ assign_newboxpoint(b, x1, y1, x2, y2)
 }
 
 static void
-cancel_movedlinepoint()
+cancel_movedlinepoint(void)
 {
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
+    /* erase the elastic line */
     elastic_linelink();
     /* erase last lengths if appres.showlengths is true */
     erase_lengths();
+    /* redraw original line */
+    redisplay_line(cur_l);
     /* turn back on all relevant markers */
     update_markers(new_objmask);
     wrapup_movepoint();
 }
 
 static void
-fix_movedlinepoint(x, y)
-    int		    x, y;
+fix_movedlinepoint(int x, int y)
 {
     (*canvas_locmove_proc) (x, y);
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     elastic_linelink();
     /* erase last lengths if appres.showlengths is true */
     erase_lengths();
@@ -789,10 +821,7 @@ fix_movedlinepoint(x, y)
 }
 
 static void
-relocate_linepoint(line, x, y, moved_point, left_point)
-    F_line	   *line;
-    int		    x, y;
-    F_point	   *moved_point, *left_point;
+relocate_linepoint(F_line *line, int x, int y, F_point *moved_point, F_point *left_point)
 {
     if (line->type == T_POLYGON)
 	if (line->points == moved_point) {
