@@ -1,17 +1,17 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2000 by Brian V. Smith
+ * Parts Copyright (c) 1989-2007 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
@@ -30,6 +30,12 @@
 #include "w_mousefun.h"
 #include "w_msgpanel.h"
 
+#include "d_text.h"
+#include "u_bound.h"
+#include "u_markers.h"
+#include "u_redraw.h"
+#include "w_cursor.h"
+
 /* EXPORTS  */
 
 int		setcenter;
@@ -42,16 +48,31 @@ float		act_rotnangle;
 
 static int	copy;
 
-static void	init_rotate();
-static void	set_unset_center();
-static void	init_copynrotate();
-static void	rotate_selected();
-static void	rotate_search();
-static void	init_rotateline();
-static void	init_rotatetext();
+static void	init_rotate(F_line *p, int type, int x, int y, int px, int py);
+static void	set_unset_center(int x, int y);
+static void	init_copynrotate(F_line *p, int type, int x, int y, int px, int py);
+static void	rotate_selected(void);
+static void	rotate_search(F_line *p, int type, int x, int y, int px, int py);
+static void	init_rotateline(F_line *l, int px, int py);
+static void	init_rotatetext(F_text *t, int px, int py);
+
+
+void init_rotatearc (F_arc *a, int px, int py);
+void init_rotateellipse (F_ellipse *e, int px, int py);
+void init_rotatespline (F_spline *s, int px, int py);
+void init_rotatecompound (F_compound *c, int px, int py);
+void rotate_line (F_line *l, int x, int y);
+void rotate_text (F_text *t, int x, int y);
+void rotate_ellipse (F_ellipse *e, int x, int y);
+void rotate_arc (F_arc *a, int x, int y);
+void rotate_spline (F_spline *s, int x, int y);
+int valid_rot_angle (F_compound *c);
+void rotate_compound (F_compound *c, int x, int y);
+void rotate_point (F_point *p, int x, int y);
+void rotate_xy (int *orig_x, int *orig_y, int x, int y);
 
 void
-rotate_cw_selected()
+rotate_cw_selected(void)
 {
     rotn_dirn = 1;
     /* erase any existing center */
@@ -66,7 +87,7 @@ rotate_cw_selected()
 }
 
 void
-rotate_ccw_selected()
+rotate_ccw_selected(void)
 {
     rotn_dirn = -1;
     /* erase any existing center */
@@ -81,23 +102,24 @@ rotate_ccw_selected()
 }
 
 static void
-rotate_selected()
+rotate_selected(void)
 {
     set_mousefun("rotate object", "copy & rotate", "set center", 
 			LOC_OBJ, LOC_OBJ, "set center");
     canvas_kbd_proc = null_proc;
     canvas_locmove_proc = null_proc;
+    canvas_ref_proc = null_proc;
     init_searchproc_left(init_rotate);
     init_searchproc_middle(init_copynrotate);
     canvas_leftbut_proc = object_search_left;
     canvas_middlebut_proc = object_search_middle;
     canvas_rightbut_proc = set_unset_center;
     set_cursor(pick15_cursor);
+    reset_action_on();
 }
 
 static void
-set_unset_center(x, y)
-    int		    x, y;
+set_unset_center(int x, int y)
 {
     if (setcenter) {
       set_mousefun("rotate object", "copy & rotate", "set center", 
@@ -120,11 +142,7 @@ set_unset_center(x, y)
 }
 
 static void
-init_rotate(p, type, x, y, px, py)
-    F_line	   *p;
-    int		    type;
-    int		    x, y;
-    int		    px, py;
+init_rotate(F_line *p, int type, int x, int y, int px, int py)
 {
     copy = 0;
     act_rotnangle = cur_rotnangle;
@@ -136,11 +154,7 @@ init_rotate(p, type, x, y, px, py)
 }
 
 static void
-init_copynrotate(p, type, x, y, px, py)
-    F_line	   *p;
-    int		    type;
-    int		    x, y;
-    int		    px, py;
+init_copynrotate(F_line *p, int type, int x, int y, int px, int py)
 {
     int		    i;
 
@@ -156,11 +170,7 @@ init_copynrotate(p, type, x, y, px, py)
 }
 
 static void
-rotate_search(p, type, x, y, px, py)
-    F_line	   *p;
-    int		    type;
-    int		    x, y;
-    int		    px, py;
+rotate_search(F_line *p, int type, int x, int y, int px, int py)
 {
     switch (type) {
     case O_POLYLINE:
@@ -193,9 +203,7 @@ rotate_search(p, type, x, y, px, py)
 }
 
 static void
-init_rotateline(l, px, py)
-    F_line	   *l;
-    int		    px, py;
+init_rotateline(F_line *l, int px, int py)
 {
     F_line	   *line;
 
@@ -217,9 +225,7 @@ init_rotateline(l, px, py)
 }
 
 static void
-init_rotatetext(t, px, py)
-    F_text	   *t;
-    int		    px, py;
+init_rotatetext(F_text *t, int px, int py)
 {
     F_text	   *text;
 
@@ -240,9 +246,7 @@ init_rotatetext(t, px, py)
     reset_cursor();
 }
 
-init_rotateellipse(e, px, py)
-    F_ellipse	   *e;
-    int		    px, py;
+void init_rotateellipse(F_ellipse *e, int px, int py)
 {
     F_ellipse	   *ellipse;
 
@@ -263,9 +267,7 @@ init_rotateellipse(e, px, py)
     reset_cursor();
 }
 
-init_rotatearc(a, px, py)
-    F_arc	   *a;
-    int		    px, py;
+void init_rotatearc(F_arc *a, int px, int py)
 {
     F_arc	   *arc;
 
@@ -286,9 +288,7 @@ init_rotatearc(a, px, py)
     reset_cursor();
 }
 
-init_rotatespline(s, px, py)
-    F_spline	   *s;
-    int		    px, py;
+void init_rotatespline(F_spline *s, int px, int py)
 {
     F_spline	   *spline;
 
@@ -309,9 +309,7 @@ init_rotatespline(s, px, py)
     reset_cursor();
 }
 
-init_rotatecompound(c, px, py)
-    F_compound	   *c;
-    int		    px, py;
+void init_rotatecompound(F_compound *c, int px, int py)
 {
     F_compound	   *compound;
 
@@ -336,15 +334,13 @@ init_rotatecompound(c, px, py)
     reset_cursor();
 }
 
-rotate_line(l, x, y)
-    F_line	   *l;
-    int		    x, y;
+void rotate_line(F_line *l, int x, int y)
 {
     F_point	   *p;
     int		    dx;
 
     /* for speed we treat 90 degrees as a special case */
-    if (act_rotnangle == 90) {
+    if (act_rotnangle == 90.0) {
 	for (p = l->points; p != NULL; p = p->next) {
 	    dx = p->x - x;
 	    p->x = x + rotn_dirn * (y - p->y);
@@ -357,30 +353,26 @@ rotate_line(l, x, y)
 
 }
 
-rotate_figure(f, x, y)
-     F_compound *f;
-     int x, y;
+void rotate_figure(F_compound *f, int x, int y)
 {
   float old_rotn_dirn, old_act_rotnangle;
 
-  old_rotn_dirn= rotn_dirn;
-  old_act_rotnangle= act_rotnangle;
-  rotn_dirn= -1;
-  act_rotnangle=90;
+  old_rotn_dirn = rotn_dirn;
+  old_act_rotnangle = act_rotnangle;
+  rotn_dirn = -1;
+  act_rotnangle = 90.0;
   rotate_compound(f,x,y);
-  rotn_dirn=old_rotn_dirn;
-  act_rotnangle=old_act_rotnangle;
+  rotn_dirn = old_rotn_dirn;
+  act_rotnangle = old_act_rotnangle;
 }
 
-rotate_spline(s, x, y)
-    F_spline	   *s;
-    int		    x, y;
+void rotate_spline(F_spline *s, int x, int y)
 {
     F_point	   *p;
     int		    dx;
 
     /* for speed we treat 90 degrees as a special case */
-    if (act_rotnangle == 90) {
+    if (act_rotnangle == 90.0) {
 	for (p = s->points; p != NULL; p = p->next) {
 	    dx = p->x - x;
 	    p->x = x + rotn_dirn * (y - p->y);
@@ -392,13 +384,11 @@ rotate_spline(s, x, y)
     }
 }
 
-rotate_text(t, x, y)
-    F_text	   *t;
-    int		    x, y;
+void rotate_text(F_text *t, int x, int y)
 {
     int		    dx;
 
-    if (act_rotnangle == 90) { /* treat 90 degs as special case for speed */
+    if (act_rotnangle == 90.0) { /* treat 90 degs as special case for speed */
 	dx = t->base_x - x;
 	t->base_x = x + rotn_dirn * (y - t->base_y);
 	t->base_y = y + rotn_dirn * dx;
@@ -413,13 +403,11 @@ rotate_text(t, x, y)
     reload_text_fstruct(t);
 }
 
-rotate_ellipse(e, x, y)
-    F_ellipse	   *e;
-    int		    x, y;
+void rotate_ellipse(F_ellipse *e, int x, int y)
 {
     int		    dxc,dxs,dxe;
 
-    if (act_rotnangle == 90) { /* treat 90 degs as special case for speed */
+    if (act_rotnangle == 90.0) { /* treat 90 degs as special case for speed */
 	dxc = e->center.x - x;
 	dxs = e->start.x - x;
 	dxe = e->end.x - x;
@@ -434,23 +422,21 @@ rotate_ellipse(e, x, y)
 	rotate_point((F_point *)&e->start, x, y);
 	rotate_point((F_point *)&e->end, x, y);
     }
-    e->angle -= (float) (rotn_dirn * act_rotnangle * M_PI / 180);
+    e->angle -= (float) (rotn_dirn * act_rotnangle * M_PI / 180.0);
     if (e->angle < 0.0)
 	e->angle += M_2PI;
     else if (e->angle >= M_2PI - 0.001)
 	e->angle -= M_2PI;
 }
 
-rotate_arc(a, x, y)
-    F_arc	   *a;
-    int		    x, y;
+void rotate_arc(F_arc *a, int x, int y)
 {
     int		    dx;
     F_pos	    p[3];
     float	    xx, yy;
 
     /* for speed we treat 90 degrees as a special case */
-    if (act_rotnangle == 90) {
+    if (act_rotnangle == 90.0) {
 	dx = a->center.x - x;
 	a->center.x = x + rotn_dirn * (y - a->center.y);
 	a->center.y = y + rotn_dirn * dx;
@@ -486,16 +472,15 @@ rotate_arc(a, x, y)
 
 /* checks to see if the objects within c can be rotated by act_rotnangle */
 
-valid_rot_angle(c)
-    F_compound     *c;
+int valid_rot_angle(F_compound *c)
 {
     F_line         *l;
     F_compound     *c1;
 
-    if (act_rotnangle == 90)
+    if (fabs(act_rotnangle) == 90.0 || fabs(act_rotnangle == 180.0))
 	return 1; /* always valid */
     for (l = c->lines; l != NULL; l = l->next)
-	if (l->type == T_ARC_BOX || l->type == T_BOX)
+	if (l->type == T_ARCBOX || l->type == T_BOX)
 	    return 0;
     for (c1 = c->compounds; c1 != NULL; c1 = c1->next)
 	if (!valid_rot_angle(c1))
@@ -503,9 +488,7 @@ valid_rot_angle(c)
     return 1;
 }
 
-rotate_compound(c, x, y)
-    F_compound	   *c;
-    int		    x, y;
+void rotate_compound(F_compound *c, int x, int y)
 {
     F_line	   *l;
     F_arc	   *a;
@@ -534,9 +517,7 @@ rotate_compound(c, x, y)
 		   &c->secorner.x, &c->secorner.y);
 }
 
-rotate_point(p, x, y)
-    F_point	   *p;
-    int		    x, y;
+void rotate_point(F_point *p, int x, int y)
 {
     /* rotate point p about coordinate (x, y) */
     double	    dx, dy;
@@ -560,8 +541,7 @@ rotate_point(p, x, y)
     p->y = round(y - sina);
 }
 
-rotate_xy(orig_x, orig_y, x, y)
-    int             *orig_x, *orig_y, x, y;
+void rotate_xy(int *orig_x, int *orig_y, int x, int y)
 {
     /* rotate coord (orig_x, orig_y) about coordinate (x, y) */
     double	    dx, dy;
