@@ -1,17 +1,17 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2000 by Brian V. Smith
+ * Parts Copyright (c) 1989-2007 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
@@ -21,6 +21,7 @@
 #include "object.h"
 #include "paintop.h"
 #include "d_text.h"
+#include "e_rotate.h"
 #include "e_scale.h"
 #include "u_create.h"
 #include "u_draw.h"
@@ -28,58 +29,74 @@
 #include "u_search.h"
 #include "u_undo.h"
 #include "w_canvas.h"
+#include "w_drawprim.h"
 #include "w_mousefun.h"
+#include "w_msgpanel.h"
+#include "w_setup.h"
 
-static Boolean	init_boxscale_ellipse();
-static Boolean	init_boxscale_line();
-static Boolean	init_boxscale_compound();
-static Boolean	init_scale_arc();
-static Boolean	init_scale_ellipse();
-static Boolean	init_scale_line();
-static Boolean	init_scale_spline();
+#include "e_movept.h"
+#include "f_util.h"
+#include "u_bound.h"
+#include "u_fonts.h"
+#include "u_geom.h"
+#include "u_list.h"
+#include "u_markers.h"
+#include "u_redraw.h"
+#include "w_cursor.h"
 
-static void	scale_line();
-static void	scale_spline();
-static void	scale_arc();
-static void	scale_ellipse();
-static void	scale_text();
-static void	scale_arrows();
-static void	scale_arrow();
+static Boolean	init_boxscale_ellipse(int x, int y);
+static Boolean	init_boxscale_line(int x, int y);
+static Boolean	init_boxscale_compound(int x, int y);
+static Boolean	init_scale_arc(void);
+static Boolean	init_scale_ellipse(void);
+static Boolean	init_scale_line(void);
+static Boolean	init_scale_spline(void);
 
-static void	init_box_scale();
-static void	boxrelocate_ellipsepoint();
-static void	init_center_scale();
-static void	init_scale_compound();
-static void	rescale_points();
-static void	relocate_ellipsepoint();
-static void	relocate_arcpoint();
+static void	scale_line(F_line *l, float sx, float sy, int refx, int refy);
+static void	scale_spline(F_spline *s, float sx, float sy, int refx, int refy);
+static void	scale_arc(F_arc *a, float sx, float sy, int refx, int refy);
+static void	scale_ellipse(F_ellipse *e, float sx, float sy, int refx, int refy);
+static void	scale_text(F_text *t, float sx, float sy, int refx, int refy);
+static void	scale_arrows(F_line *obj, float sx, float sy);
+static void	scale_arrow(F_arrow *arrow, float sx, float sy);
+
+static void	init_box_scale(F_line *obj, int type, int x, int y, int px, int py);
+static void	boxrelocate_ellipsepoint(F_ellipse *ellipse, int x, int y);
+static void	init_center_scale(F_line *obj, int type, int x, int y, int px, int py);
+static void	init_scale_compound(void);
+static void	rescale_points(F_line *obj, int x, int y);
+static void	relocate_ellipsepoint(F_ellipse *ellipse, int x, int y);
+static void	relocate_arcpoint(F_arc *arc, int x, int y);
+
+static void	fix_scale_arc(int x, int y);
+static void	fix_scale_spline(int x, int y);
+static void	fix_scale_line(int x, int y);
+static void	fix_scale_ellipse(int x, int y);
+static void	fix_boxscale_ellipse(int x, int y);
+static void	fix_boxscale_line(int x, int y);
+static void	fix_scale_compound(int x, int y);
+static void	fix_boxscale_compound(int x, int y);
+
+static void	cancel_scale_arc(void);
+static void	cancel_scale_spline(void);
+static void	cancel_scale_line(void);
+static void	cancel_scale_ellipse(void);
+static void	cancel_boxscale_ellipse(void);
+static void	cancel_boxscale_line(void);
+static void	cancel_scale_compound(void);
+static void	cancel_boxscale_compound(void);
+static void	prescale_compound(F_compound *c, int x, int y);
 
-static void	fix_scale_arc();
-static void	fix_scale_spline();
-static void	fix_scale_line();
-static void	fix_scale_ellipse();
-static void	fix_boxscale_ellipse();
-static void	fix_boxscale_line();
-static void	fix_scale_compound();
-static void	fix_boxscale_compound();
 
-static void	cancel_scale_arc();
-static void	cancel_scale_spline();
-static void	cancel_scale_line();
-static void	cancel_scale_ellipse();
-static void	cancel_boxscale_ellipse();
-static void	cancel_boxscale_line();
-static void	cancel_scale_compound();
-static void	cancel_boxscale_compound();
-static void	prescale_compound();
 
 void
-scale_selected()
+scale_selected(void)
 {
     set_mousefun("scale box", "scale about center", "", 
 			LOC_OBJ, LOC_OBJ, LOC_OBJ);
     canvas_kbd_proc = null_proc;
     canvas_locmove_proc = null_proc;
+    canvas_ref_proc = null_proc;
     init_searchproc_left(init_box_scale);
     init_searchproc_middle(init_center_scale);
     canvas_leftbut_proc = object_search_left;
@@ -93,10 +110,7 @@ char *BOX_SCL_MSG = "Can't use box scale on selected object";
 char *BOX_SCL2_MSG = "Can't use box scale on selected object; try putting it into a compound";
 
 static void
-init_box_scale(obj, type, x, y, px, py)
-    F_line	   *obj;
-    int		    type, x, y;
-    int		    px, py;
+init_box_scale(F_line *obj, int type, int x, int y, int px, int py)
 {
     switch (type) {
     case O_POLYLINE:
@@ -125,9 +139,7 @@ init_box_scale(obj, type, x, y, px, py)
 }
 
 static void
-init_center_scale(obj, type, x, y, px, py)
-    F_line	   *obj;
-    int		    type, x, y, px, py;
+init_center_scale(F_line *obj, int type, int x, int y, int px, int py)
 {
     double	    dx, dy, l;
 
@@ -173,7 +185,7 @@ init_center_scale(obj, type, x, y, px, py)
 }
 
 static void
-wrapup_scale()
+wrapup_scale(void)
 {
     reset_action_on();
     scale_selected();
@@ -183,8 +195,7 @@ wrapup_scale()
 /*************************  ellipse  *******************************/
 
 static Boolean
-init_boxscale_ellipse(x, y)
-    int		    x, y;
+init_boxscale_ellipse(int x, int y)
 {
     double	    dx, dy, l;
 
@@ -228,9 +239,11 @@ init_boxscale_ellipse(x, y)
     if ((cur_e->type == T_CIRCLE_BY_DIA) ||
 	(cur_e->type == T_CIRCLE_BY_RAD)) {
 	canvas_locmove_proc = constrained_resizing_cbd;
+	canvas_ref_proc = elastic_cbd;
 	elastic_cbd();
     } else {
 	canvas_locmove_proc = constrained_resizing_ebd;
+	canvas_ref_proc = elastic_ebd;
 	elastic_ebd();
     }
     canvas_leftbut_proc = fix_boxscale_ellipse;
@@ -239,8 +252,9 @@ init_boxscale_ellipse(x, y)
 }
 
 static void
-cancel_boxscale_ellipse()
+cancel_boxscale_ellipse(void)
 {
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     if ((cur_e->type == T_CIRCLE_BY_DIA) ||
 	(cur_e->type == T_CIRCLE_BY_RAD))
 	elastic_cbd();
@@ -251,8 +265,7 @@ cancel_boxscale_ellipse()
 }
 
 static void
-fix_boxscale_ellipse(x, y)
-    int		    x, y;
+fix_boxscale_ellipse(int x, int y)
 {
     float	    dx, dy;
 
@@ -267,20 +280,16 @@ fix_boxscale_ellipse(x, y)
     /* find how much the radii changed */
     dx = 1.0 * new_e->radiuses.x / cur_e->radiuses.x;
     dy = 1.0 * new_e->radiuses.y / cur_e->radiuses.y;
-    /* scale the line thickness */
-    scale_linewidth(new_e, dx, dy);
     change_ellipse(cur_e, new_e);
+    wrapup_scale();
     /* redraw anything under the old ellipse */
     redisplay_ellipse(cur_e);
     /* and the new one */
     redisplay_ellipse(new_e);
-    wrapup_scale();
 }
 
 static void
-boxrelocate_ellipsepoint(ellipse, x, y)
-    F_ellipse	   *ellipse;
-    int		    x, y;
+boxrelocate_ellipsepoint(F_ellipse *ellipse, int x, int y)
 {
     double	    dx, dy;
 
@@ -323,7 +332,7 @@ boxrelocate_ellipsepoint(ellipse, x, y)
 }
 
 static Boolean
-init_scale_ellipse()
+init_scale_ellipse(void)
 {
     fix_x = cur_e->center.x;
     fix_y = cur_e->center.y;
@@ -336,6 +345,7 @@ init_scale_ellipse()
     toggle_ellipsemarker(cur_e);
     set_cursor(crosshair_cursor);
     canvas_locmove_proc = scaling_ellipse;
+    canvas_ref_proc = elastic_scale_curellipse;
     elastic_scaleellipse(cur_e);
     canvas_middlebut_proc = fix_scale_ellipse;
     canvas_rightbut_proc = cancel_scale_ellipse;
@@ -349,33 +359,31 @@ init_scale_ellipse()
 }
 
 static void
-cancel_scale_ellipse()
+cancel_scale_ellipse(void)
 {
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     elastic_scaleellipse(cur_e);
     toggle_ellipsemarker(cur_e);
     wrapup_scale();
 }
 
 static void
-fix_scale_ellipse(x, y)
-    int		    x, y;
+fix_scale_ellipse(int x, int y)
 {
     elastic_scaleellipse(cur_e);
     adjust_box_pos(x, y, from_x, from_y, &cur_x, &cur_y);
     new_e = copy_ellipse(cur_e);
     relocate_ellipsepoint(new_e, cur_x, cur_y);
     change_ellipse(cur_e, new_e);
+    wrapup_scale();
     /* redraw anything under the old ellipse */
     redisplay_ellipse(cur_e);
     /* and the new one */
     redisplay_ellipse(new_e);
-    wrapup_scale();
 }
 
 static void
-relocate_ellipsepoint(ellipse, x, y)
-    F_ellipse	   *ellipse;
-    int		    x, y;
+relocate_ellipsepoint(F_ellipse *ellipse, int x, int y)
 {
     double	    newx, newy, oldx, oldy;
     double	    newd, oldd, scalefact;
@@ -390,8 +398,6 @@ relocate_ellipsepoint(ellipse, x, y)
     oldy = from_y - fix_y;
     oldd = sqrt(oldx * oldx + oldy * oldy);
     scalefact = newd / oldd;
-    /* scale the line thickness */
-    scale_linewidth(ellipse, scalefact, scalefact);
 
     ellipse->radiuses.x = round(ellipse->radiuses.x * scalefact);
     ellipse->radiuses.y = round(ellipse->radiuses.y * scalefact);
@@ -405,7 +411,7 @@ relocate_ellipsepoint(ellipse, x, y)
 /***************************  arc  *********************************/
 
 static Boolean
-init_scale_arc()
+init_scale_arc(void)
 {
     fix_x = cur_a->center.x;
     fix_y = cur_a->center.y;
@@ -418,14 +424,16 @@ init_scale_arc()
     elastic_scalearc(cur_a);
     set_cursor(crosshair_cursor);
     canvas_locmove_proc = scaling_arc;
+    canvas_ref_proc = elastic_scale_curarc;
     canvas_middlebut_proc = fix_scale_arc;
     canvas_rightbut_proc = cancel_scale_arc;
     return True;
 }
 
 static void
-cancel_scale_arc()
+cancel_scale_arc(void)
 {
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     elastic_scalearc(cur_a);
     /* erase last lengths if appres.showlengths is true */
     erase_lengths();
@@ -434,8 +442,7 @@ cancel_scale_arc()
 }
 
 static void
-fix_scale_arc(x, y)
-    int		    x, y;
+fix_scale_arc(int x, int y)
 {
     double	    newx, newy, oldx, oldy;
     double	    newd, oldd, scalefact;
@@ -453,23 +460,19 @@ fix_scale_arc(x, y)
     oldd = sqrt(oldx * oldx + oldy * oldy);
     scalefact = newd / oldd;
     /* scale any arrowheads */
-    scale_arrows(new_a,scalefact,scalefact);
-    /* scale line width */
-    scale_linewidth(new_a,scalefact,scalefact);
+    scale_arrows((F_line *)new_a,scalefact,scalefact);
 
     relocate_arcpoint(new_a, x, y);
     change_arc(cur_a, new_a);
+    wrapup_scale();
     /* redraw anything under the old arc */
     redisplay_arc(cur_a);
     /* and the new one */
     redisplay_arc(new_a);
-    wrapup_scale();
 }
 
 static void
-relocate_arcpoint(arc, x, y)
-    F_arc	   *arc;
-    int		    x, y;
+relocate_arcpoint(F_arc *arc, int x, int y)
 {
     double	    newx, newy, oldx, oldy;
     double	    newd, oldd, scalefact;
@@ -512,7 +515,7 @@ relocate_arcpoint(arc, x, y)
 /**************************  spline  *******************************/
 
 static Boolean
-init_scale_spline()
+init_scale_spline(void)
 {
     int		    sumx, sumy, cnt;
     F_point	   *p;
@@ -536,24 +539,26 @@ init_scale_spline()
     toggle_splinemarker(cur_s);
     elastic_scalepts(cur_s->points);
     canvas_locmove_proc = scaling_spline;
+    canvas_ref_proc = elastic_scale_curspline;
     canvas_middlebut_proc = fix_scale_spline;
     canvas_rightbut_proc = cancel_scale_spline;
     return True;
 }
 
 static void
-cancel_scale_spline()
+cancel_scale_spline(void)
 {
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     elastic_scalepts(cur_s->points);
     toggle_splinemarker(cur_s);
     wrapup_scale();
 }
 
 static void
-fix_scale_spline(x, y)
-    int		    x, y;
+fix_scale_spline(int x, int y)
 {
     elastic_scalepts(cur_s->points);
+    canvas_ref_proc = null_proc;
     adjust_box_pos(x, y, from_x, from_y, &x, &y);
     /* make a copy of the original and save as unchanged object */
     old_s = copy_spline(cur_s);
@@ -562,19 +567,18 @@ fix_scale_spline(x, y)
     set_action_object(F_EDIT, O_SPLINE);
     old_s->next = cur_s;
     /* now change the original to become the new object */
-    rescale_points(cur_s, x, y);
+    rescale_points((F_line *)cur_s, x, y);
+    wrapup_scale();
     /* redraw anything under the old spline */
     redisplay_spline(old_s);
     /* and the new one */
     redisplay_spline(cur_s);
-    wrapup_scale();
 }
 
 /***************************  compound	********************************/
 
 static Boolean
-init_boxscale_compound(x, y)
-    int		    x, y;
+init_boxscale_compound(int x, int y)
 {
     int		    xmin, ymin, xmax, ymax;
     double	    dx, dy, l;
@@ -651,17 +655,19 @@ init_boxscale_compound(x, y)
 	cosa = fabs(dx / l);
 	sina = fabs(dy / l);
     }
-    boxsize_msg(1);
+    boxsize_scale_msg(1);
     elastic_box(fix_x, fix_y, cur_x, cur_y);
-    canvas_locmove_proc = constrained_resizing_box;
+    canvas_locmove_proc = constrained_resizing_scale_box;
+    canvas_ref_proc = elastic_fixedbox;
     canvas_leftbut_proc = fix_boxscale_compound;
     canvas_rightbut_proc = cancel_boxscale_compound;
     return True;
 }
 
 static void
-cancel_boxscale_compound()
+cancel_boxscale_compound(void)
 {
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     elastic_box(fix_x, fix_y, cur_x, cur_y);
     /* erase last lengths if appres.showlengths is true */
     erase_lengths();
@@ -670,8 +676,7 @@ cancel_boxscale_compound()
 }
 
 static void
-fix_boxscale_compound(x, y)
-    int		    x, y;
+fix_boxscale_compound(int x, int y)
 {
     double	    scalex, scaley;
 
@@ -684,15 +689,15 @@ fix_boxscale_compound(x, y)
     scaley = (double) (y - fix_y) / (from_y - fix_y);
     scale_compound(new_c, scalex, scaley, fix_x, fix_y);
     change_compound(cur_c, new_c);
+    wrapup_scale();
     /* redraw anything under the old compound */
     redisplay_compound(cur_c);
     /* and the new one */
     redisplay_compound(new_c);
-    wrapup_scale();
 }
 
 static void
-init_scale_compound()
+init_scale_compound(void)
 {
     fix_x = (cur_c->nwcorner.x + cur_c->secorner.x) / 2;
     fix_y = (cur_c->nwcorner.y + cur_c->secorner.y) / 2;
@@ -701,13 +706,15 @@ init_scale_compound()
     set_cursor(crosshair_cursor);
     elastic_scalecompound(cur_c);
     canvas_locmove_proc = scaling_compound;
+    canvas_ref_proc = elastic_scale_curcompound;
     canvas_middlebut_proc = fix_scale_compound;
     canvas_rightbut_proc = cancel_scale_compound;
 }
 
 static void
-cancel_scale_compound()
+cancel_scale_compound(void)
 {
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     elastic_scalecompound(cur_c);
     /* erase last lengths if appres.showlengths is true */
     erase_lengths();
@@ -716,8 +723,7 @@ cancel_scale_compound()
 }
 
 static void
-fix_scale_compound(x, y)
-    int		    x, y;
+fix_scale_compound(int x, int y)
 {
     elastic_scalecompound(cur_c);
     /* erase last lengths if appres.showlengths is true */
@@ -731,17 +737,15 @@ fix_scale_compound(x, y)
     old_c->next = cur_c;
     /* now change the original to become the new object */
     prescale_compound(cur_c, cur_x, cur_y);
+    wrapup_scale();
     /* redraw anything under the old compound */
     redisplay_compound(old_c);
     /* and the new one */
     redisplay_compound(cur_c);
-    wrapup_scale();
 }
 
 static void
-prescale_compound(c, x, y)
-    F_compound	   *c;
-    int		    x, y;
+prescale_compound(F_compound *c, int x, int y)
 {
     double	    newx, newy, oldx, oldy;
     double	    newd, oldd, scalefact;
@@ -757,10 +761,7 @@ prescale_compound(c, x, y)
 }
 
 void
-scale_compound(c, sx, sy, refx, refy)
-    F_compound	   *c;
-    double	    sx, sy;
-    int		    refx, refy;
+scale_compound(F_compound *c, double sx, double sy, int refx, int refy)
 {
     F_line	   *l;
     F_spline	   *s;
@@ -770,6 +771,14 @@ scale_compound(c, sx, sy, refx, refy)
     F_compound	   *c1;
     int		    x1, y1, x2, y2;
 
+    /* if sx and sy == 1.0, return now */
+    if (sx == 0.0 && sy == 0.0)
+	return;
+
+    /* check if really a dimension line */
+    if (rescale_dimension_line(c, sx, sy, refx, refy))
+	return; /* yes, return now */
+
     x1 = round(refx + (c->nwcorner.x - refx) * sx);
     y1 = round(refy + (c->nwcorner.y - refy) * sy);
     x2 = round(refx + (c->secorner.x - refx) * sx);
@@ -796,14 +805,213 @@ scale_compound(c, sx, sy, refx, refy)
     }
     for (c1 = c->compounds; c1 != NULL; c1 = c1->next) {
 	scale_compound(c1, sx, sy, refx, refy);
+	/* if there's a dimension line in this compound reset corners */
+	c->nwcorner.x = min2(c->nwcorner.x, c1->nwcorner.x);
+	c->nwcorner.y = min2(c->nwcorner.y, c1->nwcorner.y);
+	c->secorner.x = max2(c->secorner.x, c1->secorner.x);
+	c->secorner.y = max2(c->secorner.y, c1->secorner.y);
     }
 }
 
+Boolean
+rescale_dimension_line(F_compound *dimline, float scalex, float scaley, int refx, int refy)
+{
+    F_line	   *line, *box, *tick1, *tick2;
+    F_text	   *text;
+    double	    length, dx, dy, angle;
+    float	    save_rotnangle, save_rotn_dirn;
+    int		    p1x, p1y, p2x, p2y, centerx, centery;
+    int		    x1, x2, y1, y2, save;
+    int		    theight, tlen2, tht2;
+    PR_SIZE	    tsize;
+    char	    str[80], comment[100];
+    F_point	   *pnt;
+    Boolean	    fixed_text;
+    int		    save_lthick, save_t1thick, save_t2thick;
+
+    /* locate the line components of the dimension line */
+    if (!dimline_components(dimline, &line, &tick1, &tick2, &box))
+	/* not a dimension line, return */
+	return False;
+	
+    /* see if user has deleted main line */
+    if (!line)
+	return False;
+
+    /* get the two endpoints and scale them */
+    p1x = round(refx + (line->points->x - refx) * scalex);
+    p1y = round(refy + (line->points->y - refy) * scaley);
+    p2x = round(refx + (line->points->next->x - refx) * scalex);
+    p2y = round(refy + (line->points->next->y - refy) * scaley);
+
+    /* put them back */
+    line->points->x = p1x;
+    line->points->y = p1y;
+    line->points->next->x = p2x;
+    line->points->next->y = p2y;
+
+    /* if drawn right to left or top to bottom at 90 degrees swap the two points */
+    if (p1x > p2x || (p1y < p2y && p1x == p2x)) {
+	save = p1x; p1x = p2x; p2x = save;
+	save = p1y; p1y = p2y; p2y = save;
+    }
+    dx = p2x-p1x;
+    dy = p2y-p1y;
+
+    /* get the text component */
+    text = dimline->texts;
+    if (!text) {
+	/* hmm, user must have deleted it */
+	text = create_text();
+	if (box)
+	    text->depth  = box->depth-1;
+	else
+	    text->depth  = line->depth-2;
+	add_depth(O_TEXT, text->depth);
+	text->cstring = (char *) NULL;	/* the string will be put in later */
+	text->color = cur_dimline_textcolor;
+	text->font = cur_dimline_font;
+	text->size = cur_dimline_fontsize;
+	text->flags = cur_dimline_psflag? PSFONT_TEXT: 0;
+	text->pen_style = -1;
+	text->type = T_CENTER_JUSTIFIED;
+	dimline->texts = text;
+    } 
+    fixed_text = text && text->comments && (strncasecmp(text->comments,"fixed text", 10)==0);
+    if (!fixed_text) {
+	/* length of the line */
+	length = sqrt(dx*dx + dy*dy);
+
+	/* make text with the length */
+	make_dimension_string(length, str, False);
+
+	/* put the new length in the comment */
+	sprintf(comment, "Dimension line: %s",str);
+	/* free any old comment */
+	if (dimline->comments)
+	    free(dimline->comments);
+	/* put in the new one */
+	dimline->comments = my_strdup(comment);
+    }
+
+    /* center the box for the length string */
+    centerx = (p1x+p2x)/2;
+    centery = (p1y+p2y)/2;
+    /* angle of the line */
+    angle = -atan2(dy, dx);
+
+    /* recompute the text, text angle and text box */
+    /* new string, but only if comment doesn't say "fixed text" */
+    if (!fixed_text) {
+	 /* free any old string */
+	 if (text->cstring)
+	    free(text->cstring);
+	 text->cstring = my_strdup(str);
+    }
+    /* recalculate text sizes */
+    text->fontstruct = lookfont(x_fontnum(text->flags, text->font), text->size);
+    text->zoom = 1.0;
+    tsize = textsize(text->fontstruct, strlen(text->cstring), text->cstring);
+    text->ascent  = tsize.ascent;
+    text->descent = tsize.descent;
+    text->length  = tsize.length;
+    theight = tsize.ascent + tsize.descent;
+    text->angle = angle;
+    text->base_x = centerx + sin(angle)*round(theight/2.0 - tsize.descent);
+    text->base_y = centery + cos(angle)*round(theight/2.0 - tsize.descent);
+
+    /* half the text length + a margin */
+    tlen2 = text->length/2 + 60;
+    /* half the height + a margin */
+    tht2 = theight/2 + 60;
+
+    /* save global rotation angle and direction to use our own */
+    save_rotnangle = act_rotnangle;
+    save_rotn_dirn = rotn_dirn;
+    act_rotnangle = -angle * 180.0 / M_PI;
+    rotn_dirn = 1;
+
+    /* now rescale the text box (the box) */
+    if (box) {
+	pnt = box->points;
+	pnt->x = centerx-tlen2;
+	pnt->y = centery-tht2;
+	pnt = pnt->next;
+	pnt->x = centerx-tlen2;
+	pnt->y = centery+tht2;
+	pnt = pnt->next;
+	pnt->x = centerx+tlen2;
+	pnt->y = centery+tht2;
+	pnt = pnt->next;
+	pnt->x = centerx+tlen2;
+	pnt->y = centery-tht2;
+	pnt = pnt->next;
+	pnt->x = centerx-tlen2;
+	pnt->y = centery-tht2;
+
+	/* rotate the box around the center */
+	rotate_line(box, centerx, centery);
+    } /* if (box) */
+
+    /* now recalculate the end ticks */
+    if (tick1) {
+	pnt = tick1->points;
+	pnt->x = p1x;
+	pnt->y = p1y-tht2;
+	pnt = pnt->next;
+	pnt->x = p1x;
+	pnt->y = p1y+tht2;
+	/* rotate the line around the endpoint */
+	rotate_line(tick1, p1x, p1y);
+    }
+
+    /* the other tick */
+    if (tick2) {
+	pnt = tick2->points;
+	pnt->x = p2x;
+	pnt->y = p2y-tht2;
+	pnt = pnt->next;
+	pnt->x = p2x;
+	pnt->y = p2y+tht2;
+	/* rotate the line around the endpoint */
+	rotate_line(tick2, p2x, p2y);
+    }
+
+    /* restore original rotation angle and direction */
+    act_rotnangle = save_rotnangle;
+    rotn_dirn = save_rotn_dirn;
+
+    /* get the bounds of the compound */
+    /* but set the thicknesses of the line and ticks to 0 so they aren't taken into account */
+    save_lthick = line->thickness;
+    line->thickness = 0;
+    if (tick1) {
+	save_t1thick = tick1->thickness;
+	tick1->thickness = 0;
+    }
+    if (tick2) {
+	save_t2thick = tick2->thickness;
+	tick2->thickness = 0;
+    }
+
+    compound_bound(dimline, &x1, &y1, &x2, &y2);
+    /* restore the thicknesses */
+    line->thickness = save_lthick;
+    if (tick1)
+	tick1->thickness = save_t1thick;
+    if (tick2)
+	tick2->thickness = save_t2thick;
+
+    dimline->nwcorner.x = x1;
+    dimline->nwcorner.y = y1;
+    dimline->secorner.x = x2;
+    dimline->secorner.y = y2;
+    
+    return True;
+}
+
 static void
-scale_line(l, sx, sy, refx, refy)
-    F_line	   *l;
-    float	    sx, sy;
-    int		    refx, refy;
+scale_line(F_line *l, float sx, float sy, int refx, int refy)
 {
     F_point	   *p;
 
@@ -812,7 +1020,7 @@ scale_line(l, sx, sy, refx, refy)
 	p->y = round(refy + (p->y - refy) * sy);
     }
     /* now scale the radius for an arc-box */
-    if (l->type == T_ARC_BOX) {
+    if (l->type == T_ARCBOX) {
 	int h,w;
 	/* scale by the average of height/width factor */
 	l->radius = round(l->radius * (sx+sy)/2);
@@ -828,15 +1036,10 @@ scale_line(l, sx, sy, refx, refy)
     }
     /* finally, scale any arrowheads */
     scale_arrows(l,sx,sy);
-    /* scale line width */
-    scale_linewidth(l,sx,sy);
 }
 
 static void
-scale_spline(s, sx, sy, refx, refy)
-    F_spline	   *s;
-    float	    sx, sy;
-    int		    refx, refy;
+scale_spline(F_spline *s, float sx, float sy, int refx, int refy)
 {
     F_point	   *p;
 
@@ -845,16 +1048,11 @@ scale_spline(s, sx, sy, refx, refy)
 	p->y = round(refy + (p->y - refy) * sy);
     }
     /* scale any arrowheads */
-    scale_arrows(s,sx,sy);
-    /* scale line width */
-    scale_linewidth(s,sx,sy);
+    scale_arrows((F_line *)s,sx,sy);
 }
 
 static void
-scale_arc(a, sx, sy, refx, refy)
-    F_arc	   *a;
-    float	    sx, sy;
-    int		    refx, refy;
+scale_arc(F_arc *a, float sx, float sy, int refx, int refy)
 {
     int		    i;
     F_point	    p[3];
@@ -892,16 +1090,11 @@ scale_arc(a, sx, sy, refx, refy)
     }
     a->direction = compute_direction(a->point[0], a->point[1], a->point[2]);
     /* scale any arrowheads */
-    scale_arrows(a,sx,sy);
-    /* scale line width */
-    scale_linewidth(a,sx,sy);
+    scale_arrows((F_line *)a,sx,sy);
 }
 
 static void
-scale_ellipse(e, sx, sy, refx, refy)
-    F_ellipse	   *e;
-    float	    sx, sy;
-    int		    refx, refy;
+scale_ellipse(F_ellipse *e, float sx, float sy, int refx, int refy)
 {
     e->center.x = round(refx + (e->center.x - refx) * sx);
     e->center.y = round(refy + (e->center.y - refy) * sy);
@@ -921,19 +1114,20 @@ scale_ellipse(e, sx, sy, refx, refy)
 	if (e->radiuses.x == e->radiuses.y)
 	    e->type += 2;
     }
-    /* scale line width */
-    scale_linewidth(e,sx,sy);
 }
 
 static void
-scale_text(t, sx, sy, refx, refy)
-    F_text	   *t;
-    float	    sx, sy;
-    int		    refx, refy;
+scale_text(F_text *t, float sx, float sy, int refx, int refy)
 {
+    int newsize;
     t->base_x = round(refx + (t->base_x - refx) * sx);
     t->base_y = round(refy + (t->base_y - refy) * sy);
     if (!rigid_text(t)) {
+        newsize = round(t->size * sx);
+	if (newsize < MIN_FONT_SIZE) 
+	    sx = (float) MIN_FONT_SIZE / t->size;
+	else if (MAX_FONT_SIZE < newsize)
+	    sx = (float) MAX_FONT_SIZE / t->size;
 	t->size = round(t->size * sx);
 	t->ascent = round(t->ascent * sx);
 	t->descent = round(t->descent * sx);
@@ -947,15 +1141,14 @@ scale_text(t, sx, sy, refx, refy)
 /***************************  line  ********************************/
 
 static Boolean
-init_boxscale_line(x, y)
-    int		    x, y;
+init_boxscale_line(int x, int y)
 {
     int		    xmin, ymin, xmax, ymax;
     F_point	   *p0, *p1, *p2;
     double	    dx, dy, l;
 
     if (cur_l->type != T_BOX &&
-	cur_l->type != T_ARC_BOX &&
+	cur_l->type != T_ARCBOX &&
 	cur_l->type != T_PICTURE) {
 	put_msg(BOX_SCL2_MSG);
 	beep();
@@ -1039,14 +1232,16 @@ init_boxscale_line(x, y)
     boxsize_msg(1);
     elastic_box(fix_x, fix_y, cur_x, cur_y);
     canvas_locmove_proc = constrained_resizing_box;
+    canvas_ref_proc = elastic_fixedbox;
     canvas_leftbut_proc = fix_boxscale_line;
     canvas_rightbut_proc = cancel_boxscale_line;
     return True;
 }
 
 static void
-cancel_boxscale_line()
+cancel_boxscale_line(void)
 {
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     elastic_box(fix_x, fix_y, cur_x, cur_y);
     /* erase last lengths if appres.showlengths is true */
     erase_lengths();
@@ -1055,8 +1250,7 @@ cancel_boxscale_line()
 }
 
 static void
-fix_boxscale_line(x, y)
-    int		    x, y;
+fix_boxscale_line(int x, int y)
 {
     int		owd,oht, nwd, nht;
     float	scalex, scaley;
@@ -1077,25 +1271,21 @@ fix_boxscale_line(x, y)
 	    new_l->pic->flipped = 1 - new_l->pic->flipped;
 	if (signof(fix_y - from_y) != signof(fix_y - y))
 	    new_l->pic->flipped = 1 - new_l->pic->flipped;
-    } else if (new_l->type == T_ARC_BOX) {	/* scale the radius also */
+    } else if (new_l->type == T_ARCBOX) {	/* scale the radius also */
 	scale_radius(cur_l, new_l, owd, oht, nwd, nht);
     }
     change_line(cur_l, new_l);
     scalex = 1.0 * nwd/owd;
     scaley = 1.0 * nht/oht;
-    /* scale the line width */
-    scale_linewidth(new_l, scalex, scaley);
+    wrapup_scale();
     /* redraw anything under the old line */
     redisplay_line(cur_l);
     /* and the new one */
     redisplay_line(new_l);
-    wrapup_scale();
 }
 
 void
-scale_radius(old, new, owd, oht, nwd, nht)
-    F_line	   *old, *new;
-    int owd, oht, nwd, nht;
+scale_radius(F_line *old, F_line *new, int owd, int oht, int nwd, int nht)
 {
 	float wdscale, htscale;
 
@@ -1113,7 +1303,7 @@ scale_radius(old, new, owd, oht, nwd, nht)
 }
 
 static Boolean
-init_scale_line()
+init_scale_line(void)
 {
     int		    sumx, sumy, cnt;
     F_point	   *p;
@@ -1135,18 +1325,21 @@ init_scale_line()
     set_action_on();
     toggle_linemarker(cur_l);
     set_cursor(crosshair_cursor);
-    if (cur_l->type == T_BOX || cur_l->type == T_PICTURE)
+    if (cur_l->type == T_BOX || cur_l->type == T_ARCBOX || cur_l->type == T_PICTURE) {
 	boxsize_msg(2);	/* factor of 2 because measurement is from midpoint */
+    }
     elastic_scalepts(cur_l->points);
     canvas_locmove_proc = scaling_line;
+    canvas_ref_proc = elastic_scale_curline;
     canvas_middlebut_proc = fix_scale_line;
     canvas_rightbut_proc = cancel_scale_line;
     return True;
 }
 
 static void
-cancel_scale_line()
+cancel_scale_line(void)
 {
+    canvas_ref_proc = canvas_locmove_proc = null_proc;
     elastic_scalepts(cur_l->points);
     /* erase last lengths if appres.showlengths is true */
     erase_lengths();
@@ -1155,8 +1348,7 @@ cancel_scale_line()
 }
 
 static void
-fix_scale_line(x, y)
-    int		    x, y;
+fix_scale_line(int x, int y)
 {
     int		owd,oht, nwd, nht;
 
@@ -1172,37 +1364,35 @@ fix_scale_line(x, y)
     old_l->next = cur_l;
     /* now change the original to become the new object */
     rescale_points(cur_l, x, y);
-    /* and scale the radius if ARC_BOX */
-    if (cur_l->type == T_ARC_BOX) {
+    /* and scale the radius if ARCBOX */
+    if (cur_l->type == T_ARCBOX) {
 	owd = abs(old_l->points->x - old_l->points->next->next->x);
 	oht = abs(old_l->points->y - old_l->points->next->next->y);
 	nwd = abs(cur_l->points->x - cur_l->points->next->next->x);
 	nht = abs(cur_l->points->y - cur_l->points->next->next->y);
 	scale_radius(cur_l, cur_l, owd, oht, nwd, nht);
     }
+    wrapup_scale();
     /* redraw anything under the old line */
     redisplay_line(old_l);
     /* and the new one */
     redisplay_line(cur_l);
-    wrapup_scale();
 }
 
 static void
-rescale_points(obj, x, y)
-    F_line	   *obj;
-    int		    x, y;
+rescale_points(F_line *obj, int x, int y)
 {
     F_point	   *p;
-    int		    newx, newy, oldx, oldy;
-    float	    newd, oldd, scalefact;
+    double	    newx, newy, oldx, oldy;
+    double	    newd, oldd, scalefact;
 
     newx = x - fix_x;
     newy = y - fix_y;
-    newd = sqrt((double) (newx * newx + newy * newy));
+    newd = sqrt(newx * newx + newy * newy);
 
     oldx = from_x - fix_x;
     oldy = from_y - fix_y;
-    oldd = sqrt((double) (oldx * oldx + oldy * oldy));
+    oldd = sqrt(oldx * oldx + oldy * oldy);
 
     scalefact = newd / oldd;
     for (p = obj->points; p != NULL; p = p->next) {
@@ -1211,17 +1401,13 @@ rescale_points(obj, x, y)
     }
     /* scale any arrows */
     scale_arrows(obj,scalefact,scalefact);
-    /* scale line width */
-    scale_linewidth(obj,scalefact,scalefact);
     set_modifiedflag();
 }
 
 /* scale arrowheads on passed object by sx,sy */
 
 static void
-scale_arrows(obj,sx,sy)
-    F_line	 *obj;
-    float	  sx,sy;
+scale_arrows(F_line *obj, float sx, float sy)
 {
     scale_arrow(obj->for_arrow,sx,sy);
     scale_arrow(obj->back_arrow,sx,sy);
@@ -1231,32 +1417,14 @@ scale_arrows(obj,sx,sy)
    of sx,sy for scale factor */
 
 static void
-scale_arrow(arrow,sx,sy)
-    F_arrow	 *arrow;
-    float	  sx,sy;
+scale_arrow(F_arrow *arrow, float sx, float sy)
 {
     if (arrow == 0)
 	return;
-    sx = (sx+sy)/2.0;
+
+    /* scale the thickness by the average of the horizontal and vertical scale factors */
+    sx = ((float)fabs(sx)+(float)fabs(sy))/2.0;
     arrow->ht *= sx;
     arrow->wd *= sx;
     return;
 }
-
-/* scale line width - we can call the object a F_line because all
-   Fig object structures have the same components up until the style_val */
-
-scale_linewidth(l,sx,sy)
-    F_line	 *l;
-    float	  sx,sy;
-{
-    if (l->thickness == 0)
-	return;
-
-    /* scale the thickness by the average of the horizontal and vertical scale factors */
-    sx = (sx+sy)/2.0;
-    l->thickness *= sx;
-    /* don't allow underflow */
-    if (l->thickness == 0)
-	l->thickness = 1;
-}
