@@ -1,17 +1,17 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * This software is copyright (C) 1991-1996, Thomas G. Lane.
+ * Parts Copyright (c) 1989-2007 by Brian V. Smith
  * All Rights Reserved except as specified below.
- * Parts Copyright (c) 1989-2000 by Brian V. Smith
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
@@ -59,11 +59,14 @@
 #include "fig.h"
 #include "resources.h"
 #include "object.h"
+#include "f_picobj.h"
+#include "f_util.h"
+#include "w_msgpanel.h"
 #include "w_setup.h"
 #include <setjmp.h>
 #include <jpeglib.h>
 
-static	Boolean	read_JPEG_file();
+static	Boolean	read_JPEG_file(FILE *file);
 
 static	F_pic	   *pict;
 static	unsigned char *bitmapptr;
@@ -73,38 +76,38 @@ static	unsigned char *bitmapptr;
 */
 
 int
-read_jpg(file,filetype,pic)
-    FILE	   *file;
-    int		    filetype;
-    F_pic	   *pic;
+read_jpg(FILE *file, int filetype, F_pic *pic)
 {
 	/* make scale factor smaller for metric */
 	float scale = (appres.INCHES ?
 				(float)PIX_PER_INCH :
 				2.54*PIX_PER_CM)/(float)DISPLAY_PIX_PER_INCH;
 	pict = pic;
-	if (!read_JPEG_file(file))
+	if (!read_JPEG_file(file)) {
+	    close_picfile(file,filetype);
 	    return FileInvalid;
+	}
 
 	/* number of colors is put in by read_JPEG_file() */
-	pic->subtype = T_PIC_JPEG;
 	pic->pixmap = None;
-	pic->size_x = pic->bit_size.x * scale;
-	pic->size_y = pic->bit_size.y * scale;
-	pic->hw_ratio = (float) pic->bit_size.y/pic->bit_size.x;
+	pic->pic_cache->subtype = T_PIC_JPEG;
+	pic->pic_cache->size_x = pic->pic_cache->bit_size.x * scale;
+	pic->pic_cache->size_y = pic->pic_cache->bit_size.y * scale;
+	pic->hw_ratio = (float) pic->pic_cache->bit_size.y/pic->pic_cache->bit_size.x;
 
 	/* we have the image here, see if we need to map it to monochrome display */
 	if (tool_cells <= 2 || appres.monochrome)
 	    map_to_mono(pic);
 
+	close_picfile(file,filetype);
 	return PicSuccess;
 }
 
 /* These static variables are needed by the error routines. */
 
 static	jmp_buf setjmp_buffer;		/* for return to caller */
-static	void	error_exit();
-static	void	error_output();
+static	void	error_exit(j_common_ptr cinfo);
+static	void	error_output(j_common_ptr cinfo);
 
 struct error_mgr {
   struct jpeg_error_mgr pub;	/* "public" fields */
@@ -122,8 +125,7 @@ typedef struct error_mgr * error_ptr;
  */
 
 static Boolean
-read_JPEG_file (file)
-   FILE  *file;
+read_JPEG_file (FILE *file)
 {
 	int i;
 
@@ -173,12 +175,14 @@ read_JPEG_file (file)
 
 	/* Now fill in the pict parameters */
 
-	pict->bit_size.x = cinfo.image_width;
-	pict->bit_size.y = cinfo.image_height;
-	if ((pict->bitmap = (unsigned char *) 
-	     malloc(cinfo.image_width * cinfo.image_height)) == NULL)
-		error_exit("Can't alloc memory for JPEG image");
-	bitmapptr = pict->bitmap;
+	pict->pic_cache->bit_size.x = cinfo.image_width;
+	pict->pic_cache->bit_size.y = cinfo.image_height;
+	if ((pict->pic_cache->bitmap = (unsigned char *) 
+	  malloc(cinfo.image_width * cinfo.image_height)) == NULL) {
+	    file_msg("Can't alloc memory for JPEG image");
+	    longjmp(jerr.setjmp_buffer, 1);
+	}
+	bitmapptr = pict->pic_cache->bitmap;
 
 	/* Step 4: set parameters for decompression */
 
@@ -217,15 +221,15 @@ read_JPEG_file (file)
 	/* Step 7: fill up the colortable in the pict object */
 	/* (Must do this before jpeg_finish_decompress or jpeg_destroy_decompress) */
 
-	pict->numcols = cinfo.actual_number_of_colors;
-	for (i = 0; i < pict->numcols; i++) {
-	    pict->cmap[i].red   = cinfo.colormap[0][i];
+	pict->pic_cache->numcols = cinfo.actual_number_of_colors;
+	for (i = 0; i < pict->pic_cache->numcols; i++) {
+	    pict->pic_cache->cmap[i].red   = cinfo.colormap[0][i];
 	    /* set other colors to first if grayscale */
 	    if (cinfo.jpeg_color_space == JCS_GRAYSCALE) {
-		pict->cmap[i].green = pict->cmap[i].blue = pict->cmap[i].red;
+		pict->pic_cache->cmap[i].green = pict->pic_cache->cmap[i].blue = pict->pic_cache->cmap[i].red;
 	    } else {
-		pict->cmap[i].green = cinfo.colormap[1][i];
-		pict->cmap[i].blue  = cinfo.colormap[2][i];
+		pict->pic_cache->cmap[i].green = cinfo.colormap[1][i];
+		pict->pic_cache->cmap[i].blue  = cinfo.colormap[2][i];
 	    }
 	}
 
@@ -254,8 +258,7 @@ read_JPEG_file (file)
  */
 
 static void
-error_exit (cinfo)
-     j_common_ptr cinfo;
+error_exit (j_common_ptr cinfo)
 {
     char buffer[JMSG_LENGTH_MAX];
     /* cinfo->err really points to a error_mgr struct, so coerce pointer */
@@ -275,8 +278,7 @@ error_exit (cinfo)
 }
 
 static void
-error_output(cinfo)
-     j_common_ptr cinfo;
+error_output(j_common_ptr cinfo)
 {
   char	buffer[JMSG_LENGTH_MAX];
 
