@@ -1,27 +1,32 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2000 by Brian V. Smith
+ * Parts Copyright (c) 1989-2007 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
 #ifndef FIG_H
 #define FIG_H
 
+extern	char	*my_strdup(char *str);
+
 /* For the X stuff, include only Xlib.h and Intrinsic.h here - 
    use figx.h for widget stuff */
 
-#if defined(ultrix) || defined(__bsdi__) || defined(Mips)
+#if defined(ultrix) || defined(__bsdi__) || defined(Mips) || defined(apollo) || defined(__hpux)
+#if defined(__hpux)
+#define _HPUX_SOURCE /* for typedef caddr_t :-(( */
+#endif 
 #include <sys/types.h>	/* for stat structure */
 #endif
 #include <sys/stat.h>
@@ -43,16 +48,19 @@
 #include <ctype.h>
 #include <errno.h>
 
-#if !defined(__bsdi__) && !defined(__NetBSD__) && !defined(__GNU_LIBRARY__)
-extern int	errno;
-extern int	sys_nerr;
-#if ( !(defined(BSD) && (BSD >= 199306)) && !defined(__NetBSD__) && \
-	!defined(__GNU_LIBRARY__) && !defined(__FreeBSD__) && !defined(__GLIBC__))
-extern char    *sys_errlist[];
-#endif
-#endif
+#ifdef NEED_STRERROR
+extern char	*strerror();
+#  if !defined(__bsdi__) && !defined(__NetBSD__) && !defined(__GNU_LIBRARY__)
+      extern int	errno;
+      extern int	sys_nerr;
+#     if ( !(defined(BSD) && (BSD - 0 >= 199306)) && !defined(__NetBSD__) && \
+	   !defined(__GNU_LIBRARY__) && !defined(__FreeBSD__) && !defined(__GLIBC__))
+	 extern char    *sys_errlist[];
+#     endif
+#  endif
+#endif /* NEED_STRERROR */
 
-extern char    *mktemp();
+extern char    *mktemp(char *);
 
 #include <math.h>	/* for sin(), cos() etc */
 
@@ -108,15 +116,14 @@ extern char    *mktemp();
  *
  * Copyright 1991 Massachusetts Institute of Technology
  *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of M.I.T. not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  M.I.T. makes no representations about the
- * suitability of this software for any purpose.  It is provided "as is"
- * without express or implied warranty.
+ * Any party obtaining a copy of these files is granted, free of charge, a
+ * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
+ * nonexclusive right and license to deal in this software and
+ * documentation files (the "Software"), including without limitation the
+ * rights to use, copy, modify, merge, publish and/or distribute copies of
+ * the Software, and to permit persons who receive copies from any such 
+ * party to do so, with the only requirement being that this copyright 
+ * notice remain intact.
  *
  * M.I.T. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL M.I.T.
@@ -295,22 +302,12 @@ extern char *getenv();
 #endif /* X_NOT_STDC_ENV */
 #endif /* defined(SYSV) && defined(SYSV386) */
 
-/* 
-#if defined(SYSV) || defined(SVR4) || defined(__osf__) || defined(USE_DIRENT)
-#define u_int uint
-#define USE_DIRENT
-#define DIRSTRUCT	struct dirent
-#else
-#define DIRSTRUCT	struct direct
-#endif
-*/
 #ifdef HAVE_NO_DIRENT
 #define DIRSTRUCT struct direct
 #else
 #define DIRSTRUCT struct dirent
 #endif
 
-/* define PATH_MAX if not already defined */
 /* taken from the X11R5 server/os/osfonts.c file */
 #ifndef X_NOT_POSIX
 #ifdef _POSIX_SOURCE
@@ -324,6 +321,7 @@ extern char *getenv();
 #endif /* _POSIX_SOURCE */
 #endif /* X_NOT_POSIX */
 
+/* define PATH_MAX if not already defined */
 #ifndef PATH_MAX
 #include <sys/param.h>
 #ifdef MAXPATHLEN
@@ -354,7 +352,6 @@ extern char *getenv();
 #define INLINE
 #endif /* USE_INLINE */
 
-#endif /* FIG_H */
 
 #ifdef NOSTRSTR
 extern char *strstr();
@@ -377,11 +374,17 @@ extern	double		drand48();
 extern	long		random();
 extern	void		srandom(unsigned int);
 
-#elif !defined(__osf__) && !defined(__CYGWIN__)
-extern	void		srandom();
+#elif !defined(__osf__) && !defined(__CYGWIN__) && !defined(linux) && !defined(__FreeBSD__) && !defined(__GLIBC__)
+extern	void		srandom(int);
 
 #endif
 
 #ifndef frandom
 #define	frandom()	(random()*(1./2147483648.))
 #endif
+
+#ifdef I18N
+#include <locale.h>
+#endif  /* I18N */
+
+#endif /* FIG_H */
