@@ -1,17 +1,17 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2000 by Brian V. Smith
+ * Parts Copyright (c) 1989-2007 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
@@ -20,6 +20,10 @@
 
 /* values to signify color used for transparent GIF color */
 
+#define		CANVAS_BG		-7	/* use canvas background color */
+#define		DARK_GRAY		-6	/* pseudo color to Greek small text */
+#define		MED_GRAY		-5	/* pseudo colors to gray out inactive layers */
+#define		LT_GRAY			-4
 #define		TRANSP_BACKGROUND	-3	/* use background of figure as transp color */
 #define		TRANSP_NONE		-2	/* no transp color */
 #define		COLOR_NONE		-2	/* no background color (exporting) */
@@ -48,6 +52,17 @@
 #define		WHITE			7
 #define		GREEN4			12
 
+/* defaults for line attributes */
+
+#define		DEF_BOXRADIUS		7
+#define		DEF_DASHLENGTH		4
+#define		DEF_DOTGAP		3
+
+/* arrowhead defaults */
+
+#define		DEF_ARROW_WID 4.0
+#define		DEF_ARROW_HT  8.0
+
 /** VERY IMPORTANT:  The f_line, f_spline and f_arc objects all must have the
 		components up to and including the arrows in the same order.
 		This is for the get/put_generic_arrows() in e_edit.c.
@@ -55,19 +70,69 @@
 		components up to and including the f_points in the same order.
 **/
 
+/***********************************************************************/
+/* NOTE: If you change this you must change _pic_names in e_edit.c too */
+/***********************************************************************/
+
+/* These values are used internally, so changing them doesn't
+   affect any Fig files */
+
+enum pictypes {
+	T_PIC_NONE,
+	T_PIC_EPS,
+	T_PIC_GIF,
+#ifdef USE_JPEG
+	T_PIC_JPEG,
+#endif /* USE_JPEG */
+	T_PIC_PCX,
+	T_PIC_PNG,
+	T_PIC_PPM,
+	T_PIC_TIF,
+	T_PIC_XBM,
+#ifdef USE_XPM
+	T_PIC_XPM,
+#endif /* USE_XPM */
+	LAST_PIC
+     } ;
+
+#define NUM_PIC_TYPES LAST_PIC-1
+
+/* structure to contain a point */
+typedef struct f_pos {
+    int		    x, y;
+}
+	F_pos;
+
+struct _pics {
+		char	     *file;
+		char	     *realname;		/* in case the actual file is compressed (.gz, etc) */
+		time_t	      time_stamp;	/* to see if the file has changed */
+		char	     *bitmap;
+	        enum pictypes subtype;
+	        int	      size_x, size_y;	/* picture size (fig units) */
+	        F_pos	      bit_size;		/* size of bitmap in pixels */
+	        struct Cmap   cmap[MAX_COLORMAP_SIZE];  /* for GIF/XPM/JPEG files */
+	        int	      numcols;		/* number of colors in cmap */
+	        int	      transp;		/* transparent color (TRANSP_NONE if none) for GIFs */
+		int	      refcount;		/* number of references to picture */
+		struct _pics *prev;
+		struct _pics *next;
+	     };
+
+/*******************/
+/* point structure */
+/*******************/
+
 typedef struct f_point {
     int		    x, y;
     struct f_point *next;
 }
-		F_point;
+	F_point;
 
-typedef struct f_pos {
-    int		    x, y;
-}
-		F_pos;
 
-#define DEF_ARROW_WID 4
-#define DEF_ARROW_HT 8
+/***********************/
+/* Arrowhead structure */
+/***********************/
 
 typedef struct f_arrow {
     int		    type;
@@ -76,9 +141,11 @@ typedef struct f_arrow {
     float	    wd;
     float	    ht;
 }
-		F_arrow;
-
+	F_arrow;
+
+/******************/
 /* Ellipse object */
+/******************/
 
 typedef struct f_ellipse {
     int		    tagged;
@@ -96,7 +163,7 @@ typedef struct f_ellipse {
     int		    depth;
     int		    pen_style;
     float	    style_val;
-    float	    angle;
+    float	    angle;		/* in radians */
     int		    direction;
 #define					UNFILLED	-1
     struct f_pos    center;
@@ -106,20 +173,23 @@ typedef struct f_ellipse {
     char	   *comments;
     struct f_ellipse *next;
 }
-		F_ellipse;
-
+	F_ellipse;
+
 /* SEE NOTE AT TOP BEFORE CHANGING ANYTHING IN THE f_arc STRUCTURE */
 
+/**************/
 /* Arc object */
+/**************/
 
 typedef struct f_arc {
     int		    tagged;
     int		    distrib;
     int		    type;
 				/* note: these arc types are the internal values */
-				/* in the file, they are open=1, wedge=2 */
+				/* in the file, they are open=1, wedge=2, elliptical=3 */
 #define					T_OPEN_ARC		0
 #define					T_PIE_WEDGE_ARC		1
+#define					T_ELLIPTICAL		2
     int		    style;
     int		    thickness;
     Color	    pen_color;
@@ -137,6 +207,7 @@ typedef struct f_arc {
 
 /* THE PRECEDING VARS MUST BE IN THE SAME ORDER IN f_arc, f_line and f_spline */
 
+    float           angle;		/* for elliptical arcs, new for V4.0 */
     int		    direction;
     struct {
 	float		x, y;
@@ -145,77 +216,38 @@ typedef struct f_arc {
     char	   *comments;
     struct f_arc   *next;
 }
-		F_arc;
-
-#define		CLOSED_PATH		0
-#define		OPEN_PATH		1
-#define		DEF_BOXRADIUS		7
-#define		DEF_DASHLENGTH		4
-#define		DEF_DOTGAP		3
-
-/***********************************************************************/
-/* NOTE: If you change this you must change _pic_names in e_edit.c too */
-/***********************************************************************/
-
-/* These values are used internally, so changing them doesn't
-   affect any Fig files */
-
-enum pictypes {
-	T_PIC_NONE,
-#ifdef V4_0
-	T_PIC_FIG,
-#endif
-	T_PIC_EPS,
-	T_PIC_GIF,
-#ifdef USE_JPEG
-	T_PIC_JPEG,
-#endif
-	T_PIC_PCX,
-	T_PIC_PPM,
-	T_PIC_TIF,
-	T_PIC_XBM,
-#ifdef USE_XPM
-	T_PIC_XPM,
-#endif
-	LAST_PIC
-     } ;
-
-#define NUM_PIC_TYPES LAST_PIC-1
+	F_arc;
 
+
 #define PicSuccess	1
 #define FileInvalid    -2
-
-/* Picture sub-type */
+/************************************/
+/* Picture sub-type for Line object */
+/************************************/
 
 typedef struct f_pic {
-    char	    file[PATH_MAX];
-    enum pictypes   subtype;
+    struct _pics   *pic_cache;		/* picture repository (bitmap, refcount etc) */
+    Boolean	    new;		/* set when creating object, cleared when
+					   Done is clicked in edit operation */
     int		    flipped;
-    unsigned char  *bitmap;
-    Pixmap	    mask;
-    struct Cmap	    cmap[MAX_COLORMAP_SIZE];  /* for GIF/XPM/JPEG files */
-    int		    numcols;		/* number of colors in cmap */
-    int		    transp;		/* transparent color (-1 if none) for GIFs */
     float	    hw_ratio;
-    int		    size_x, size_y;	/* picture size (fig units) */
-    struct f_pos    bit_size;		/* size of bitmap in pixels */
+    Pixmap	    mask;		/* for GIF transparency */
     Color	    color;		/* only used for XBM */
     Pixmap	    pixmap;		/* pixmap created for canvas */
     int		    pix_rotation,
 		    pix_width,		/* current width of pixmap (pixels) */
 		    pix_height,		/* current height of pixmap (pixels) */
 		    pix_flipped;
-#ifdef V4_0
-    struct f_compound *figure;		/* Fig compound if picture type == T_PIC_FIG */
-#endif
 }
-		F_pic;
+	F_pic;
 
 extern char	EMPTY_PIC[];
 
 /* SEE NOTE AT TOP BEFORE CHANGING ANYTHING IN THE f_line STRUCTURE */
 
+/***************/
 /* Line object */
+/***************/
 
 typedef struct f_line {
     int		    tagged;
@@ -224,7 +256,7 @@ typedef struct f_line {
 #define					T_POLYLINE	1
 #define					T_BOX		2
 #define					T_POLYGON	3
-#define					T_ARC_BOX	4
+#define					T_ARCBOX	4
 #define					T_PICTURE	5
     int		    style;
     int		    thickness;
@@ -248,14 +280,16 @@ typedef struct f_line {
 #define					JOIN_MITER	0
 #define					JOIN_ROUND	1
 #define					JOIN_BEVEL	2
-    int		    radius;	/* corner radius for T_ARC_BOX */
-    struct f_pic   *pic;
+    int		    radius;	/* corner radius for T_ARCBOX */
+    F_pic   	   *pic;	/* picture object, if type = T_PICTURE */
     char	   *comments;
     struct f_line  *next;
 }
-		F_line;
-
+	F_line;
+
+/***************/
 /* Text object */
+/***************/
 
 typedef struct f_text {
     int		    tagged;
@@ -288,7 +322,9 @@ typedef struct f_text {
     char	   *comments;
     struct f_text  *next;
 }
-		F_text;
+	F_text;
+
+/* text macros */
 
 #define MAXFONT(T) (psfont_text(T) ? NUM_FONTS : NUM_LATEX_FONTS)
 
@@ -312,15 +348,13 @@ typedef struct f_text {
 			(hidden_text(t) ? hidden_text_length : t->length)
 
 #define		using_ps	(cur_textflags & PSFONT_TEXT)
-
-typedef struct f_shape {
-    double s;
-    struct f_shape *next;
-}
-		F_sfactor;
-
+
 /* SEE NOTE AT TOP BEFORE CHANGING ANYTHING IN THE f_spline STRUCTURE */
 
+/*****************/
+/* Spline object */
+/*****************/
+
 #define		int_spline(s)		(s->type & 0x2)
 #define         x_spline(s)	        (s->type & 0x4)
 #define		approx_spline(s)	(!(int_spline(s)|x_spline(s)))
@@ -366,35 +400,50 @@ typedef struct f_spline {
     char	   *comments;
     struct f_spline *next;
 }
-		F_spline;
+	F_spline;
+
+/**************************************/
+/* Shape factor structure for splines */
+/**************************************/
+
+typedef struct f_shape {
+    double s;
+    struct f_shape *next;
+}
+	F_sfactor;
+
+/*******************/
+/* Compound object */
+/*******************/
 
 typedef struct f_compound {
-    int		    tagged;
-    int		    distrib;
-    struct f_pos    nwcorner;
-    struct f_pos    secorner;
-    struct f_line  *lines;
-    struct f_ellipse *ellipses;
-    struct f_spline *splines;
-    struct f_text  *texts;
-    struct f_arc   *arcs;
-    char	   *comments;
-    struct f_compound *parent;	/* for "enter/leave compound" */
+    int		       tagged;
+    int		       distrib;
+    struct f_pos       nwcorner;
+    struct f_pos       secorner;
+    struct f_line     *lines;
+    struct f_ellipse  *ellipses;
+    struct f_spline   *splines;
+    struct f_text     *texts;
+    struct f_arc      *arcs;
+    char	      *comments;
+    struct f_compound *parent;	/* for "open/close compound" */
     struct f_compound *GABPtr;	/* Where original compound came from */
+    Boolean	       draw_parent;
     struct f_compound *compounds;
     struct f_compound *next;
 }
-		F_compound;
+	F_compound;
 
 typedef struct f_linkinfo {
-    struct f_line  *line;
-    struct f_point *endpt;
-    struct f_point *prevpt;
-    int		    two_pts;
+    struct f_line     *line;
+    struct f_point    *endpt;
+    struct f_point    *prevpt;
+    int		       two_pts;
     struct f_linkinfo *next;
 }
-		F_linkinfo;
-
+	F_linkinfo;
+
 /* separate the "type" and the "style" from the cur_arrowtype */
 #define		ARROW_TYPE(x)	((x)==0? 0 : ((x)+1)/2)
 #define		ARROW_STYLE(x)	((x)==0? 0 : ((x)+1)%2)
@@ -458,6 +507,11 @@ typedef struct f_linkinfo {
 #define M_NO_TEXT	(M_ELLIPSE | M_POLYLINE | M_SPLINE | M_COMPOUND | M_ARC)
 #define M_ALL		(M_OBJECT | M_COMPOUND)
 
+#define M_ANGLEMEAS_OBJECT  (M_POLYLINE_LINE | M_POLYLINE_POLYGON | M_ARC)
+#define M_LENMEAS_OBJECT    (M_POLYLINE_LINE | M_POLYLINE_POLYGON | M_POLYLINE_BOX | M_ARC | M_ELLIPSE)
+#define M_AREAMEAS_OBJECT   (M_POLYLINE_POLYGON | M_POLYLINE_BOX | M_ARC | M_ELLIPSE)
+#define M_TANGENT_OBJECT    (M_VARPTS_OBJECT | M_POLYLINE_BOX | M_ARC | M_ELLIPSE)
+
 /************************  Objects  **********************/
 
 extern F_compound objects;
