@@ -1,17 +1,17 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-1998 by Brian V. Smith
+ * Parts Copyright (c) 1989-2007 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
@@ -23,8 +23,14 @@
 #include "u_elastic.h"
 #include "u_geom.h"
 #include "w_canvas.h"
+#include "w_drawprim.h"
 #include "w_setup.h"
 #include "w_zoom.h"
+#include "d_arc.h"
+
+#include "u_draw.h"
+#include "w_cursor.h"
+#include "w_msgpanel.h"
 
 /********************** EXPORTS **************/
 
@@ -40,17 +46,18 @@ F_point	       *left_point, *right_point;
 
 /**************** LOCAL ***********/
 
-static void	elastic_links();
-static void	angle0_line();
-static void	angle45_line();
-static void	angle90_line();
-static void	angle135_line();
+static void	elastic_links(int dx, int dy, float sx, float sy);
+static void	angle0_line(int x, int y);
+static void	angle45_line(int x, int y);
+static void	angle90_line(int x, int y);
+static void	angle135_line(int x, int y);
 
 /*************************** BOXES *************************/
 
+
+
 void
-elastic_box(x1, y1, x2, y2)
-    int		    x1, y1, x2, y2;
+elastic_box(int x1, int y1, int x2, int y2)
 {
     int		    wid, ht;
 
@@ -62,7 +69,13 @@ elastic_box(x1, y1, x2, y2)
 }
 
 void
-elastic_movebox()
+elastic_fixedbox(void)
+{
+    elastic_box(fix_x, fix_y, cur_x, cur_y);
+}
+
+void
+elastic_movebox(void)
 {
     register int    x1, y1, x2, y2;
 
@@ -75,8 +88,7 @@ elastic_movebox()
 }
 
 void
-moving_box(x, y)
-    int		    x, y;
+moving_box(int x, int y)
 {
     elastic_movebox();
     adjust_pos(x, y, fix_x, fix_y, &cur_x, &cur_y);
@@ -85,29 +97,35 @@ moving_box(x, y)
 }
 
 void
-resizing_box(x, y)
-    int		    x, y;
+resizing_box(int x, int y)
 {
-    elastic_box(fix_x, fix_y, cur_x, cur_y);
+    elastic_fixedbox();
     cur_x = x;
     cur_y = y;
-    elastic_box(fix_x, fix_y, cur_x, cur_y);
     boxsize_msg(1);
+    elastic_fixedbox();
 }
 
 void
-constrained_resizing_box(x, y)
-    int		    x, y;
+constrained_resizing_box(int x, int y)
 {
-    elastic_box(fix_x, fix_y, cur_x, cur_y);
+    elastic_fixedbox();
     adjust_box_pos(x, y, from_x, from_y, &cur_x, &cur_y);
-    elastic_box(fix_x, fix_y, cur_x, cur_y);
     boxsize_msg(1);
+    elastic_fixedbox();
 }
 
 void
-scaling_compound(x, y)
-    int		    x, y;
+constrained_resizing_scale_box(int x, int y)
+{
+    elastic_fixedbox();
+    adjust_box_pos(x, y, from_x, from_y, &cur_x, &cur_y);
+    elastic_fixedbox();
+    boxsize_scale_msg(1);
+}
+
+void
+scaling_compound(int x, int y)
 {
     elastic_scalecompound(cur_c);
     adjust_box_pos(x, y, fix_x, fix_y, &cur_x, &cur_y);
@@ -115,8 +133,13 @@ scaling_compound(x, y)
 }
 
 void
-elastic_scalecompound(c)
-    F_compound	   *c;
+elastic_scale_curcompound(void)
+{
+    elastic_scalecompound(cur_c);
+}
+
+void
+elastic_scalecompound(F_compound *c)
 {
     double	    newx, newy, oldx, oldy;
     double	    newd, oldd, scalefact;
@@ -133,14 +156,16 @@ elastic_scalecompound(c)
     y1 = fix_y + round((c->secorner.y - fix_y) * scalefact);
     x2 = fix_x + round((c->nwcorner.x - fix_x) * scalefact);
     y2 = fix_y + round((c->nwcorner.y - fix_y) * scalefact);
+    boxsize_scale_msg(2);
     elastic_box(x1, y1, x2, y2);
-    boxsize_msg(2);
 }
 
 /*************************** LINES *************************/
 
+/* refresh a line segment */
+
 void
-elastic_line()
+elastic_line(void)
 {
     pw_vector(canvas_win, fix_x, fix_y, cur_x, cur_y,
 	      INV_PAINT, 1, RUBBER_LINE, 0.0, DEFAULT);
@@ -153,8 +178,7 @@ elastic_line()
    the canvas_locmove_proc for drawing arcs */
 
 void
-unconstrained_line(x, y, numpoint)
-    int		    x, y, numpoint;
+unconstrained_line(int x, int y)
 {
     elastic_line();
     cur_x = x;
@@ -164,14 +188,13 @@ unconstrained_line(x, y, numpoint)
 }
 
 void
-latex_line(x, y)
-    int		    x, y;
+latex_line(int x, int y)
 {
     Cursor c;
 
     elastic_line();
     latex_endpoint(fix_x, fix_y, x, y, &cur_x, &cur_y, latexarrow_mode,
-		   (cur_pointposn == P_ANY) ? 1 : posn_rnd[cur_pointposn]);
+		   (cur_pointposn == P_ANY) ? 1 : posn_rnd[cur_gridunit][cur_pointposn]);
     length_msg(MSG_LENGTH);
     elastic_line();
     c = (x == cur_x && y == cur_y) ? null_cursor : crosshair_cursor;
@@ -182,8 +205,7 @@ latex_line(x, y)
 }
 
 void
-constrainedangle_line(x, y)
-    int		    x, y;
+constrainedangle_line(int x, int y)
 {
     double	    angle, dx, dy;
 
@@ -231,25 +253,22 @@ constrainedangle_line(x, y)
     elastic_line();
 }
 
-void
-angle0_line(x, y)
-    int		    x, y;
+static void
+angle0_line(int x, int y)
 {
     cur_x = x;
     cur_y = fix_y;
 }
 
-void
-angle90_line(x, y)
-    int		    x, y;
+static void
+angle90_line(int x, int y)
 {
     cur_y = y;
     cur_x = fix_x;
 }
 
-void
-angle45_line(x, y)
-    int		    x, y;
+static void
+angle45_line(int x, int y)
 {
     if (abs(x - fix_x) < abs(y - fix_y)) {
 	cur_x = fix_x - y + fix_y;
@@ -260,9 +279,8 @@ angle45_line(x, y)
     }
 }
 
-void
-angle135_line(x, y)
-    int		    x, y;
+static void
+angle135_line(int x, int y)
 {
     if (abs(x - fix_x) < abs(y - fix_y)) {
 	cur_x = fix_x + y - fix_y;
@@ -274,8 +292,7 @@ angle135_line(x, y)
 }
 
 void
-reshaping_line(x, y)
-    int		    x, y;
+reshaping_line(int x, int y)
 {
     elastic_linelink();
     adjust_pos(x, y, from_x, from_y, &cur_x, &cur_y);
@@ -292,7 +309,7 @@ reshaping_line(x, y)
 }
 
 void
-elastic_linelink()
+elastic_linelink(void)
 {
     if (left_point != NULL) {
 	pw_vector(canvas_win, left_point->x, left_point->y,
@@ -305,8 +322,7 @@ elastic_linelink()
 }
 
 void
-moving_line(x, y)
-    int		    x, y;
+moving_line(int x, int y)
 {
     elastic_moveline(new_l->points);
     adjust_pos(x, y, fix_x, fix_y, &cur_x, &cur_y);
@@ -315,8 +331,13 @@ moving_line(x, y)
 }
 
 void
-elastic_moveline(pts)
-    F_point	   *pts;
+elastic_movenewline(void)
+{
+    elastic_moveline(new_l->points);
+}
+
+void
+elastic_moveline(F_point *pts)
 {
     F_point	   *p;
     int		    dx, dy, x, y, xx, yy;
@@ -345,9 +366,7 @@ elastic_moveline(pts)
 }
 
 static void
-elastic_links(dx, dy, sx, sy)
-    int		    dx, dy;
-    float	    sx, sy;
+elastic_links(int dx, int dy, float sx, float sy)
 {
     F_linkinfo	   *k;
 
@@ -387,19 +406,23 @@ elastic_links(dx, dy, sx, sy)
 }
 
 void
-scaling_line(x, y)
-    int		    x, y;
+scaling_line(int x, int y)
 {
     elastic_scalepts(cur_l->points);
     adjust_box_pos(x, y, fix_x, fix_y, &cur_x, &cur_y);
-    elastic_scalepts(cur_l->points);
-    if (cur_l->type == T_BOX || cur_l->type == T_PICTURE)
+    if (cur_l->type == T_BOX || cur_l->type == T_ARCBOX || cur_l->type == T_PICTURE)
 	boxsize_msg(2);
+    elastic_scalepts(cur_l->points);
 }
 
 void
-scaling_spline(x, y)
-    int		    x, y;
+elastic_scale_curline(int x, int y)
+{
+    elastic_scalepts(cur_l->points);
+}
+
+void
+scaling_spline(int x, int y)
 {
     elastic_scalepts(cur_s->points);
     adjust_box_pos(x, y, fix_x, fix_y, &cur_x, &cur_y);
@@ -407,8 +430,13 @@ scaling_spline(x, y)
 }
 
 void
-elastic_scalepts(pts)
-    F_point	   *pts;
+elastic_scale_curspline(void)
+{
+    elastic_scalepts(cur_s->points);
+}
+
+void
+elastic_scalepts(F_point *pts)
 {
     F_point	   *p;
     double	    newx, newy, oldx, oldy;
@@ -436,8 +464,7 @@ elastic_scalepts(pts)
 }
 
 void
-elastic_poly(x1, y1, x2, y2, numsides)
-    int		    x1, y1, x2, y2, numsides;
+elastic_poly(int x1, int y1, int x2, int y2, int numsides)
 {
     register float  angle;
     register int    nx, ny, i; 
@@ -469,8 +496,7 @@ elastic_poly(x1, y1, x2, y2, numsides)
 }
 
 void
-resizing_poly(x, y)
-    int		    x, y;
+resizing_poly(int x, int y)
 {
     elastic_poly(fix_x, fix_y, cur_x, cur_y, work_numsides);
     cur_x = x;
@@ -483,7 +509,7 @@ resizing_poly(x, y)
 /*********************** ELLIPSES *************************/
 
 void
-elastic_ebr()
+elastic_ebr(void)
 {
     register int    x1, y1, x2, y2;
     int		    rx, ry;
@@ -491,21 +517,20 @@ elastic_ebr()
     rx = cur_x - fix_x;
     ry = cur_y - fix_y;
     if (cur_angle != 0.0) {
-	angle_ellipse(fix_x, fix_y, rx, ry, cur_angle, INV_PAINT, 1, 
+	angle_ellipse(fix_x, fix_y, rx, ry, cur_angle, INV_PAINT, MAX_DEPTH+1, 1, 
 	     RUBBER_LINE, 0.0, UNFILLED, DEFAULT, DEFAULT);
     } else {
 	x1 = fix_x + rx;
 	x2 = fix_x - rx;
 	y1 = fix_y + ry;
 	y2 = fix_y - ry;
-	pw_curve(canvas_win, x1, y1, x2, y2, INV_PAINT, 1,
+	pw_curve(canvas_win, x1, y1, x2, y2, INV_PAINT, MAX_DEPTH+1, 1,
 	     RUBBER_LINE, 0.0, UNFILLED, DEFAULT, DEFAULT, CAP_BUTT);
     }
 }
 
 void
-resizing_ebr(x, y)
-    int		    x, y;
+resizing_ebr(int x, int y)
 {
     elastic_ebr();
     cur_x = x;
@@ -515,8 +540,7 @@ resizing_ebr(x, y)
 }
 
 void
-constrained_resizing_ebr(x, y)
-    int		    x, y;
+constrained_resizing_ebr(int x, int y)
 {
     elastic_ebr();
     adjust_box_pos(x, y, from_x, from_y, &cur_x, &cur_y);
@@ -525,7 +549,7 @@ constrained_resizing_ebr(x, y)
 }
 
 void
-elastic_ebd()
+elastic_ebd(void)
 {
     int	    centx,centy;
 
@@ -535,16 +559,16 @@ elastic_ebd()
     if (cur_angle != 0.0) {
 	angle_ellipse(centx, centy, abs(cur_x-fix_x)/2, 
 		  abs(cur_y-fix_y)/2, cur_angle,
-		  INV_PAINT, 1, RUBBER_LINE, 0.0, UNFILLED, DEFAULT, DEFAULT);
+		  INV_PAINT, MAX_DEPTH+1, 1, RUBBER_LINE, 0.0, UNFILLED,
+		  DEFAULT, DEFAULT);
     } else {
-	pw_curve(canvas_win, fix_x, fix_y, cur_x, cur_y,
-	     INV_PAINT, 1, RUBBER_LINE, 0.0, UNFILLED, DEFAULT, DEFAULT, CAP_BUTT);
+	pw_curve(canvas_win, fix_x, fix_y, cur_x, cur_y, INV_PAINT, MAX_DEPTH+1, 1,
+			RUBBER_LINE, 0.0, UNFILLED, DEFAULT, DEFAULT, CAP_BUTT);
     }
 }
 
 void
-resizing_ebd(x, y)
-    int		    x, y;
+resizing_ebd(int x, int y)
 {
     elastic_ebd();
     cur_x = x;
@@ -554,8 +578,7 @@ resizing_ebd(x, y)
 }
 
 void
-constrained_resizing_ebd(x, y)
-    int		    x, y;
+constrained_resizing_ebd(int x, int y)
 {
     elastic_ebd();
     adjust_box_pos(x, y, from_x, from_y, &cur_x, &cur_y);
@@ -564,7 +587,7 @@ constrained_resizing_ebd(x, y)
 }
 
 void
-elastic_cbr()
+elastic_cbr(void)
 {
     register int    radius, x1, y1, x2, y2;
     double	    rx, ry;
@@ -576,13 +599,12 @@ elastic_cbr()
     x2 = fix_x - radius;
     y1 = fix_y + radius;
     y2 = fix_y - radius;
-    pw_curve(canvas_win, x1, y1, x2, y2, INV_PAINT, 1,
+    pw_curve(canvas_win, x1, y1, x2, y2, INV_PAINT, MAX_DEPTH+1, 1,
 	     RUBBER_LINE, 0.0, UNFILLED, DEFAULT, DEFAULT, CAP_BUTT);
 }
 
 void
-resizing_cbr(x, y)
-    int		    x, y;
+resizing_cbr(int x, int y)
 {
     elastic_cbr();
     cur_x = x;
@@ -592,7 +614,7 @@ resizing_cbr(x, y)
 }
 
 void
-elastic_cbd()
+elastic_cbd(void)
 {
     register int    x1, y1, x2, y2;
     int		    radius;
@@ -605,13 +627,12 @@ elastic_cbd()
     x2 = fix_x + rx - radius;
     y1 = fix_y + ry + radius;
     y2 = fix_y + ry - radius;
-    pw_curve(canvas_win, x1, y1, x2, y2, INV_PAINT, 1,
+    pw_curve(canvas_win, x1, y1, x2, y2, INV_PAINT, MAX_DEPTH+1, 1,
 	     RUBBER_LINE, 0.0, UNFILLED, DEFAULT, DEFAULT, CAP_BUTT);
 }
 
 void
-resizing_cbd(x, y)
-    int		    x, y;
+resizing_cbd(int x, int y)
 {
     elastic_cbd();
     cur_x = x;
@@ -621,8 +642,7 @@ resizing_cbd(x, y)
 }
 
 void
-constrained_resizing_cbd(x, y)
-    int		    x, y;
+constrained_resizing_cbd(int x, int y)
 {
     elastic_cbd();
     adjust_box_pos(x, y, from_x, from_y, &cur_x, &cur_y);
@@ -631,7 +651,7 @@ constrained_resizing_cbd(x, y)
 }
 
 void
-elastic_moveellipse()
+elastic_moveellipse(void)
 {
     register int    x1, y1, x2, y2;
 
@@ -641,16 +661,16 @@ elastic_moveellipse()
     y2 = cur_y + y2off;
     if (cur_angle != 0.0) {
 	angle_ellipse((x1+x2)/2, (y1+y2)/2, abs(x1-x2)/2, abs(y1-y2)/2, cur_angle,
-		  INV_PAINT, 1, RUBBER_LINE, 0.0, UNFILLED, DEFAULT, DEFAULT);
+		  INV_PAINT, MAX_DEPTH+1, 1, RUBBER_LINE, 0.0, UNFILLED,
+		  DEFAULT, DEFAULT);
     } else {
-	pw_curve(canvas_win, x1, y1, x2, y2, INV_PAINT, 1,
+	pw_curve(canvas_win, x1, y1, x2, y2, INV_PAINT, MAX_DEPTH+1, 1,
 	     RUBBER_LINE, 0.0, UNFILLED, DEFAULT, DEFAULT, CAP_BUTT);
     }
 }
 
 void
-moving_ellipse(x, y)
-    int		    x, y;
+moving_ellipse(int x, int y)
 {
     elastic_moveellipse();
     adjust_pos(x, y, fix_x, fix_y, &cur_x, &cur_y);
@@ -659,8 +679,7 @@ moving_ellipse(x, y)
 }
 
 void
-elastic_scaleellipse(e)
-    F_ellipse	   *e;
+elastic_scaleellipse(F_ellipse *e)
 {
     register int    x1, y1, x2, y2;
     int		    rx, ry;
@@ -681,20 +700,20 @@ elastic_scaleellipse(e)
     ry = round(e->radiuses.y * scalefact);
     if (cur_angle != 0.0) {
 	angle_ellipse(e->center.x, e->center.y, rx, ry, cur_angle,
-		  INV_PAINT, 1, RUBBER_LINE, 0.0, UNFILLED, DEFAULT, DEFAULT);
+		  INV_PAINT, MAX_DEPTH+1, 1, RUBBER_LINE, 0.0, UNFILLED,
+		  DEFAULT, DEFAULT);
     } else {
 	x1 = fix_x + rx;
 	x2 = fix_x - rx;
 	y1 = fix_y + ry;
 	y2 = fix_y - ry;
-	pw_curve(canvas_win, x1, y1, x2, y2, INV_PAINT, 1,
+	pw_curve(canvas_win, x1, y1, x2, y2, INV_PAINT, MAX_DEPTH+1, 1,
 	     RUBBER_LINE, 0.0, UNFILLED, DEFAULT, DEFAULT, CAP_BUTT);
     }
 }
 
 void
-scaling_ellipse(x, y)
-    int		    x, y;
+scaling_ellipse(int x, int y)
 {
     elastic_scaleellipse(cur_e);
     adjust_box_pos(x, y, fix_x, fix_y, &cur_x, &cur_y);
@@ -707,20 +726,35 @@ scaling_ellipse(x, y)
     elastic_scaleellipse(cur_e);
 }
 
+void
+elastic_scale_curellipse(void)
+{
+    elastic_scaleellipse(cur_e);
+}
+
 /*************************** ARCS *************************/
 
 void
-reshaping_arc(x, y)
-    int		    x, y;
+arc_point(int x, int y, int numpoint)
+{
+    elastic_line();
+    cur_x = x;
+    cur_y = y;
+    altlength_msg(MSG_RADIUS_ANGLE, center_point.x, center_point.y);
+    elastic_line();
+}
+
+void
+reshaping_arc(int x, int y)
 {
     elastic_arclink();
     adjust_pos(x, y, cur_a->point[movedpoint_num].x,
 	       cur_a->point[movedpoint_num].y, &cur_x, &cur_y);
     if (movedpoint_num == 1) {
 	/* middle point */
-	length_msg2(cur_a->point[0].x, cur_a->point[0].y,
-		    cur_a->point[2].x, cur_a->point[2].y,
-		    cur_x, cur_y);
+        arc_msg(cur_a->point[0].x, cur_a->point[0].y,
+		 cur_a->point[2].x, cur_a->point[2].y,
+		 cur_x, cur_y);
     } else {
 	/* end point */
 	altlength_msg(MSG_LENGTH,cur_a->point[1].x,cur_a->point[1].y);
@@ -729,7 +763,7 @@ reshaping_arc(x, y)
 }
 
 void
-elastic_arclink()
+elastic_arclink(void)
 {
     switch (movedpoint_num) {
     case 0:
@@ -750,8 +784,7 @@ elastic_arclink()
 }
 
 void
-moving_arc(x, y)
-    int		    x, y;
+moving_arc(int x, int y)
 {
     elastic_movearc(new_a);
     adjust_pos(x, y, fix_x, fix_y, &cur_x, &cur_y);
@@ -760,8 +793,13 @@ moving_arc(x, y)
 }
 
 void
-elastic_movearc(a)
-    F_arc	   *a;
+elastic_movenewarc(void)
+{
+    elastic_movearc(new_a);
+}
+
+void
+elastic_movearc(F_arc *a)
 {
     int		    dx, dy;
 
@@ -776,8 +814,7 @@ elastic_movearc(a)
 }
 
 void
-scaling_arc(x, y)
-    int		    x, y;
+scaling_arc(int x, int y)
 {
     elastic_scalearc(cur_a);
     adjust_box_pos(x, y, fix_x, fix_y, &cur_x, &cur_y);
@@ -785,8 +822,13 @@ scaling_arc(x, y)
 }
 
 void
-elastic_scalearc(a)
-    F_arc	   *a;
+elastic_scale_curarc(void)
+{ 
+    elastic_scalearc(cur_a);
+}
+
+void
+elastic_scalearc(F_arc *a)
 {
     double	    newx, newy, oldx, oldy;
     double	    newd, oldd, scalefact;
@@ -822,8 +864,7 @@ elastic_scalearc(a)
 /*************************** TEXT *************************/
 
 void
-moving_text(x, y)
-    int		    x, y;
+moving_text(int x, int y)
 {
     elastic_movetext();
     adjust_pos(x, y, fix_x, fix_y, &cur_x, &cur_y);
@@ -835,19 +876,18 @@ moving_text(x, y)
    shifted under the cursor */
 
 void
-elastic_movetext()
+elastic_movetext(void)
 {
-    pw_text(canvas_win, cur_x + x1off, cur_y + y1off, INV_PAINT,
+    pw_text(canvas_win, cur_x + x1off, cur_y + y1off, INV_PAINT, MAX_DEPTH+1,
 	    new_t->fontstruct, new_t->angle, 
-	    new_t->cstring, new_t->color);
+	    new_t->cstring, new_t->color, COLOR_NONE);
 }
 
 
 /*************************** SPLINES *************************/
 
 void
-moving_spline(x, y)
-    int		    x, y;
+moving_spline(int x, int y)
 {
     elastic_moveline(new_s->points);
     adjust_pos(x, y, fix_x, fix_y, &cur_x, &cur_y);
@@ -855,12 +895,16 @@ moving_spline(x, y)
     elastic_moveline(new_s->points);
 }
 
+void
+elastic_movenewspline(void)
+{
+    elastic_moveline(new_s->points);
+}
+
 /*********** AUXILIARY FUNCTIONS FOR CONSTRAINED MOVES ******************/
 
 void
-adjust_box_pos(curs_x, curs_y, orig_x, orig_y, ret_x, ret_y)
-    int		    curs_x, curs_y, orig_x, orig_y;
-    int		   *ret_x, *ret_y;
+adjust_box_pos(int curs_x, int curs_y, int orig_x, int orig_y, int *ret_x, int *ret_y)
 {
     int		    xx, sgn_csr2fix_x, yy, sgn_csr2fix_y;
     double	    mag_csr2fix_x, mag_csr2fix_y;
@@ -915,9 +959,7 @@ adjust_box_pos(curs_x, curs_y, orig_x, orig_y, ret_x, ret_y)
 }
 
 void
-adjust_pos(curs_x, curs_y, orig_x, orig_y, ret_x, ret_y)
-    int		    curs_x, curs_y, orig_x, orig_y;
-    int		   *ret_x, *ret_y;
+adjust_pos(int curs_x, int curs_y, int orig_x, int orig_y, int *ret_x, int *ret_y)
 {
     if (constrained) {
 	if (abs(orig_x - curs_x) > abs(orig_y - curs_y)) {
