@@ -1,18 +1,18 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2000 by Brian V. Smith
+ * Parts Copyright (c) 1989-2007 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
  * Parts Copyright (c) 1995 by C. Blanc and C. Schlick
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
@@ -21,6 +21,7 @@
 #include "mode.h"
 #include "object.h"
 #include "paintop.h"
+#include "f_read.h"
 #include "u_create.h"
 #include "u_list.h"
 #include "u_elastic.h"
@@ -29,13 +30,19 @@
 #include "w_layers.h"
 #include "w_setup.h"
 
+#include "u_draw.h"
+#include "u_markers.h"
+
 /*************************************/
 /****** DELETE object from list ******/
 /*************************************/
 
+
+int point_on_perim (F_point *p, int llx, int lly, int urx, int ury);
+int point_on_inside (F_point *p, int llx, int lly, int urx, int ury);
+
 void
-list_delete_arc(arc_list, arc)
-    F_arc	  **arc_list, *arc;
+list_delete_arc(F_arc **arc_list, F_arc *arc)
 {
     F_arc	   *a, *aa;
 
@@ -59,8 +66,7 @@ list_delete_arc(arc_list, arc)
 }
 
 void
-list_delete_ellipse(ellipse_list, ellipse)
-    F_ellipse	  **ellipse_list, *ellipse;
+list_delete_ellipse(F_ellipse **ellipse_list, F_ellipse *ellipse)
 {
     F_ellipse	   *q, *r;
 
@@ -84,8 +90,7 @@ list_delete_ellipse(ellipse_list, ellipse)
 }
 
 void
-list_delete_line(line_list, line)
-    F_line	   *line, **line_list;
+list_delete_line(F_line **line_list, F_line *line)
 {
     F_line	   *q, *r;
 
@@ -109,8 +114,7 @@ list_delete_line(line_list, line)
 }
 
 void
-list_delete_spline(spline_list, spline)
-    F_spline	  **spline_list, *spline;
+list_delete_spline(F_spline **spline_list, F_spline *spline)
 {
     F_spline	   *q, *r;
 
@@ -134,8 +138,7 @@ list_delete_spline(spline_list, spline)
 }
 
 void
-list_delete_text(text_list, text)
-    F_text	  **text_list, *text;
+list_delete_text(F_text **text_list, F_text *text)
 {
     F_text	   *q, *r;
 
@@ -158,8 +161,7 @@ list_delete_text(text_list, text)
 }
 
 void
-list_delete_compound(list, compound)
-    F_compound	  **list, *compound;
+list_delete_compound(F_compound **list, F_compound *compound)
 {
     F_compound	   *c, *cc;
 
@@ -184,9 +186,7 @@ list_delete_compound(list, compound)
 }
 
 void
-remove_depth(type, depth)
-    int		    type;
-    int		    depth;
+remove_depth(int type, int depth)
 {
     int		    i;
 
@@ -250,8 +250,7 @@ remove_depth(type, depth)
 }
 
 void
-remove_compound_depth(comp)
-    F_compound	   *comp;
+remove_compound_depth(F_compound *comp)
 {
     F_arc	   *aa;
     F_ellipse	   *ee;
@@ -260,6 +259,11 @@ remove_compound_depth(comp)
     F_text	   *tt;
     F_compound	   *cc;
 
+    if (comp == (F_compound *) 0)
+	return;
+
+    /* defer updating of layer buttons until we've added the entire compound */
+    defer_update_layers++;
     for (aa=comp->arcs; aa; aa=aa->next)
 	remove_depth(O_ARC, aa->depth);
     for (ee=comp->ellipses; ee; ee=ee->next)
@@ -272,6 +276,9 @@ remove_compound_depth(comp)
 	remove_depth(O_TEXT, tt->depth);
     for (cc=comp->compounds; cc; cc=cc->next)
 	remove_compound_depth(cc);
+    /* decrement the defer flag and update layer buttons if it hits 0 */
+    defer_update_layers--;
+    update_layers();
 }
 
 
@@ -280,8 +287,7 @@ remove_compound_depth(comp)
 /********************************/
 
 void
-list_add_arc(list, a)
-    F_arc	  **list, *a;
+list_add_arc(F_arc **list, F_arc *a)
 {
     F_arc	   *aa;
 
@@ -290,7 +296,7 @@ list_add_arc(list, a)
 	*list = a;
     else
 	aa->next = a;
-    if (list == &objects.arcs || list == &saved_objects.arcs)
+    if (list == &objects.arcs)
 	while (a) {
 	    add_depth(O_ARC, a->depth);
 	    a = a->next;
@@ -298,8 +304,7 @@ list_add_arc(list, a)
 }
 
 void
-list_add_ellipse(list, e)
-    F_ellipse	  **list, *e;
+list_add_ellipse(F_ellipse **list, F_ellipse *e)
 {
     F_ellipse	   *ee;
 
@@ -308,7 +313,7 @@ list_add_ellipse(list, e)
 	*list = e;
     else
 	ee->next = e;
-    if (list == &objects.ellipses || list == &saved_objects.ellipses)
+    if (list == &objects.ellipses)
 	while (e) {
 	    add_depth(O_ELLIPSE, e->depth);
 	    e = e->next;
@@ -316,8 +321,7 @@ list_add_ellipse(list, e)
 }
 
 void
-list_add_line(list, l)
-    F_line	  **list, *l;
+list_add_line(F_line **list, F_line *l)
 {
     F_line	   *ll;
 
@@ -326,7 +330,7 @@ list_add_line(list, l)
 	*list = l;
     else
 	ll->next = l;
-    if (list == &objects.lines || list == &saved_objects.lines)
+    if (list == &objects.lines)
 	while (l) {
 	    add_depth(O_POLYLINE, l->depth);
 	    l = l->next;
@@ -334,8 +338,7 @@ list_add_line(list, l)
 }
 
 void
-list_add_spline(list, s)
-    F_spline	  **list, *s;
+list_add_spline(F_spline **list, F_spline *s)
 {
     F_spline	   *ss;
 
@@ -344,7 +347,7 @@ list_add_spline(list, s)
 	*list = s;
     else
 	ss->next = s;
-    if (list == &objects.splines || list == &saved_objects.splines)
+    if (list == &objects.splines)
 	while (s) {
 	    add_depth(O_SPLINE, s->depth);
 	    s = s->next;
@@ -352,8 +355,7 @@ list_add_spline(list, s)
 }
 
 void
-list_add_text(list, t)
-    F_text	  **list, *t;
+list_add_text(F_text **list, F_text *t)
 {
     F_text	   *tt;
 
@@ -362,7 +364,7 @@ list_add_text(list, t)
 	*list = t;
     else
 	tt->next = t;
-    if (list == &objects.texts || list == &saved_objects.texts)
+    if (list == &objects.texts)
 	while (t) {
 	    add_depth(O_TEXT, t->depth);
 	    t = t->next;
@@ -370,8 +372,7 @@ list_add_text(list, t)
 }
 
 void
-list_add_compound(list, c)
-    F_compound	  **list, *c;
+list_add_compound(F_compound **list, F_compound *c)
 {
     F_compound	   *cc;
 
@@ -381,20 +382,19 @@ list_add_compound(list, c)
     else
 	cc->next = c;
     
-    if (list == &objects.compounds || list == &saved_objects.compounds)
+    if (list == &objects.compounds) {
 	while (c) {
 	    add_compound_depth(c);
 	    c = c->next;
 	}
+    }
 }
 
 /* increment objects_depth[depth] for a new object, and add to the 
    counter for this "type" (line, arc, etc) */
 
 void
-add_depth(type, depth)
-    int		    type;
-    int		    depth;
+add_depth(int type, int depth)
 {
     int		    i;
 
@@ -452,8 +452,7 @@ add_depth(type, depth)
 }
 
 void
-add_compound_depth(comp)
-    F_compound	   *comp;
+add_compound_depth(F_compound *comp)
 {
     F_arc	   *aa;
     F_ellipse	   *ee;
@@ -462,6 +461,8 @@ add_compound_depth(comp)
     F_text	   *tt;
     F_compound	   *cc;
 
+    /* defer updating of layer buttons until we've added the entire compound */
+    defer_update_layers++;
     for (aa=comp->arcs; aa; aa=aa->next)
 	add_depth(O_ARC, aa->depth);
     for (ee=comp->ellipses; ee; ee=ee->next)
@@ -474,6 +475,9 @@ add_compound_depth(comp)
 	add_depth(O_TEXT, tt->depth);
     for (cc=comp->compounds; cc; cc=cc->next)
 	add_compound_depth(cc);
+    /* decrement the defer flag and update layer buttons if it hits 0 */
+    defer_update_layers--;
+    update_layers();
 }
 
 /**********************************/
@@ -481,8 +485,7 @@ add_compound_depth(comp)
 /**********************************/
 
 void
-delete_line(old_l)
-    F_line	   *old_l;
+delete_line(F_line *old_l)
 {
     list_delete_line(&objects.lines, old_l);
     clean_up();
@@ -492,8 +495,7 @@ delete_line(old_l)
 }
 
 void
-delete_arc(old_a)
-    F_arc	   *old_a;
+delete_arc(F_arc *old_a)
 {
     list_delete_arc(&objects.arcs, old_a);
     clean_up();
@@ -503,8 +505,7 @@ delete_arc(old_a)
 }
 
 void
-delete_ellipse(old_e)
-    F_ellipse	   *old_e;
+delete_ellipse(F_ellipse *old_e)
 {
     list_delete_ellipse(&objects.ellipses, old_e);
     clean_up();
@@ -514,8 +515,7 @@ delete_ellipse(old_e)
 }
 
 void
-delete_text(old_t)
-    F_text	   *old_t;
+delete_text(F_text *old_t)
 {
     list_delete_text(&objects.texts, old_t);
     clean_up();
@@ -525,8 +525,7 @@ delete_text(old_t)
 }
 
 void
-delete_spline(old_s)
-    F_spline	   *old_s;
+delete_spline(F_spline *old_s)
 {
     list_delete_spline(&objects.splines, old_s);
     clean_up();
@@ -536,8 +535,7 @@ delete_spline(old_s)
 }
 
 void
-delete_compound(old_c)
-    F_compound	   *old_c;
+delete_compound(F_compound *old_c)
 {
     list_delete_compound(&objects.compounds, old_c);
     clean_up();
@@ -551,8 +549,7 @@ delete_compound(old_c)
 /*******************************/
 
 void
-add_line(new_l)
-    F_line	   *new_l;
+add_line(F_line *new_l)
 {
     list_add_line(&objects.lines, new_l);
     clean_up();
@@ -562,8 +559,7 @@ add_line(new_l)
 }
 
 void
-add_arc(new_a)
-    F_arc	   *new_a;
+add_arc(F_arc *new_a)
 {
     list_add_arc(&objects.arcs, new_a);
     clean_up();
@@ -573,8 +569,7 @@ add_arc(new_a)
 }
 
 void
-add_ellipse(new_e)
-    F_ellipse	   *new_e;
+add_ellipse(F_ellipse *new_e)
 {
     list_add_ellipse(&objects.ellipses, new_e);
     clean_up();
@@ -584,8 +579,7 @@ add_ellipse(new_e)
 }
 
 void
-add_text(new_t)
-    F_text	   *new_t;
+add_text(F_text *new_t)
 {
     list_add_text(&objects.texts, new_t);
     clean_up();
@@ -595,8 +589,7 @@ add_text(new_t)
 }
 
 void
-add_spline(new_s)
-    F_spline	   *new_s;
+add_spline(F_spline *new_s)
 {
     list_add_spline(&objects.splines, new_s);
     clean_up();
@@ -606,8 +599,7 @@ add_spline(new_s)
 }
 
 void
-add_compound(new_c)
-    F_compound	   *new_c;
+add_compound(F_compound *new_c)
 {
     list_add_compound(&objects.compounds, new_c);
     clean_up();
@@ -618,8 +610,7 @@ add_compound(new_c)
 
 
 void
-change_line(old_l, new_l)
-    F_line	   *old_l, *new_l;
+change_line(F_line *old_l, F_line *new_l)
 {
     list_delete_line(&objects.lines, old_l);
     list_add_line(&objects.lines, new_l);
@@ -631,8 +622,7 @@ change_line(old_l, new_l)
 }
 
 void
-change_arc(old_a, new_a)
-    F_arc	   *old_a, *new_a;
+change_arc(F_arc *old_a, F_arc *new_a)
 {
     list_delete_arc(&objects.arcs, old_a);
     list_add_arc(&objects.arcs, new_a);
@@ -644,8 +634,7 @@ change_arc(old_a, new_a)
 }
 
 void
-change_ellipse(old_e, new_e)
-    F_ellipse	   *old_e, *new_e;
+change_ellipse(F_ellipse *old_e, F_ellipse *new_e)
 {
     list_delete_ellipse(&objects.ellipses, old_e);
     list_add_ellipse(&objects.ellipses, new_e);
@@ -657,8 +646,7 @@ change_ellipse(old_e, new_e)
 }
 
 void
-change_text(old_t, new_t)
-    F_text	   *old_t, *new_t;
+change_text(F_text *old_t, F_text *new_t)
 {
     list_delete_text(&objects.texts, old_t);
     list_add_text(&objects.texts, new_t);
@@ -670,8 +658,7 @@ change_text(old_t, new_t)
 }
 
 void
-change_spline(old_s, new_s)
-    F_spline	   *old_s, *new_s;
+change_spline(F_spline *old_s, F_spline *new_s)
 {
     list_delete_spline(&objects.splines, old_s);
     list_add_spline(&objects.splines, new_s);
@@ -683,8 +670,7 @@ change_spline(old_s, new_s)
 }
 
 void
-change_compound(old_c, new_c)
-    F_compound	   *old_c, *new_c;
+change_compound(F_compound *old_c, F_compound *new_c)
 {
     list_delete_compound(&objects.compounds, old_c);
     list_add_compound(&objects.compounds, new_c);
@@ -697,8 +683,7 @@ change_compound(old_c, new_c)
 
 /* find the tails of all the object lists */
 
-tail(ob, tails)
-    F_compound	   *ob, *tails;
+void tail(F_compound *ob, F_compound *tails)
 {
     F_arc	   *a;
     F_compound	   *c;
@@ -708,17 +693,23 @@ tail(ob, tails)
     F_text	   *t;
 
     if (NULL != (a = ob->arcs))
-	for (; a->next != NULL; a = a->next);
+	for (; a->next != NULL; a = a->next)
+		;
     if (NULL != (c = ob->compounds))
-	for (; c->next != NULL; c = c->next);
+	for (; c->next != NULL; c = c->next)
+		;
     if (NULL != (e = ob->ellipses))
-	for (; e->next != NULL; e = e->next);
+	for (; e->next != NULL; e = e->next)
+		;
     if (NULL != (l = ob->lines))
-	for (; l->next != NULL; l = l->next);
+	for (; l->next != NULL; l = l->next)
+		;
     if (NULL != (s = ob->splines))
-	for (; s->next != NULL; s = s->next);
+	for (; s->next != NULL; s = s->next)
+		;
     if (NULL != (t = ob->texts))
-	for (; t->next != NULL; t = t->next);
+	for (; t->next != NULL; t = t->next)
+		;
 
     tails->arcs = a;
     tails->compounds = c;
@@ -734,9 +725,11 @@ tail(ob, tails)
  * defined prior to calling append.
  */
 
-append_objects(l1, l2, tails)
-    F_compound	   *l1, *l2, *tails;
+void append_objects(F_compound *l1, F_compound *l2, F_compound *tails)
 {
+    /* don't forget to account for the depths */
+    add_compound_depth(l2);
+
     if (tails->arcs)
 	tails->arcs->next = l2->arcs;
     else
@@ -765,38 +758,85 @@ append_objects(l1, l2, tails)
 
 /* Cut is the dual of append. */
 
-cut_objects(objects, tails)
-    F_compound	   *objects, *tails;
+void cut_objects(F_compound *objects, F_compound *tails)
 {
-    if (tails->arcs)
+    if (tails->arcs) {
+	remove_arc_depths(tails->arcs->next);
 	tails->arcs->next = NULL;
-    else
+    } else if (objects->arcs) {
+	remove_arc_depths(objects->arcs);
 	objects->arcs = NULL;
-    if (tails->compounds)
+    }
+    if (tails->compounds) {
+	remove_compound_depth(tails->compounds->next);
 	tails->compounds->next = NULL;
-    else
+    } else if (objects->compounds) {
+	remove_compound_depth(objects->compounds);
 	objects->compounds = NULL;
-    if (tails->ellipses)
+    }
+    if (tails->ellipses) {
+	remove_ellipse_depths(tails->ellipses->next);
 	tails->ellipses->next = NULL;
-    else
+    } else if (objects->ellipses) {
+	remove_ellipse_depths(objects->ellipses);
 	objects->ellipses = NULL;
-    if (tails->lines)
+    }
+    if (tails->lines) {
+	remove_line_depths(tails->lines->next);
 	tails->lines->next = NULL;
-    else
+    } else if (objects->lines) {
+	remove_line_depths(objects->lines);
 	objects->lines = NULL;
-    if (tails->splines)
+    }
+    if (tails->splines) {
+	remove_spline_depths(tails->splines->next);
 	tails->splines->next = NULL;
-    else
+    } else if (objects->splines) {
+	remove_spline_depths(objects->splines);
 	objects->splines = NULL;
-    if (tails->texts)
+    }
+    if (tails->texts) {
+	remove_text_depths(tails->texts->next);
 	tails->texts->next = NULL;
-    else
+    } else if (objects->texts) {
+	remove_text_depths(objects->texts);
 	objects->texts = NULL;
+    }
 }
 
-append_point(x, y, point)    /** used in d_arcbox **/
-    int		    x, y;
-    F_point	  **point;
+void remove_arc_depths(F_arc *a)
+{
+    for ( ; a; a= a->next)
+	remove_depth(O_ARC, a->depth);
+}
+
+void remove_ellipse_depths(F_ellipse *e)
+{
+    for ( ; e; e = e->next)
+	remove_depth(O_ELLIPSE, e->depth);
+}
+
+void remove_line_depths(F_line *l)
+{
+    for ( ; l; l = l->next)
+	remove_depth(O_POLYLINE, l->depth);
+}
+
+void remove_spline_depths(F_spline *s)
+{
+    for ( ; s; s = s->next)
+	remove_depth(O_SPLINE, s->depth);
+}
+
+void remove_text_depths(F_text *t)
+{
+    for ( ; t; t = t->next)
+	remove_depth(O_TEXT, t->depth);
+}
+
+void append_point(int x, int y, F_point **point)    /** used in d_arcbox **/
+       		         
+           	          
 {
     F_point	   *p;
 
@@ -811,9 +851,7 @@ append_point(x, y, point)    /** used in d_arcbox **/
 }
 
 Boolean
-insert_point(x,y,point)
-    int		    x, y;
-    F_point	  *point;
+insert_point(int x, int y, F_point *point)
 {
     F_point	  *p;
 
@@ -828,9 +866,7 @@ insert_point(x,y,point)
 }
 
 Boolean
-append_sfactor(s, cpoint)
-     double        s;
-     F_sfactor     *cpoint;
+append_sfactor(double s, F_sfactor *cpoint)
 {
   F_sfactor *newpoint;
 
@@ -844,10 +880,7 @@ append_sfactor(s, cpoint)
 
 
 Boolean
-first_spline_point(x, y, s , spline)
-     int           x, y;
-     double        s;
-     F_spline      *spline;
+first_spline_point(int x, int y, double s, F_spline *spline)
 {
   F_point   *newpoint;
   F_sfactor *cpoint;
@@ -867,124 +900,122 @@ first_spline_point(x, y, s , spline)
   return True;
 }
 
-num_points(points)
-    F_point	   *points;
+int num_points(F_point *points)
 {
     int		    n;
     F_point	   *p;
 
-    for (p = points, n = 0; p != NULL; p = p->next, n++);
+    for (p = points, n = 0; p != NULL; p = p->next, n++)
+	    ;
     return (n);
 }
 
 F_text	       *
-last_text(list)
-    F_text	   *list;
+last_text(F_text *list)
 {
     F_text	   *tt;
 
     if (list == NULL)
 	return NULL;
 
-    for (tt = list; tt->next != NULL; tt = tt->next);
+    for (tt = list; tt->next != NULL; tt = tt->next)
+	    ;
     return tt;
 }
 
 F_line	       *
-last_line(list)
-    F_line	   *list;
+last_line(F_line *list)
 {
     F_line	   *ll;
 
     if (list == NULL)
 	return NULL;
 
-    for (ll = list; ll->next != NULL; ll = ll->next);
+    for (ll = list; ll->next != NULL; ll = ll->next)
+	    ;
     return ll;
 }
 
 F_spline       *
-last_spline(list)
-    F_spline	   *list;
+last_spline(F_spline *list)
 {
     F_spline	   *ss;
 
     if (list == NULL)
 	return NULL;
 
-    for (ss = list; ss->next != NULL; ss = ss->next);
+    for (ss = list; ss->next != NULL; ss = ss->next)
+	    ;
     return ss;
 }
 
 F_arc	       *
-last_arc(list)
-    F_arc	   *list;
+last_arc(F_arc *list)
 {
     F_arc	   *tt;
 
     if (list == NULL)
 	return NULL;
 
-    for (tt = list; tt->next != NULL; tt = tt->next);
+    for (tt = list; tt->next != NULL; tt = tt->next)
+	    ;
     return tt;
 }
 
 F_ellipse      *
-last_ellipse(list)
-    F_ellipse	   *list;
+last_ellipse(F_ellipse *list)
 {
     F_ellipse	   *tt;
 
     if (list == NULL)
 	return NULL;
 
-    for (tt = list; tt->next != NULL; tt = tt->next);
+    for (tt = list; tt->next != NULL; tt = tt->next)
+	    ;
     return tt;
 }
 
 F_compound     *
-last_compound(list)
-    F_compound	   *list;
+last_compound(F_compound *list)
 {
     F_compound	   *tt;
 
     if (list == NULL)
 	return NULL;
 
-    for (tt = list; tt->next != NULL; tt = tt->next);
+    for (tt = list; tt->next != NULL; tt = tt->next)
+	    ;
     return tt;
 }
 
 F_point	       *
-last_point(list)
-    F_point	   *list;
+last_point(F_point *list)
 {
     F_point	   *tt;
 
     if (list == NULL)
 	return NULL;
 
-    for (tt = list; tt->next != NULL; tt = tt->next);
+    for (tt = list; tt->next != NULL; tt = tt->next)
+	    ;
     return tt;
 }
 
 F_sfactor       *
-last_sfactor(list)
-    F_sfactor	   *list;
+last_sfactor(F_sfactor *list)
 {
     F_sfactor	   *tt;
 
     if (list == NULL)
 	return NULL;
 
-    for (tt = list; tt->next != NULL; tt = tt->next);
+    for (tt = list; tt->next != NULL; tt = tt->next)
+	    ;
     return tt;
 }
 
 F_point        *
-search_line_point(line, x, y)
-     F_line      *line;
-     int           x, y;
+search_line_point(F_line *line, int x, int y)
 {
   F_point *point;
 
@@ -994,9 +1025,7 @@ search_line_point(line, x, y)
 }
 
 F_point        *
-search_spline_point(spline, x, y)
-     F_spline      *spline;
-     int           x, y;
+search_spline_point(F_spline *spline, int x, int y)
 {
   F_point *point;
 
@@ -1007,9 +1036,7 @@ search_spline_point(spline, x, y)
 
 
 F_sfactor      *
-search_sfactor(spline, selected_point)
-     F_spline      *spline;
-     F_point       *selected_point;
+search_sfactor(F_spline *spline, F_point *selected_point)
 {
   F_sfactor *c_point = spline->sfactors;
   F_point *cursor;
@@ -1022,99 +1049,98 @@ search_sfactor(spline, selected_point)
 
 
 F_arc	       *
-prev_arc(list, arc)
-    F_arc	   *list, *arc;
+prev_arc(F_arc *list, F_arc *arc)
 {
     F_arc	   *csr;
 
     if (list == arc)
 	return NULL;
 
-    for (csr = list; csr->next != arc; csr = csr->next);
+    for (csr = list; csr->next != arc; csr = csr->next)
+	    ;
     return csr;
 }
 
 F_compound     *
-prev_compound(list, compound)
-    F_compound	   *list, *compound;
+prev_compound(F_compound *list, F_compound *compound)
 {
     F_compound	   *csr;
 
     if (list == compound)
 	return NULL;
 
-    for (csr = list; csr->next != compound; csr = csr->next);
+    for (csr = list; csr->next != compound; csr = csr->next)
+	    ;
     return csr;
 }
 
 F_ellipse      *
-prev_ellipse(list, ellipse)
-    F_ellipse	   *list, *ellipse;
+prev_ellipse(F_ellipse *list, F_ellipse *ellipse)
 {
     F_ellipse	   *csr;
 
     if (list == ellipse)
 	return NULL;
 
-    for (csr = list; csr->next != ellipse; csr = csr->next);
+    for (csr = list; csr->next != ellipse; csr = csr->next)
+	    ;
     return csr;
 }
 
 F_line	       *
-prev_line(list, line)
-    F_line	   *list, *line;
+prev_line(F_line *list, F_line *line)
 {
     F_line	   *csr;
 
     if (list == line)
 	return NULL;
 
-    for (csr = list; csr->next != line; csr = csr->next);
+    for (csr = list; csr->next != line; csr = csr->next)
+	    ;
     return csr;
 }
 
 F_spline       *
-prev_spline(list, spline)
-    F_spline	   *list, *spline;
+prev_spline(F_spline *list, F_spline *spline)
 {
     F_spline	   *csr;
 
     if (list == spline)
 	return NULL;
 
-    for (csr = list; csr->next != spline; csr = csr->next);
+    for (csr = list; csr->next != spline; csr = csr->next)
+	    ;
     return csr;
 }
 
 F_text	       *
-prev_text(list, text)
-    F_text	   *list, *text;
+prev_text(F_text *list, F_text *text)
 {
     F_text	   *csr;
 
     if (list == text)
 	return NULL;
 
-    for (csr = list; csr->next != text; csr = csr->next);
+    for (csr = list; csr->next != text; csr = csr->next)
+	    ;
     return csr;
 }
 
 F_point	       *
-prev_point(list, point)
-    F_point	   *list, *point;
+prev_point(F_point *list, F_point *point)
 {
     F_point	   *csr;
 
     if (list == point)
 	return NULL;
 
-    for (csr = list; csr->next != point; csr = csr->next);
+    for (csr = list; csr->next != point; csr = csr->next)
+	    ;
     return csr;
 }
 
 int
-object_count(list)
-    F_compound	   *list;
+object_count(F_compound *list)
 {
     register int    cnt;
     F_arc	   *a;
@@ -1125,18 +1151,22 @@ object_count(list)
     F_spline	   *s;
 
     cnt = 0;
-    for (a = list->arcs; a != NULL; a = a->next, cnt++);
-    for (t = list->texts; t != NULL; t = t->next, cnt++);
-    for (c = list->compounds; c != NULL; c = c->next, cnt++);
-    for (e = list->ellipses; e != NULL; e = e->next, cnt++);
-    for (l = list->lines; l != NULL; l = l->next, cnt++);
-    for (s = list->splines; s != NULL; s = s->next, cnt++);
+    for (a = list->arcs; a != NULL; a = a->next, cnt++)
+	    ;
+    for (t = list->texts; t != NULL; t = t->next, cnt++)
+	    ;
+    for (c = list->compounds; c != NULL; c = c->next, cnt++)
+	    ;
+    for (e = list->ellipses; e != NULL; e = e->next, cnt++)
+	    ;
+    for (l = list->lines; l != NULL; l = l->next, cnt++)
+	    ;
+    for (s = list->splines; s != NULL; s = s->next, cnt++)
+	    ;
     return (cnt);
 }
 
-set_tags(list, tag)
-    F_compound	   *list;
-    int		    tag;
+void set_tags(F_compound *list, int tag)
 {
     F_arc	   *a;
     F_text	   *t;
@@ -1178,8 +1208,7 @@ set_tags(list, tag)
 }
 
 void
-get_links(llx, lly, urx, ury)
-    int		    llx, lly, urx, ury;
+get_links(int llx, int lly, int urx, int ury)
 {
     F_line	   *l;
     F_point	   *a;
@@ -1222,9 +1251,7 @@ get_links(llx, lly, urx, ury)
 static int LINK_TOL = 3 * PIX_PER_INCH / DISPLAY_PIX_PER_INCH;
 
 int
-point_on_perim(p, llx, lly, urx, ury)
-    F_point	   *p;
-    int		    llx, lly, urx, ury;
+point_on_perim(F_point *p, int llx, int lly, int urx, int ury)
 {
     return ((abs(p->x - llx) <= LINK_TOL && p->y >= lly - LINK_TOL
 	     && p->y <= ury + LINK_TOL) ||
@@ -1241,8 +1268,7 @@ point_on_perim(p, llx, lly, urx, ury)
 */
 
 void
-get_interior_links(llx, lly, urx, ury)
-    int		    llx, lly, urx, ury;
+get_interior_links(int llx, int lly, int urx, int ury)
 {
     F_line	   *l;
     F_point	   *a;
@@ -1283,9 +1309,7 @@ get_interior_links(llx, lly, urx, ury)
 }
 
 int
-point_on_inside(p, llx, lly, urx, ury)
-    F_point	   *p;
-    int		    llx, lly, urx, ury;
+point_on_inside(F_point *p, int llx, int lly, int urx, int ury)
 {
     return ((p->x >= llx) && (p->x <= urx) &&
 	    (p->y >= lly) && (p->y <= ury));
@@ -1293,13 +1317,13 @@ point_on_inside(p, llx, lly, urx, ury)
 }
 
 void
-adjust_links(mode, links, dx, dy, cx, cy, sx, sy, copying)
-    int		    mode;
-    F_linkinfo	   *links;
-    int		    dx, dy;		/* delta */
-    int		    cx, cy;		/* center of scale - NOT USED YET */
-    float	    sx, sy;		/* scale factor - NOT USED YET */
-    Boolean	    copying;
+adjust_links(int mode, F_linkinfo *links, int dx, int dy, int cx, int cy, float sx, float sy, Boolean copying)
+       		         
+              	          
+       		           		/* delta */
+       		           		/* center of scale - NOT USED YET */
+         	           		/* scale factor - NOT USED YET */
+           	            
 {
     F_linkinfo	   *k;
     F_line	   *l;
