@@ -1,17 +1,17 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-1998 by Brian V. Smith
+ * Parts Copyright (c) 1989-2007 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
@@ -24,11 +24,28 @@
 #include "e_flip.h"
 #include "e_rotate.h"
 #include "u_draw.h"
+#include "u_redraw.h"
 #include "w_canvas.h"
+#include "w_drawprim.h"
+#include "w_file.h"
+#include "w_indpanel.h"
+#include "w_layers.h"
 #include "w_setup.h"
 #include "w_util.h"
 #include "w_zoom.h"
 
+#include "d_text.h"
+#include "u_bound.h"
+#include "u_elastic.h"
+#include "u_markers.h"
+#include "w_cursor.h"
+#include "w_rulers.h"
+
+/* EXPORTS */
+
+/* set in redisplay_region if called when preview_in_progress is true */
+Boolean	request_redraw = False;
+
 /*
  * Support for rendering based on correct object depth.	 A simple depth based
  * caching scheme; anything more will require major surgery on the object
@@ -40,33 +57,29 @@
  * down on search loop overhead.
  */
 
-struct counts {
-    unsigned	    num_arcs;		/* # arcs at this depth */
-    unsigned	    num_lines;		/* # lines at this depth */
-    unsigned	    num_ellipses;	/* # ellipses at this depth */
-    unsigned	    num_splines;	/* # splines at this depth */
-    unsigned	    num_texts;		/* # texts at this depth */
-    unsigned	    cnt_arcs;		/* count of arcs drawn at this depth */
-    unsigned	    cnt_lines;		/* count of lines drawn at this depth */
-    unsigned	    cnt_ellipses;	/* count of ellipses drawn at this
-					 * depth */
-    unsigned	    cnt_splines;	/* count of splines drawn at this depth */
-    unsigned	    cnt_texts;		/* count of texts drawn at this depth */
-};
-
 /*
  * The array of ``counts'' structures.	All objects at depth >= MAX_DEPTH are
  * accounted for in the counts[MAX_DEPTH] entry.
  */
 
-struct counts	counts[MAX_DEPTH + 1];
+struct counts	counts[MAX_DEPTH + 1], saved_counts[MAX_DEPTH + 1];
 
 /*
- * Function to clear the array of object counts prior to each redraw.
+ * Function to clear the array of object counts with file load or new command.
  */
 
-static void
-clearcounts()
+
+void redisplay_arcobject (F_arc *arcs, int depth);
+void redisplay_compoundobject (F_compound *compounds, int depth);
+void redisplay_ellipseobject (F_ellipse *ellipses, int depth);
+void redisplay_lineobject (F_line *lines, int depth);
+void redisplay_splineobject (F_spline *splines, int depth);
+void redisplay_textobject (F_text *texts, int depth);
+void redraw_pageborder (void);
+void draw_pb (int x, int y, int w, int h);
+
+void
+clearallcounts(void)
 {
     register struct counts *cp;
 
@@ -76,6 +89,20 @@ clearcounts()
 	cp->num_ellipses = 0;
 	cp->num_splines = 0;
 	cp->num_texts = 0;
+    }
+    clearcounts();
+}
+
+/*
+ * Clear count of objects drawn at each depth
+ */
+
+void
+clearcounts(void)
+{
+    register struct counts *cp;
+
+    for (cp = &counts[0]; cp <= &counts[MAX_DEPTH]; ++cp) {
 	cp->cnt_arcs = 0;
 	cp->cnt_lines = 0;
 	cp->cnt_ellipses = 0;
@@ -84,43 +111,66 @@ clearcounts()
     }
 }
 
-unsigned int	max_depth;
-
-redisplay_objects(objects)
-    F_compound	   *objects;
+void redisplay_objects(F_compound *active_objects)
 {
     int		    depth;
+    F_compound	   *objects, *save_objects;
+  
+    objects = active_objects;
+    save_objects = (F_compound *) NULL;
+
+    draw_parent_gray = False;
 
     if (objects == NULL)
 	return;
 
     /*
-     * Clear object counts, and then get the max. depth of any object from
-     * the max. depths of each object type in the top level compound.
+     * Opened compound with `keep parent visible'?
      */
+    for (; (objects->parent != NULL) && (objects->draw_parent); ) {
+	if (!save_objects)
+	    save_objects = objects;
+	/* put in any changes */
+	*((F_compound*)objects->GABPtr) = *objects;            
+	/* follow parent to the top */
+	objects = objects->parent;
+	/* instruct lower level procs to draw gray */
+	draw_parent_gray = True;
+    }
 
     clearcounts();
-    max_depth = max2(compound_depths(objects->compounds),
-		  max2(text_depths(objects->texts),
-		       spline_depths(objects->splines)));
-    max_depth = max2(arc_depths(objects->arcs),
-		     max2(line_depths(objects->lines),
-			  max2(ellipse_depths(objects->ellipses),
-			       max_depth)));
 
-    /*
-     * A new outer loop, executing once per depth level from max_depth down
-     * to 0 (negative depths are not supported).  The code inside the loop is
-     * the original code for redisplay_objects.
-     */
+    /* if user wants gray inactive layers, draw them first */
+    if (gray_layers || draw_parent_gray) {
+	for (depth = max_depth; depth >= min_depth; --depth) {
+	    if (!active_layer(depth) || draw_parent_gray) {
+		redisplay_arcobject(objects->arcs, depth);
+		redisplay_compoundobject(objects->compounds, depth);
+		redisplay_ellipseobject(objects->ellipses, depth);
+		redisplay_lineobject(objects->lines, depth);
+		redisplay_splineobject(objects->splines, depth);
+		redisplay_textobject(objects->texts, depth);
+	    }
+	}
+    }
 
-    for (depth = max_depth; depth >= 0; --depth) {
-	redisplay_arcobject(objects->arcs, depth);
-	redisplay_compoundobject(objects->compounds, depth);
-	redisplay_ellipseobject(objects->ellipses, depth);
-	redisplay_lineobject(objects->lines, depth);
-	redisplay_splineobject(objects->splines, depth);
-	redisplay_textobject(objects->texts, depth);
+    if (draw_parent_gray) {
+	/* now point to just the open compound and (re)draw it */
+	clearcounts();
+	objects = save_objects;
+	draw_parent_gray = False;
+    }
+
+    /* now draw the active layers in their normal colors */
+    for (depth = max_depth; depth >= min_depth; --depth) {
+	if (active_layer(depth)) {
+	    redisplay_arcobject(objects->arcs, depth);
+	    redisplay_compoundobject(objects->compounds, depth);
+	    redisplay_ellipseobject(objects->ellipses, depth);
+	    redisplay_lineobject(objects->lines, depth);
+	    redisplay_splineobject(objects->splines, depth);
+	    redisplay_textobject(objects->texts, depth);
+	}
     }
 
     /*
@@ -129,154 +179,29 @@ redisplay_objects(objects)
      */
 
     /* show the markers if they are on */
-    toggle_markers_in_compound(objects);
+    toggle_markers_in_compound(active_objects);
+    /* mark any center if requested */
     if (setcenter)
 	center_marker(setcenter_x, setcenter_y);
+    /* and any anchor */
     if (setanchor)
 	center_marker(setanchor_x, setanchor_y);
 }
 
-/*
- * Find the maximum depth of any arc, recording the number of arcs per each
- * level along the way.
- */
-
-int
-arc_depths(arcs)
-    F_arc	   *arcs;
-{
-    int		    maxdepth = 0;
-    F_arc	   *fp;
-
-    for (fp = arcs; fp != NULL; fp = fp->next) {
-	if (maxdepth < fp->depth)
-	    maxdepth = fp->depth;
-
-	++counts[min2(fp->depth, MAX_DEPTH)].num_arcs;
-    }
-    return maxdepth;
-}
-
-/*
- * Find the maximum depth of any line, recording the number of lines per each
- * level along the way.
- */
-
-int
-line_depths(lines)
-    F_line	   *lines;
-{
-    int		    maxdepth = 0;
-    F_line	   *fp;
-
-    for (fp = lines; fp != NULL; fp = fp->next) {
-	if (maxdepth < fp->depth)
-	    maxdepth = fp->depth;
-
-	++counts[min2(fp->depth, MAX_DEPTH)].num_lines;
-    }
-    return maxdepth;
-}
-
-/*
- * Find the maximum depth of any ellipse, recording the number of ellipses
- * per each level along the way.
- */
-
-int
-ellipse_depths(ellipses)
-    F_ellipse	   *ellipses;
-{
-    int		    maxdepth = 0;
-    F_ellipse	   *fp;
-
-    for (fp = ellipses; fp != NULL; fp = fp->next) {
-	if (maxdepth < fp->depth)
-	    maxdepth = fp->depth;
-
-	++counts[min2(fp->depth, MAX_DEPTH)].num_ellipses;
-    }
-    return maxdepth;
-}
-
-/*
- * Find the maximum depth of any spline, recording the number of splines per
- * each level along the way.
- */
-
-int
-spline_depths(splines)
-    F_spline	   *splines;
-{
-    int		    maxdepth = 0;
-    F_spline	   *fp;
-
-    for (fp = splines; fp != NULL; fp = fp->next) {
-	if (maxdepth < fp->depth)
-	    maxdepth = fp->depth;
-
-	++counts[min2(fp->depth, MAX_DEPTH)].num_splines;
-    }
-    return maxdepth;
-}
-
-/*
- * Find the maximum depth of any text, recording the number of texts per each
- * level along the way.
- */
-
-int
-text_depths(texts)
-    F_text	   *texts;
-{
-    int		    maxdepth = 0;
-    F_text	   *fp;
-
-    for (fp = texts; fp != NULL; fp = fp->next) {
-	if (maxdepth < fp->depth)
-	    maxdepth = fp->depth;
-
-	++counts[min2(fp->depth, MAX_DEPTH)].num_texts;
-    }
-    return maxdepth;
-}
-
-/*
- * Find the maximum depth of any of the objects contained in the compound.
- */
-
-int
-compound_depths(compounds)
-    F_compound	   *compounds;
-{
-    int		    maxdepth = 0;
-    F_compound	   *fp;
-
-    for (fp = compounds; fp != NULL; fp = fp->next) {
-	maxdepth = max2(compound_depths(fp->compounds),
-		      max2(text_depths(fp->texts),
-			   max2(spline_depths(fp->splines),
-				maxdepth)));
-	maxdepth = max2(arc_depths(fp->arcs),
-			max2(line_depths(fp->lines),
-			     max2(ellipse_depths(fp->ellipses),
-				  maxdepth)));
-    }
-    return maxdepth;
-}
-
 /*
  * Redisplay a list of arcs.  Only display arcs of the correct depth.
  * For each arc drawn, update the count for the appropriate depth in
  * the counts array.
  */
 
-redisplay_arcobject(arcs, depth)
-    F_arc	   *arcs;
-    int		    depth;
+void redisplay_arcobject(F_arc *arcs, int depth)
 {
     F_arc	   *arc;
-    struct counts  *cp = &counts[min2(depth, MAX_DEPTH)];
+    struct counts  *cp;
+    
+    if (arcs == NULL)
+	return;
+    cp = &counts[min2(depth, MAX_DEPTH)];
 
     arc = arcs;
     while (arc != NULL && cp->cnt_arcs < cp->num_arcs) {
@@ -294,12 +219,14 @@ redisplay_arcobject(arcs, depth)
  * appropriate depth in the counts array.
  */
 
-redisplay_ellipseobject(ellipses, depth)
-    F_ellipse	   *ellipses;
-    int		    depth;
+void redisplay_ellipseobject(F_ellipse *ellipses, int depth)
 {
     F_ellipse	   *ep;
-    struct counts  *cp = &counts[min2(depth, MAX_DEPTH)];
+    struct counts  *cp;
+    
+    if (ellipses == NULL)
+	return;
+    cp = &counts[min2(depth, MAX_DEPTH)];
 
 
     ep = ellipses;
@@ -318,12 +245,14 @@ redisplay_ellipseobject(ellipses, depth)
  * depth in the counts array.
  */
 
-redisplay_lineobject(lines, depth)
-    F_line	   *lines;
-    int		    depth;
+void redisplay_lineobject(F_line *lines, int depth)
 {
     F_line	   *lp;
-    struct counts  *cp = &counts[min2(depth, MAX_DEPTH)];
+    struct counts  *cp;
+    
+    if (lines == NULL)
+	return;
+    cp = &counts[min2(depth, MAX_DEPTH)];
 
 
     lp = lines;
@@ -342,12 +271,14 @@ redisplay_lineobject(lines, depth)
  * appropriate depth in the counts array.
  */
 
-redisplay_splineobject(splines, depth)
-    F_spline	   *splines;
-    int		    depth;
+void redisplay_splineobject(F_spline *splines, int depth)
 {
     F_spline	   *spline;
-    struct counts  *cp = &counts[min2(depth, MAX_DEPTH)];
+    struct counts  *cp;
+    
+    if (splines == NULL)
+	return;
+    cp = &counts[min2(depth, MAX_DEPTH)];
 
     spline = splines;
     while (spline != NULL && cp->cnt_splines < cp->num_splines) {
@@ -365,12 +296,14 @@ redisplay_splineobject(splines, depth)
  * array.
  */
 
-redisplay_textobject(texts, depth)
-    F_text	   *texts;
-    int		    depth;
+void redisplay_textobject(F_text *texts, int depth)
 {
     F_text	   *text;
-    struct counts  *cp = &counts[min2(depth, MAX_DEPTH)];
+    struct counts  *cp;
+    
+    if (texts == NULL)
+	return;
+    cp = &counts[min2(depth, MAX_DEPTH)];
 
     text = texts;
     while (text != NULL && cp->cnt_texts < cp->num_texts) {
@@ -387,9 +320,7 @@ redisplay_textobject(texts, depth)
  * work out to the objects contained in the compound.
  */
 
-redisplay_compoundobject(compounds, depth)
-    F_compound	   *compounds;
-    int		    depth;
+void redisplay_compoundobject(F_compound *compounds, int depth)
 {
     F_compound	   *c;
 
@@ -406,14 +337,19 @@ redisplay_compoundobject(compounds, depth)
 /*
  * Redisplay the entire drawing.
  */
-redisplay_canvas()
+void
+redisplay_canvas(void)
 {
+    /* turn off Compose key LED */
+    setCompLED(0);
+
     redisplay_region(0, 0, CANVAS_WD, CANVAS_HT);
+    reset_rulers();
 }
 
 /* redisplay the object currently being created by the user (if any) */
 
-redisplay_curobj()
+void redisplay_curobj(void)
 {
     F_point	*p;
     int		 rx,ry,i;
@@ -424,12 +360,14 @@ redisplay_curobj()
 
     /* find which type of object we need to refresh */
 
-    if (cur_mode >= FIRST_EDIT_MODE && canvas_locmove_proc != null_proc) {
-	(*canvas_locmove_proc)(last_x, last_y);
-    } else {
+    /* creating an object */
+    /* placing a library object is slightly different */
+    if (cur_mode < FIRST_EDIT_MODE) {
       switch (cur_mode) {
+	case F_PLACE_LIB_OBJ:
+	   break;	/* don't do anything */
 	case F_PICOBJ:
-	case F_ARC_BOX:
+	case F_ARCBOX:
 	case F_BOX:
 	   elastic_box(fix_x, fix_y, cur_x, cur_y);
 	   break;
@@ -461,23 +399,32 @@ redisplay_curobj()
 		    ry = cur_y;
 		}
 		pw_vector(canvas_win, p->x, p->y, rx, ry,
-		      INV_PAINT, 1, RUBBER_LINE, 0.0, DEFAULT);
+		      PAINT, 1, RUBBER_LINE, 0.0, DEFAULT);
 		/* erase endpoint because next seg will redraw it */
 		pw_vector(canvas_win, rx, ry, rx, ry,
 		      INV_PAINT, 1, RUBBER_LINE, 0.0, DEFAULT);
 	    }
 	    break;
 	case F_CIRCULAR_ARC:
-	    for (i=0; i<num_point; i++) {
-		if (i < num_point-1) {
-		    rx = point[i+1].x;
-		    ry = point[i+1].y;
-		} else {
-		    rx = cur_x;
-		    ry = cur_y;
+	    if (center_marked) {
+	        center_marker(center_point.x, center_point.y);
+		/* redraw either circle or partial arc */
+		if (num_point)
+		    elastic_arc();
+		else
+		    elastic_cbr();
+	    } else {
+		for (i=0; i<num_point; i++) {
+		    if (i < num_point-1) {
+			rx = point[i+1].x;
+			ry = point[i+1].y;
+		    } else {
+			rx = cur_x;
+			ry = cur_y;
+		    }
+		    pw_vector(canvas_win, point[i].x, point[i].y, rx, ry,
+				INV_PAINT, 1, RUBBER_LINE, 0.0, DEFAULT);
 		}
-		pw_vector(canvas_win, point[i].x, point[i].y, rx, ry,
-		      INV_PAINT, 1, RUBBER_LINE, 0.0, DEFAULT);
 	    }
 	    break;
 	case F_TEXT:
@@ -489,12 +436,23 @@ redisplay_curobj()
 	    draw_char_string();
 	    break;
       }
+    } else {
+	/* editing an object, just refresh it as is */
+	(*canvas_ref_proc)(cur_x, cur_y);
     }
 }
 
-redisplay_region(xmin, ymin, xmax, ymax)
-    int		    xmin, ymin, xmax, ymax;
+void redisplay_region(int xmin, int ymin, int xmax, int ymax)
 {
+    /* if we're generating a preview, don't redisplay the canvas 
+       but set request flag so preview will call us with full canvas 
+       after it is done generating the preview */
+
+    if (preview_in_progress) {
+	request_redraw = True;
+	return;
+    }
+
     set_temp_cursor(wait_cursor);
     /* kludge so that markers are redrawn */
     xmin -= 10;
@@ -509,14 +467,118 @@ redisplay_region(xmin, ymin, xmax, ymax)
     reset_cursor();
 }
 
-redisplay_zoomed_region(xmin, ymin, xmax, ymax)
-    int		    xmin, ymin, xmax, ymax;
+/* update page border with new page size */
+
+void update_pageborder(void)
+{
+    if (appres.show_pageborder) {
+	clear_canvas();
+	redisplay_canvas();		
+    }
+}
+
+/* make a light blue line showing the right and bottom edge of the current page size */
+/* also make light blue lines vertically and horizontally through 0,0 if
+   showaxislines is true */
+
+void redisplay_pageborder(void)
+{
+
+    set_clip_window(clip_xmin, clip_ymin, clip_xmax, clip_ymax);
+    /* first the axis lines */
+    if (appres.showaxislines) {
+	/* set the color */
+	XSetForeground(tool_d, border_gc, axis_lines_color);
+	XDrawLine(tool_d, canvas_win, border_gc, round(-zoomxoff*zoomscale), 0, 
+					  round(-zoomxoff*zoomscale), CANVAS_HT);
+	XDrawLine(tool_d, canvas_win, border_gc, 0, round(-zoomyoff*zoomscale), CANVAS_WD, 
+					  round(-zoomyoff*zoomscale));
+    }
+    /* now the page border if user wants it */
+    if (appres.show_pageborder) 
+	redraw_pageborder();
+}
+
+void redraw_pageborder(void)
+{
+    int		   pwd, pht, ux;
+    int		   x, y;
+    char	   pname[80];
+
+    /* set the color */
+    XSetForeground(tool_d, border_gc, pageborder_color);
+
+    pwd = paper_sizes[appres.papersize].width;
+    pht = paper_sizes[appres.papersize].height;
+    if (!appres.INCHES) {
+	pwd = (int)(pwd*2.54*PIX_PER_CM/PIX_PER_INCH);
+	pht = (int)(pht*2.54*PIX_PER_CM/PIX_PER_INCH);
+    }
+    /* swap height and width if landscape */
+    if (appres.landscape) {
+	ux = pwd;
+	pwd = pht;
+	pht = ux;
+    }
+    /* if multiple page mode, draw all page borders that are visible */
+    if (appres.multiple) {
+	int	wd, ht;
+
+	wd = CANVAS_WD/zoomscale;
+	ht = CANVAS_HT/zoomscale;
+
+	/* draw the page borders in the positive direction */
+	for (y=0; y < ht+zoomyoff; y += pht) {
+	    for (x=0; x < wd+zoomxoff; x += pwd) {
+		draw_pb(x, y, pwd, pht);
+	    }
+	}
+	/* now draw the page borders in the -X, -Y direction */
+	for (y=0; y > zoomyoff; y -= pht) {
+	    for (x=0; x > zoomxoff; x -= pwd) {
+		draw_pb(x, y, -pwd, -pht);
+	    }
+	}
+	/* now draw the page borders in the -X, +Y direction */
+	for (y=0; y < ht+zoomyoff; y += pht) {
+	    for (x=0; x > zoomxoff; x -= pwd) {
+		draw_pb(x, y, -pwd, pht);
+	    }
+	}
+	/* now draw the page borders in the +X, -Y direction */
+	for (y=0; y > zoomyoff; y -= pht) {
+	    for (x=0; x < wd+zoomxoff; x += pwd) {
+		draw_pb(x, y, pwd, -pht);
+	    }
+	}
+    } else {
+	zXDrawLine(tool_d, canvas_win, border_gc, 0,   0,   pwd, 0);
+	zXDrawLine(tool_d, canvas_win, border_gc, pwd, 0,   pwd, pht);
+	zXDrawLine(tool_d, canvas_win, border_gc, pwd, pht, 0,   pht);
+	zXDrawLine(tool_d, canvas_win, border_gc, 0,   pht, 0,   0);
+    }
+    /* Now put the paper size in the lower-right corner just outside the page border.
+       We want the full name except for the size in parenthesis */
+    strcpy(pname,paper_sizes[appres.papersize].fname);
+    /* truncate after first part of name */
+    *(strchr(pname, '('))= '\0';
+    XDrawString(tool_d,canvas_win,border_gc,ZOOMX(pwd)+3,ZOOMY(pht),pname,strlen(pname));
+}
+
+void draw_pb(int x, int y, int w, int h)
+{
+	zXDrawLine(tool_d, canvas_win, border_gc, x,   y,   x+w, y);
+	zXDrawLine(tool_d, canvas_win, border_gc, x+w, y,   x+w, y+h);
+	zXDrawLine(tool_d, canvas_win, border_gc, x+w, y+h, x,   y+h);
+	zXDrawLine(tool_d, canvas_win, border_gc, x,   y+h, x,   y);
+}
+
+void redisplay_zoomed_region(int xmin, int ymin, int xmax, int ymax)
 {
     redisplay_region(ZOOMX(xmin), ZOOMY(ymin), ZOOMX(xmax), ZOOMY(ymax));
 }
 
-redisplay_ellipse(e)
-    F_ellipse	   *e;
+void redisplay_ellipse(F_ellipse *e)
 {
     int		    xmin, ymin, xmax, ymax;
 
@@ -524,8 +586,7 @@ redisplay_ellipse(e)
     redisplay_zoomed_region(xmin, ymin, xmax, ymax);
 }
 
-redisplay_ellipses(e1, e2)
-    F_ellipse	   *e1, *e2;
+void redisplay_ellipses(F_ellipse *e1, F_ellipse *e2)
 {
     int		    xmin1, ymin1, xmax1, ymax1;
     int		    xmin2, ymin2, xmax2, ymax2;
@@ -535,17 +596,36 @@ redisplay_ellipses(e1, e2)
     redisplay_regions(xmin1, ymin1, xmax1, ymax1, xmin2, ymin2, xmax2, ymax2);
 }
 
-redisplay_arc(a)
-    F_arc	   *a;
+void redisplay_arc(F_arc *a)
 {
     int		    xmin, ymin, xmax, ymax;
+    int		    cx, cy;
 
     arc_bound(a, &xmin, &ymin, &xmax, &ymax);
+    /* if vertices (and center point) are shown, make sure to include them in the clip area */
+    if (appres.shownums) {
+	/* first adjust for height/width of labels (don't really care which way they are) */
+	xmin -= 175;
+	xmax += 175;
+	ymin -= 175;
+	/* (the numbers are above the vertices so we don't need to move ymax) */
+
+	/* now adjust for center point if it is outside the arc */
+	cx = a->center.x;
+	cy = a->center.y;
+	if (cx < xmin+80)
+	    xmin = cx-80;
+	if (cx > xmax-150)
+	    xmax = cx+150;
+	if (cy < ymin+150)
+	    ymin = cy-150;
+	if (cy > ymax-80)
+	    ymax = cy;
+    }
     redisplay_zoomed_region(xmin, ymin, xmax, ymax);
 }
 
-redisplay_arcs(a1, a2)
-    F_arc	   *a1, *a2;
+void redisplay_arcs(F_arc *a1, F_arc *a2)
 {
     int		    xmin1, ymin1, xmax1, ymax1;
     int		    xmin2, ymin2, xmax2, ymax2;
@@ -555,8 +635,7 @@ redisplay_arcs(a1, a2)
     redisplay_regions(xmin1, ymin1, xmax1, ymax1, xmin2, ymin2, xmax2, ymax2);
 }
 
-redisplay_spline(s)
-    F_spline	   *s;
+void redisplay_spline(F_spline *s)
 {
     int		    xmin, ymin, xmax, ymax;
 
@@ -564,8 +643,7 @@ redisplay_spline(s)
     redisplay_zoomed_region(xmin, ymin, xmax, ymax);
 }
 
-redisplay_splines(s1, s2)
-    F_spline	   *s1, *s2;
+void redisplay_splines(F_spline *s1, F_spline *s2)
 {
     int		    xmin1, ymin1, xmax1, ymax1;
     int		    xmin2, ymin2, xmax2, ymax2;
@@ -575,8 +653,7 @@ redisplay_splines(s1, s2)
     redisplay_regions(xmin1, ymin1, xmax1, ymax1, xmin2, ymin2, xmax2, ymax2);
 }
 
-redisplay_line(l)
-    F_line	   *l;
+void redisplay_line(F_line *l)
 {
     int		    xmin, ymin, xmax, ymax;
 
@@ -584,8 +661,7 @@ redisplay_line(l)
     redisplay_zoomed_region(xmin, ymin, xmax, ymax);
 }
 
-redisplay_lines(l1, l2)
-    F_line	   *l1, *l2;
+void redisplay_lines(F_line *l1, F_line *l2)
 {
     int		    xmin1, ymin1, xmax1, ymax1;
     int		    xmin2, ymin2, xmax2, ymax2;
@@ -595,15 +671,13 @@ redisplay_lines(l1, l2)
     redisplay_regions(xmin1, ymin1, xmax1, ymax1, xmin2, ymin2, xmax2, ymax2);
 }
 
-redisplay_compound(c)
-    F_compound	   *c;
+void redisplay_compound(F_compound *c)
 {
     redisplay_zoomed_region(c->nwcorner.x, c->nwcorner.y,
 			    c->secorner.x, c->secorner.y);
 }
 
-redisplay_compounds(c1, c2)
-    F_compound	   *c1, *c2;
+void redisplay_compounds(F_compound *c1, F_compound *c2)
 {
     redisplay_regions(c1->nwcorner.x, c1->nwcorner.y,
 		      c1->secorner.x, c1->secorner.y,
@@ -611,8 +685,7 @@ redisplay_compounds(c1, c2)
 		      c2->secorner.x, c2->secorner.y);
 }
 
-redisplay_text(t)
-    F_text	   *t;
+void redisplay_text(F_text *t)
 {
     int		    xmin, ymin, xmax, ymax;
     int		    dum;
@@ -622,8 +695,7 @@ redisplay_text(t)
     redisplay_zoomed_region(xmin, ymin, xmax, ymax);
 }
 
-redisplay_texts(t1, t2)
-    F_text	   *t1, *t2;
+void redisplay_texts(F_text *t1, F_text *t2)
 {
     int		    xmin1, ymin1, xmax1, ymax1;
     int		    xmin2, ymin2, xmax2, ymax2;
@@ -637,8 +709,7 @@ redisplay_texts(t1, t2)
 		      xmin2, ymin2, xmax2, ymax2);
 }
 
-redisplay_regions(xmin1, ymin1, xmax1, ymax1, xmin2, ymin2, xmax2, ymax2)
-    int		    xmin1, ymin1, xmax1, ymax1, xmin2, ymin2, xmax2, ymax2;
+void redisplay_regions(int xmin1, int ymin1, int xmax1, int ymax1, int xmin2, int ymin2, int xmax2, int ymax2)
 {
     if (xmin1 == xmin2 && ymin1 == ymin2 && xmax1 == xmax2 && ymax1 == ymax2) {
 	redisplay_zoomed_region(xmin1, ymin1, xmax1, ymax1);
@@ -653,3 +724,4 @@ redisplay_regions(xmin1, ymin1, xmax1, ymax1, xmin2, ymin2, xmax2, ymax2)
 	redisplay_zoomed_region(xmin2, ymin2, xmax2, ymax2);
     }
 }
+
