@@ -1,18 +1,18 @@
 /*
  * FIG : Facility for Interactive Generation of figures
- * Copyright (c) 1989-2000 by Brian V. Smith
+ * Copyright (c) 1989-2007 by Brian V. Smith
  * Parts Copyright (c) 1990 by Digital Equipment Corporation. All Rights Reserved.
  * Parts Copyright (c) 1991 by Paul King
  * Parts Copyright (c) 1990 by Digital Equipment Corporation
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  * Original xdir code:
  *
@@ -58,14 +58,21 @@
 #include "w_drawprim.h"		/* for max_char_height */
 #include "w_export.h"
 #include "w_file.h"
+#include "w_indpanel.h"
 #include "w_listwidget.h"
+#include "w_msgpanel.h"
 #include "w_setup.h"
 #include "w_util.h"
+
+#include "object.h"
+#include "f_util.h"
+#include "w_cursor.h"
+
 #ifdef HAVE_NO_DIRENT
 #include <sys/dir.h>
 #else
 #include <dirent.h>
-#endif
+#endif /* HAVE_NO_DIRENT */
 
 static char	CurrentSelectionName[PATH_MAX];
 static int	file_entry_cnt, dir_entry_cnt;
@@ -78,12 +85,12 @@ static Boolean	show_hidden = False;
 
 /* Functions */
 
-void		DoChangeDir(),
-		SetDir(),
-		Rescan(),
-		CallbackRescan();
+void		DoChangeDir(char *dir),
+		SetDir(Widget widget, XEvent *event, String *params, Cardinal *num_params),
+		Rescan(Widget widget, XEvent *event, String *params, Cardinal *num_params),
+		CallbackRescan(Widget widget, XtPointer closure, XtPointer call_data);
 
-static void	ParentDir();
+static void	ParentDir(Widget w, XEvent *event, String *params, Cardinal *num_params);
 
 /* Static variables */
 
@@ -116,11 +123,12 @@ static XtActionsRec actionTable[] = {
  * Notes:
  */
 
+
+int wild_match (char *string, char *pattern);
+void NewList (Widget listwidget, String *list);
+
 void
-FileSelected(w, client_data, call_data)
-    Widget	    w;
-    XtPointer	    client_data;
-    XtPointer	    call_data;
+FileSelected(Widget w, XtPointer client_data, XtPointer call_data)
 {
     XawListReturnStruct *ret_struct = (XawListReturnStruct *) call_data;
 
@@ -128,16 +136,16 @@ FileSelected(w, client_data, call_data)
     FirstArg(XtNstring, CurrentSelectionName);
     if (browse_up) {
 	SetValues(browse_selfile);
-		XawTextSetInsertionPoint(browse_selfile, strlen(CurrentSelectionName));
+	XawTextSetInsertionPoint(browse_selfile, strlen(CurrentSelectionName));
     } else if (file_up) {
 	SetValues(file_selfile);
-		XawTextSetInsertionPoint(file_selfile, strlen(CurrentSelectionName));
+	XawTextSetInsertionPoint(file_selfile, strlen(CurrentSelectionName));
 	/* and show a preview of the figure in the preview canvas */
 	preview_figure(CurrentSelectionName, file_popup, preview_widget,
-			preview_size, preview_pixmap);
+			preview_size, preview_port_pixmap, preview_land_pixmap);
     } else if (export_up) {
 	SetValues(exp_selfile);
-		XawTextSetInsertionPoint(exp_selfile, strlen(CurrentSelectionName));
+	XawTextSetInsertionPoint(exp_selfile, strlen(CurrentSelectionName));
     }
     /* if nothing is selected it probably means that the user was impatient and
 	double clicked the file name again while it was still loading the first */
@@ -151,10 +159,7 @@ FileSelected(w, client_data, call_data)
  */
 
 void
-DirSelected(w, client_data, call_data)
-    Widget	    w;
-    XtPointer	    client_data;
-    XtPointer	    call_data;
+DirSelected(Widget w, XtPointer client_data, XtPointer call_data)
 {
     XawListReturnStruct *ret_struct = (XawListReturnStruct *) call_data;
 
@@ -163,10 +168,7 @@ DirSelected(w, client_data, call_data)
 }
 
 void
-ShowHidden(w, client_data, ret_val)
-    Widget	    w;
-    XtPointer	    client_data;
-    XtPointer	    ret_val;
+ShowHidden(Widget w, XtPointer client_data, XtPointer ret_val)
 {
     show_hidden = !show_hidden;
     FirstArg(XtNlabel, show_hidden? "Hide Hidden": "Show Hidden");
@@ -176,13 +178,20 @@ ShowHidden(w, client_data, ret_val)
 }
 
 void
-GoHome(w, client_data, ret_val)
-    Widget	    w;
-    XtPointer	    client_data;
-    XtPointer	    ret_val;
+GoHome(Widget w, XtPointer client_data, XtPointer ret_val)
 {
-     parseuserpath("~",cur_dir);
-     DoChangeDir(cur_dir);
+    char	    dir[PATH_MAX];
+
+    parseuserpath("~",dir);
+    if (browse_up) {
+	strcpy(cur_browse_dir,dir);
+    } else if (file_up) {
+	strcpy(cur_file_dir,dir);
+	strcpy(cur_export_dir,dir);	/* set export directory to file directory */
+    } else if (export_up) {
+	strcpy(cur_export_dir,dir);
+    }
+    DoChangeDir(dir);
 }
 
 /*
@@ -198,11 +207,7 @@ GoHome(w, client_data, ret_val)
  */
 
 void
-SetDir(widget, event, params, num_params)
-    Widget	    widget;
-    XEvent	   *event;
-    String	   *params;
-    Cardinal	   *num_params;
+SetDir(Widget widget, XEvent *event, String *params, Cardinal *num_params)
 {
     char	   *ndir;
 
@@ -210,12 +215,14 @@ SetDir(widget, event, params, num_params)
     FirstArg(XtNstring, &ndir);
     if (browse_up) {
 	GetValues(browse_dir);
-	strcpy(browse_cur_dir,ndir);
+	strcpy(cur_browse_dir,ndir);	/* save in global var */
     } else if (file_up) {
 	GetValues(file_dir);
+	strcpy(cur_file_dir,ndir);	/* save in global var */
+	strcpy(cur_export_dir,ndir);	/* set export directory to file directory */
     } else if (export_up) {
 	GetValues(exp_dir);
-	strcpy(export_cur_dir,ndir);	/* save in global var */
+	strcpy(cur_export_dir,ndir);	/* save in global var */
     }
     /* if there is a ~ in the directory, parse the username */
     if (ndir[0]=='~') {
@@ -227,8 +234,7 @@ SetDir(widget, event, params, num_params)
 }
 
 /* make the full path from ~/partialpath */
-parseuserpath(path,longpath)
-char *path,*longpath;
+void parseuserpath(char *path, char *longpath)
 {
     char	  *p;
     struct passwd *who;
@@ -264,32 +270,35 @@ static Boolean      actions_added=False;
 /* make the filename list file_width wide */
 
 void
-create_dirinfo(file_exp, parent, below, ret_beside, ret_below,
-	       mask_w, dir_w, flist_w, dlist_w, file_width, file_panel)
-    Boolean	    file_exp;
-    Widget	    parent, below, *ret_beside, *ret_below, *mask_w, *dir_w,
-		   *flist_w, *dlist_w;
-    int		    file_width;
-    Boolean	    file_panel;
-
+create_dirinfo(Boolean file_exp, Widget parent, Widget below, Widget *ret_beside, Widget *ret_below, Widget *mask_w, Widget *dir_w, Widget *flist_w, Widget *dlist_w, int file_width, Boolean file_panel)
 {
     Widget	    w,dir_alt,home;
     Widget	    file_viewport;
     Widget	    dir_viewport;
     XFontStruct	   *temp_font;
     int		    char_ht,char_wd;
+    char	   *dir;
 
     dir_entry_cnt = NENTRIES;
     file_entry_cnt = NENTRIES;
     filelist = (char **) calloc(file_entry_cnt, sizeof(char *));
     dirlist = (char **) calloc(dir_entry_cnt, sizeof(char *));
 
-    get_directory(cur_dir);
+    if (browse_up) {
+	get_directory(cur_browse_dir);
+	dir = cur_browse_dir;
+    } else if (export_up) {
+	get_directory(cur_export_dir);
+	dir = cur_export_dir;
+    } else {
+	get_directory(cur_file_dir);
+	dir = cur_file_dir;
+    }
 
-    if (file_panel) {
+    if (file_up) {
 	FirstArg(XtNlabel, "Fig files");
     } else {
-	FirstArg(XtNlabel, " Alternatives");
+	FirstArg(XtNlabel, "     Existing");
     }
     NextArg(XtNfromVert, below);
     NextArg(XtNborderWidth, 0);
@@ -307,7 +316,7 @@ create_dirinfo(file_exp, parent, below, ret_beside, ret_below,
 
     /* make a viewport to hold the list widget of filenames */
     FirstArg(XtNallowVert, True);
-    if (file_panel) {
+    if (file_up) {
 	/* for the file panel, put the viewport below the Alternatives label */
 	NextArg(XtNfromVert, w);
 	NextArg(XtNheight, char_ht * 15);	/* show first 15 filenames */
@@ -315,11 +324,15 @@ create_dirinfo(file_exp, parent, below, ret_beside, ret_below,
 	/* for the export or browse panel, put the viewport beside the Alternatives label */
 	NextArg(XtNfromVert, below);
 	NextArg(XtNfromHoriz, w);
-	NextArg(XtNheight, char_ht * 10);	/* show 10 lines */
+	if (browse_up) {
+	    NextArg(XtNheight, char_ht * 10);	/* show 10 lines for existing browse files */
+	} else {
+	    NextArg(XtNheight, char_ht * 4);	/* show 4 lines for existing export files */
+	}
     }
     NextArg(XtNborderWidth, INTERNAL_BW);
     NextArg(XtNwidth, file_width);
-    NextArg(XtNtop, XtChainTop);	/* chain the viewport resizes fully */
+    NextArg(XtNtop, XtChainTop);		/* chain so the viewport resizes fully */
     NextArg(XtNbottom, XtChainBottom);
     NextArg(XtNleft, XtChainLeft);
     NextArg(XtNright, XtChainRight);
@@ -341,10 +354,10 @@ create_dirinfo(file_exp, parent, below, ret_beside, ret_below,
     /* text widget for the filename mask */
 
     FirstArg(XtNeditType, XawtextEdit);
-    NextArg(XtNscrollHorizontal, XawtextScrollNever);
+    NextArg(XtNleftMargin, 4);
+    NextArg(XtNheight, char_ht * 2);
+    NextArg(XtNscrollHorizontal, XawtextScrollWhenNeeded);
     NextArg(XtNborderWidth, INTERNAL_BW);
-    NextArg(XtNscrollVertical, XawtextScrollNever);
-    NextArg(XtNresize, XawtextResizeWidth);
     NextArg(XtNwidth, file_panel? F_FILE_WIDTH: E_FILE_WIDTH);
     NextArg(XtNfromHoriz, w);
     NextArg(XtNfromVert, file_viewport);
@@ -360,7 +373,7 @@ create_dirinfo(file_exp, parent, below, ret_beside, ret_below,
 
     FirstArg(XtNstring, &dirmask);
     GetValues(*mask_w);
-    if (MakeFileList(cur_dir, dirmask, &dir_list, &file_list) == False)
+    if (MakeFileList(dir, dirmask, &dir_list, &file_list) == False)
 	file_msg("No files in directory?");
 
     FirstArg(XtNlabel, "  Current Dir");
@@ -373,8 +386,9 @@ create_dirinfo(file_exp, parent, below, ret_beside, ret_below,
     w = XtCreateManagedWidget("dir_label", labelWidgetClass,
 			      parent, Args, ArgCount);
 
-    FirstArg(XtNstring, cur_dir);
-    NextArg(XtNinsertPosition, strlen(cur_dir));
+    FirstArg(XtNstring, dir);
+    NextArg(XtNleftMargin, 4);
+    NextArg(XtNinsertPosition, strlen(dir));
     NextArg(XtNheight, char_ht * 2);
     NextArg(XtNborderWidth, INTERNAL_BW);
     NextArg(XtNscrollHorizontal, XawtextScrollWhenNeeded);
@@ -432,6 +446,7 @@ create_dirinfo(file_exp, parent, below, ret_beside, ret_below,
     XtAddCallback(hidden, XtNcallback, ShowHidden, (XtPointer) NULL);
 
     FirstArg(XtNallowVert, True);
+    NextArg(XtNforceBars, True);
     NextArg(XtNfromHoriz, dir_alt);
     NextArg(XtNfromVert, *dir_w);
     NextArg(XtNborderWidth, INTERNAL_BW);
@@ -447,8 +462,8 @@ create_dirinfo(file_exp, parent, below, ret_beside, ret_below,
     FirstArg(XtNlist, file_list);
     /* for file panel use only one column */
     if (file_panel) {
-	NextArg(XtNforceColumns, True);
 	NextArg(XtNdefaultColumns, 1);
+	NextArg(XtNforceColumns, True);
     }
     *flist_w = XtCreateManagedWidget("file_list_panel", figListWidgetClass,
 				     file_viewport, Args, ArgCount);
@@ -459,9 +474,9 @@ create_dirinfo(file_exp, parent, below, ret_beside, ret_below,
     FirstArg(XtNlist, dir_list);
     *dlist_w = XtCreateManagedWidget("dir_list_panel", figListWidgetClass,
 				     dir_viewport, Args, ArgCount);
-
     XtOverrideTranslations(*dlist_w,
 			   XtParseTranslationTable(list_panel_translations));
+
     XtAddCallback(*dlist_w, XtNcallback, DirSelected, (XtPointer) NULL);
 
     if (!actions_added) {
@@ -490,6 +505,7 @@ create_dirinfo(file_exp, parent, below, ret_beside, ret_below,
 
     *ret_beside = w;
     *ret_below = dir_viewport;
+
     return;
 }
 
@@ -500,8 +516,7 @@ create_dirinfo(file_exp, parent, below, ret_beside, ret_below,
  */
 
 static int
-SPComp(s1, s2)
-    char	  **s1, **s2;
+SPComp(char **s1, char **s2)
 {
     return (strcmp(*s1, *s2));
 }
@@ -509,9 +524,7 @@ SPComp(s1, s2)
 #define MAX_MASKS 20
 
 Boolean
-MakeFileList(dir_name, mask, dir_list, file_list)
-    char	   *dir_name;
-    char	   *mask, ***dir_list, ***file_list;
+MakeFileList(char *dir_name, char *mask, char ***dir_list, char ***file_list)
 {
     DIR		  *dirp;
     DIRSTRUCT	  *dp;
@@ -544,7 +557,7 @@ MakeFileList(dir_name, mask, dir_list, file_list)
     app_flush();
 
     /* make copy of mask */
-    cmask = strdup(mask);
+    cmask = my_strdup(mask);
     if ((cmask == NULL) || (*cmask == '\0'))
 	strcpy(cmask,"*");
     wild[0] = strtok(cmask," \t");
@@ -560,7 +573,7 @@ MakeFileList(dir_name, mask, dir_list, file_list)
 	    continue;
 
 	if (IsDirectory(dir_name, dp->d_name)) {
-	    *cur_directory++ = strdup(dp->d_name);
+	    *cur_directory++ = my_strdup(dp->d_name);
 	    if (cur_directory == last_dir) {	/* out of space, make more */
 		dirlist = (char **) realloc(dirlist,
 					2 * dir_entry_cnt * sizeof(char *));
@@ -573,6 +586,7 @@ MakeFileList(dir_name, mask, dir_list, file_list)
 	    match=False;
 	    for (i=0; i<nmasks; i++) {
 		if (wild_match(dp->d_name, wild[i])) {
+		    int wild_match (char *string, char *pattern);
 		    match = True;
 		    break;
 		}
@@ -581,7 +595,7 @@ MakeFileList(dir_name, mask, dir_list, file_list)
 		continue;	/* no, do next */
 	    if (wild[i][0] == '*' && dp->d_name[0] == '.')
 		continue;	/* skip files with leading . */
-	    *cur_file++ = strdup(dp->d_name);
+	    *cur_file++ = my_strdup(dp->d_name);
 	    if (cur_file == last_file) {	/* out of space, make more */
 		filelist = (char **) realloc(filelist,
 				       2 * file_entry_cnt * sizeof(char *));
@@ -612,11 +626,7 @@ MakeFileList(dir_name, mask, dir_list, file_list)
  */
 
 static void
-ParentDir(w, event, params, num_params)
-    Widget	    w;
-    XEvent*	    event;
-    String*	    params;
-    Cardinal*	    num_params;
+ParentDir(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     DoChangeDir("..");
 }
@@ -631,18 +641,23 @@ ParentDir(w, event, params, num_params)
  */
 
 void
-DoChangeDir(dir)
-    char	   *dir;
+DoChangeDir(char *dir)
 {
     char	   *p;
-    char	    ndir[PATH_MAX], tmpdir[PATH_MAX];
+    char	    ndir[PATH_MAX];
 
     
-    strcpy(ndir, cur_dir);
+    if (browse_up) {
+	strcpy(ndir, cur_browse_dir);
+    } else if (file_up) {
+	strcpy(ndir, cur_file_dir);
+    } else if (export_up) {
+	strcpy(ndir, cur_export_dir);
+    }
     if (dir != NULL && dir[0] != '/') { /* relative path, prepend current dir */
 	if (dir[strlen(dir) - 1] == '/')
 	    dir[strlen(dir) - 1] = '\0';
-	if (!strcmp(dir, "..")) {	/* Parent directory. */
+	if (strcmp(dir, "..")==0) {	/* Parent directory. */
 	    if (*ndir == '\0')
 		return;			/* no current directory, */
 					/* can't do anything unless absolute path */
@@ -650,7 +665,7 @@ DoChangeDir(dir)
 	    *p = EOS;
 	    if (ndir[0] == EOS)
 		strcpy(ndir, "/");
-	} else {
+	} else if (strcmp(dir, ".")!=0) {
 	    if (strcmp(ndir, "/"))	/* At the root already */
 		strcat(ndir, "/");
 	    strcat(ndir, dir);
@@ -658,34 +673,32 @@ DoChangeDir(dir)
     } else {
 	strcpy(ndir, dir);		/* abs path copy to ndir */
     }
-    strcpy(tmpdir, cur_dir);		/* save cur_dir in case ndir is bad */
-    strcpy(cur_dir, ndir);
-    if (change_directory(cur_dir) != 0 ) {
-	strcpy(cur_dir, tmpdir);	/* can't go there */
-	return;
+    if (change_directory(ndir) != 0 ) {
+	return;				/* some problem, return */
     } else if (MakeFileList(ndir, dirmask, &dirlist, &filelist) == False) {
 	file_msg("Unable to list directory %s", ndir);
-	strcpy(cur_dir, tmpdir);
 	return;
     }
 
-    FirstArg(XtNstring, cur_dir);
+    FirstArg(XtNstring, ndir);
     /* update the current directory and file/dir list widgets */
     if (browse_up) {
 	SetValues(browse_dir);
-	strcpy(browse_cur_dir,cur_dir);	/* save in global var */
-	XawTextSetInsertionPoint(browse_dir, strlen(cur_dir));
+	strcpy(cur_browse_dir,ndir);	/* update global var */
+	XawTextSetInsertionPoint(browse_dir, strlen(ndir));
 	NewList(browse_flist, filelist);
 	NewList(browse_dlist, dirlist);
     } else if (file_up) {
 	SetValues(file_dir);
-	XawTextSetInsertionPoint(file_dir, strlen(cur_dir));
+	strcpy(cur_file_dir,ndir);	/* update global var */
+	strcpy(cur_export_dir,ndir);	/* set export directory to file directory */
+	XawTextSetInsertionPoint(file_dir, strlen(ndir));
 	NewList(file_flist,filelist);
 	NewList(file_dlist,dirlist);
     } else if (export_up) {
 	SetValues(exp_dir);
-	strcpy(export_cur_dir,cur_dir);	/* save in global var */
-	XawTextSetInsertionPoint(exp_dir, strlen(cur_dir));
+	strcpy(cur_export_dir,ndir);	/* update global var */
+	XawTextSetInsertionPoint(exp_dir, strlen(ndir));
 	NewList(exp_flist, filelist);
 	NewList(exp_dlist, dirlist);
     }
@@ -693,20 +706,13 @@ DoChangeDir(dir)
 }
 
 void 
-CallbackRescan(widget, closure, call_data)
-    Widget    widget;
-    XtPointer closure;
-    XtPointer call_data;
+CallbackRescan(Widget widget, XtPointer closure, XtPointer call_data)
 {
      Rescan(0, 0, 0, 0);
 }
 
 void
-Rescan(widget, event, params, num_params)
-    Widget	widget;
-    XEvent*	event;
-    String*	params;
-    Cardinal*	num_params;
+Rescan(Widget widget, XEvent *event, String *params, Cardinal *num_params)
 {
     char	*dir;
 
@@ -721,7 +727,7 @@ Rescan(widget, event, params, num_params)
 	GetValues(browse_dir);
 	if (change_directory(dir))	/* make sure we are there */
 	    return;
-	strcpy(browse_cur_dir,dir);	/* save in global var */
+	strcpy(cur_browse_dir,dir);	/* save in global var */
 	(void) MakeFileList(dir, dirmask, &dir_list, &file_list);
 	NewList(browse_flist, file_list);
 	NewList(browse_dlist, dir_list);
@@ -732,6 +738,8 @@ Rescan(widget, event, params, num_params)
 	GetValues(file_dir);
 	if (change_directory(dir))	/* make sure we are there */
 	    return;
+	strcpy(cur_file_dir,dir);	/* save in global var */
+	strcpy(cur_export_dir,dir);	/* set export directory to file directory */
 	(void) MakeFileList(dir, dirmask, &dir_list, &file_list);
 	NewList(file_flist,file_list);
 	NewList(file_dlist,dir_list);
@@ -742,32 +750,20 @@ Rescan(widget, event, params, num_params)
 	GetValues(exp_dir);
 	if (change_directory(dir))	/* make sure we are there */
 	    return;
-	strcpy(export_cur_dir,dir);	/* save in global var */
+	strcpy(cur_export_dir,dir);	/* save in global var */
 	(void) MakeFileList(dir, dirmask, &dir_list, &file_list);
 	NewList(exp_flist, file_list);
 	NewList(exp_dlist, dir_list);
     }
 }
 
-NewList(listwidget, list)
-    Widget    listwidget;
-    String   *list;
+void NewList(Widget listwidget, String *list)
 {
-
-	/* delete scrollbars because of a bug in the viewport widget */
-	FirstArg(XtNallowVert, False);
-	NextArg(XtNallowHoriz, False);
-	SetValues(XtParent(listwidget));
-
 	/* install the new list */
 	XawListChange(listwidget, list, 0, 0, True);
 
-	/* re-enable the scrollbars */
-	FirstArg(XtNallowVert, True);
-	NextArg(XtNallowHoriz, True);
-	SetValues(XtParent(listwidget));
-
-	XawListChange(listwidget, list, 0, 0, True);
+	/* install the wheel scrolling for the scrollbar */
+	InstallScroll(listwidget);
 }
 
 /* Function:	IsDirectory() tests to see if a pathname is a directory.
@@ -778,22 +774,20 @@ NewList(listwidget, list)
  */
 
 Boolean
-IsDirectory(path, file)
-    char	   *path;
-    char	   *file;
+IsDirectory(char *path, char *file)
 {
     char	    fullpath[PATH_MAX];
     struct stat	    statbuf;
 
     if (file == NULL)
-	return (False);
+	return False;
     MakeFullPath(path, file, fullpath);
     if (stat(fullpath, &statbuf))	/* error, report that it is not a directory */
-	return (False);
+	return False;
     if (statbuf.st_mode & S_IFDIR)
-	return (True);
+	return True;
     else
-	return (False);
+	return False;
 }
 
 /* Function:	MakeFullPath() creates the full pathname for the given file.
@@ -804,10 +798,7 @@ IsDirectory(path, file)
  */
 
 void
-MakeFullPath(root, filename, pathname)
-    char	   *root;
-    char	   *filename;
-    char	   *pathname;
+MakeFullPath(char *root, char *filename, char *pathname)
 {
     strcpy(pathname, root);
     strcat(pathname, "/");
@@ -852,14 +843,13 @@ MakeFullPath(root, filename, pathname)
 /* The character that inverts a character class; '!' or '^'. */
 #define INVERT '!'
 
-static int	star();
+static int	star(char *string, char *pattern);
 
 /* Return nonzero if `string' matches Unix-style wildcard pattern
    `pattern'; zero if not. */
 
 int
-wild_match(string, pattern)
-    char	   *string, *pattern;
+wild_match(char *string, char *pattern)
 {
     int		    prev;	/* Previous character in character class. */
     int		    matched;	/* If 1, character class has been matched. */
@@ -902,8 +892,7 @@ wild_match(string, pattern)
 }
 
 static int
-star(string, pattern)
-    char	   *string, *pattern;
+star(char *string, char *pattern)
 {
     while (wild_match(string, pattern) == 0)
 	if (*++string == '\0')
