@@ -1,17 +1,17 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2000 by Brian V. Smith
+ * Parts Copyright (c) 1989-2007 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
@@ -21,53 +21,67 @@
 #include "resources.h"
 #include "object.h"
 #include "mode.h"
-#include "w_drawprim.h"		/* for max_char_height */
+#include "f_read.h"
+#include "w_drawprim.h"
 #include "w_dir.h"
 #include "w_file.h"
+#include "w_indpanel.h"
 #include "w_layers.h"
 #include "w_util.h"
 #include "w_setup.h"
 #include "u_redraw.h"
+#include "w_snap.h"
+
+#include "w_cmdpanel.h"
+#include "w_msgpanel.h"
+#include "w_canvas.h"
 
 /* EXPORTS */
 
 /* which layer buttons are active */
 Boolean	active_layers[MAX_DEPTH +1];
-Boolean	active_layer();
-Boolean	any_active_in_compound();
+Boolean	any_active_in_compound(F_compound *cmpnd);
 Widget	layer_form;
+Boolean	gray_layers = True;
 int	object_depths[MAX_DEPTH +1];	/* count of objects at each depth */
 int	saved_depths[MAX_DEPTH +1];	/* saved when delete all is done */
+int	saved_min_depth, saved_max_depth; /* saved min/max depth */
 int	max_depth_request, min_depth_request;
+Boolean	save_layers[MAX_DEPTH+1];
 int	LAYER_WD=DEF_LAYER_WD;
 int	LAYER_HT;
 
 /* LOCALS */
 
-DeclareStaticArgs(12);
+DeclareStaticArgs(20);
 
 static	Widget		all_active_but, all_inactive_but, toggle_all_but;
+static	Widget		graytoggle, blanktoggle, graylabel, blanklabel;
 static	Widget		layer_viewp, layer_canvas;
-static	Widget		below;
-static	XtActionProc	toggle_layer();
-static	XtActionProc	sweep_layer();
-static	XtActionProc	leave_layer();
-static	XtCallbackProc	all_active();
-static	XtCallbackProc	all_inactive();
-static	XtCallbackProc	toggle_all();
-static	XtActionProc	layer_exposed();
+static	void	toggle_layer(Widget w, XButtonEvent *event, String *params, Cardinal *nparams);
+static	void	set_depth_to_layer(Widget w, XButtonEvent *event, String *params, Cardinal *nparams);
+static	void	sweep_layer(Widget w, XButtonEvent *event, String *params, Cardinal *nparams);
+static	void	leave_layer(Widget w, XButtonEvent *event, String *params, Cardinal *nparams);
+static	void	all_active(Widget w, XtPointer closure, XtPointer call_data);
+static	void	all_inactive(Widget w, XtPointer closure, XtPointer call_data);
+static	void	toggle_all(Widget w, XtPointer closure, XtPointer call_data);
+static	void	switch_layer_mode(Widget w, XtPointer closure, XtPointer call_data);
+static	void	layer_exposed(Widget w, XExposeEvent *event, String *params, Cardinal *nparams);
 
+#ifndef XAW3D1_5E
 /* popup message over command button when mouse enters it */
-static void     layer_balloon_trigger();
-static void     layer_unballoon();
+static void     layer_balloon_trigger(Widget widget, XtPointer closure, XEvent *event, Boolean *continue_to_dispatch);
+static void     layer_unballoon(Widget widget, XtPointer closure, XEvent *event, Boolean *continue_to_dispatch);
+#endif /* XAW3D1_5E */
 
-static  void	draw_layer_button();
-static  void	draw_layer_buttons();
+static  void	draw_layer_button(Window win, int but);
+static  void	draw_layer_buttons(void);
 
 XtActionsRec	layer_actions[] =
 {
     {"ExposeLayerButtons", (XtActionProc) layer_exposed},
     {"ToggleLayerButton", (XtActionProc) toggle_layer},
+    {"SetDepthToLayer", (XtActionProc) set_depth_to_layer},
     {"SweepLayerButton", (XtActionProc) sweep_layer},
     {"LeaveLayerButton", (XtActionProc) leave_layer},
 };
@@ -75,45 +89,49 @@ XtActionsRec	layer_actions[] =
 static String	layer_translations =
     "<Btn1Down>:ToggleLayerButton()\n\
      <Btn1Motion>: SweepLayerButton()\n\
+     <Btn3Down>:SetDepthToLayer()\n\
      <Leave>: LeaveLayerButton()\n\
      <Expose>:ExposeLayerButtons()\n";
 
+
+
 void
-reset_layers()
+reset_layers(void)
 {
   int i;
-  for (i=0; i<MAX_DEPTH+1;  i++) {
+  for (i=0; i<=MAX_DEPTH;  i++) {
      active_layers[i] = True;
   }
 }
 
 void
-reset_depths()
+reset_depths(void)
 {
   int i;
-  for (i=0; i<MAX_DEPTH+1;  i++) {
+  for (i=0; i<=MAX_DEPTH;  i++) {
      object_depths[i] = 0;
   }
 }
 
 void
-init_depth_panel(parent)
-    Widget	 parent;
+init_depth_panel(Widget parent)
 {
-    Widget	 label;
+    Widget	 label, below;
+    Widget	 layer_viewform;
 
     /* MOUSEFUN_HT and ind_panel height aren't known yet */
     LAYER_HT = TOPRULER_HT + CANVAS_HT;	
 
-    /* a form to hold the label and a viewport widget */
+    /* main form to hold all the layer stuff */
+
     FirstArg(XtNfromHoriz, sideruler_sw);
     NextArg(XtNdefaultDistance, 1);
     NextArg(XtNwidth, LAYER_WD);
     NextArg(XtNheight, LAYER_HT);
     NextArg(XtNleft, XtChainRight);
     NextArg(XtNright, XtChainRight);
-    layer_form = XtCreateWidget("layer_form",formWidgetClass,
-			parent, Args, ArgCount);     
+    NextArg(XtNfromVert, snap_indicator_panel);
+    layer_form = XtCreateWidget("layer_form",formWidgetClass, parent, Args, ArgCount);     
     if (appres.showdepthmanager)
 	XtManageChild(layer_form);
 
@@ -137,11 +155,13 @@ init_depth_panel(parent)
     below = all_active_but = XtCreateManagedWidget("all_active", commandWidgetClass,
 				layer_form, Args, ArgCount);
     XtAddCallback(below, XtNcallback, (XtCallbackProc)all_active, (XtPointer) NULL);
+#ifndef XAW3D1_5E
     /* popup when mouse passes over button */
-    XtAddEventHandler(below, EnterWindowMask, (Boolean) 0,
-			  layer_balloon_trigger, (XtPointer) "Display all depths");
-    XtAddEventHandler(below, LeaveWindowMask, (Boolean) 0,
-			  layer_unballoon, (XtPointer) NULL);
+    XtAddEventHandler(below, EnterWindowMask, False,
+			  (XtEventHandler)layer_balloon_trigger, (XtPointer) "Display all depths");
+    XtAddEventHandler(below, LeaveWindowMask, False,
+			   (XtEventHandler)layer_unballoon, (XtPointer) NULL);
+#endif /* XAW3D1_5E */
 
     FirstArg(XtNlabel, "All Off");
     NextArg(XtNfromVert, below);
@@ -152,11 +172,13 @@ init_depth_panel(parent)
     below = all_inactive_but = XtCreateManagedWidget("all_inactive", commandWidgetClass,
 				layer_form, Args, ArgCount);
     XtAddCallback(below, XtNcallback, (XtCallbackProc)all_inactive, (XtPointer) NULL);
+#ifndef XAW3D1_5E
     /* popup when mouse passes over button */
-    XtAddEventHandler(below, EnterWindowMask, (Boolean) 0,
-			  layer_balloon_trigger, (XtPointer) "Hide all depths");
-    XtAddEventHandler(below, LeaveWindowMask, (Boolean) 0,
-			  layer_unballoon, (XtPointer) NULL);
+    XtAddEventHandler(below, EnterWindowMask, False,
+			   (XtEventHandler)layer_balloon_trigger, (XtPointer) "Hide all depths");
+    XtAddEventHandler(below, LeaveWindowMask, False,
+			   (XtEventHandler)layer_unballoon, (XtPointer) NULL);
+#endif /* XAW3D1_5E */
 
     FirstArg(XtNlabel, "Toggle ");
     NextArg(XtNfromVert, below);
@@ -167,13 +189,123 @@ init_depth_panel(parent)
     below = toggle_all_but = XtCreateManagedWidget("toggle_all", commandWidgetClass,
 				layer_form, Args, ArgCount);
     XtAddCallback(below, XtNcallback, (XtCallbackProc)toggle_all, (XtPointer) NULL);
+#ifndef XAW3D1_5E
     /* popup when mouse passes over button */
-    XtAddEventHandler(below, EnterWindowMask, (Boolean) 0,
-			  layer_balloon_trigger, (XtPointer) "Toggle displayed/hidden depths");
-    XtAddEventHandler(below, LeaveWindowMask, (Boolean) 0,
-			  layer_unballoon, (XtPointer) NULL);
+    XtAddEventHandler(below, EnterWindowMask, False,
+			   (XtEventHandler)layer_balloon_trigger, (XtPointer) "Toggle displayed/hidden depths");
+    XtAddEventHandler(below, LeaveWindowMask, False,
+			   (XtEventHandler)layer_unballoon, (XtPointer) NULL);
+#endif /* XAW3D1_5E */
+
+    /* radio buttons to switch between graying out inactive layers or blanking altogether */
+
+    /* radio for graying out */
+    /* we'll insert the checkmark bitmap in setup_depth_panel */
+    FirstArg(XtNwidth, sm_check_width+6);
+    NextArg(XtNheight, sm_check_height+6);
+    NextArg(XtNfromVert, below);
+    NextArg(XtNtop, XtChainTop);
+    NextArg(XtNbottom, XtChainTop);
+    NextArg(XtNleft, XtChainLeft);
+    NextArg(XtNright, XtChainLeft);
+    NextArg(XtNinternalWidth, 1);
+    NextArg(XtNinternalHeight, 1);
+    NextArg(XtNfont, bold_font);
+    NextArg(XtNlabel, "  ");
+    NextArg(XtNsensitive, (gray_layers? False : True));	/* make opposite button sens */
+    NextArg(XtNstate, gray_layers); 	/* initial state */
+    NextArg(XtNradioData, 1);		/* when this is pressed the value is 1 */
+    graytoggle = XtCreateManagedWidget("graytoggle", toggleWidgetClass,
+				layer_form, Args, ArgCount);
+    XtAddCallback(graytoggle, XtNcallback, (XtCallbackProc) switch_layer_mode,
+					(XtPointer) NULL);
+
+    /* label - "Gray" */
+
+    FirstArg(XtNborderWidth, 0);
+    NextArg(XtNfromVert, below);
+    NextArg(XtNfromHoriz, graytoggle);
+    NextArg(XtNtop, XtChainTop);
+    NextArg(XtNbottom, XtChainTop);
+    NextArg(XtNleft, XtChainRight);
+    NextArg(XtNright, XtChainRight);
+    NextArg(XtNlabel, "Gray ");
+    graylabel = below = XtCreateManagedWidget("graylabel", labelWidgetClass,
+				layer_form, Args, ArgCount);
+#ifndef XAW3D1_5E
+    /* popup when mouse passes over button */
+    XtAddEventHandler(below, EnterWindowMask, False,
+			   (XtEventHandler)layer_balloon_trigger, (XtPointer) "Display inactive layers in gray");
+    XtAddEventHandler(below, LeaveWindowMask, False,
+			   (XtEventHandler)layer_unballoon, (XtPointer) NULL);
+#endif /* XAW3D1_5E */
+
+    /* radio for blanking */
+    FirstArg(XtNwidth, sm_check_width+6);
+    NextArg(XtNheight, sm_check_height+6);
+    NextArg(XtNfromVert, graytoggle);
+    NextArg(XtNtop, XtChainTop);
+    NextArg(XtNbottom, XtChainTop);
+    NextArg(XtNleft, XtChainLeft);
+    NextArg(XtNright, XtChainLeft);
+    NextArg(XtNinternalWidth, 1);
+    NextArg(XtNinternalHeight, 1);
+    NextArg(XtNfont, bold_font);
+    NextArg(XtNlabel, "  ");
+    NextArg(XtNsensitive, (gray_layers? True : False));	/* make opposite button sens */
+    NextArg(XtNstate, !gray_layers);	/* initial state */
+    NextArg(XtNradioData, 2);		/* when this is pressed the value is 2 */
+    NextArg(XtNradioGroup, graytoggle);	/* this is the other radio button in the group */
+    blanktoggle = XtCreateManagedWidget("blanktoggle", toggleWidgetClass,
+				layer_form, Args, ArgCount);
+    XtAddCallback(blanktoggle, XtNcallback, (XtCallbackProc) switch_layer_mode,
+					(XtPointer) NULL);
+
+    /* label - "Blank" */
+
+    FirstArg(XtNborderWidth, 0);
+    NextArg(XtNfromVert, graytoggle);
+    NextArg(XtNfromHoriz, blanktoggle);
+    NextArg(XtNlabel, "Blank");
+    NextArg(XtNtop, XtChainTop);
+    NextArg(XtNbottom, XtChainTop);
+    NextArg(XtNleft, XtChainRight);
+    NextArg(XtNright, XtChainRight);
+    blanklabel = below = XtCreateManagedWidget("blanklabel", labelWidgetClass,
+				layer_form, Args, ArgCount);
+#ifndef XAW3D1_5E
+    /* popup when mouse passes over button */
+    XtAddEventHandler(below, EnterWindowMask, False,
+			   (XtEventHandler)layer_balloon_trigger, (XtPointer) "Blank inactive layers");
+    XtAddEventHandler(below, LeaveWindowMask, False,
+			   (XtEventHandler)layer_unballoon, (XtPointer) NULL);
+#endif /* XAW3D1_5E */
+
+    /* a form to hold the "Front" and "Back" labels and the viewport widget */
+
+    FirstArg(XtNborderWidth, 1);
+    NextArg(XtNfromVert, below);
+    NextArg(XtNdefaultDistance, 1);
+    NextArg(XtNtop, XtChainTop);
+    NextArg(XtNbottom, XtChainBottom);
+    NextArg(XtNleft, XtChainRight);
+    NextArg(XtNright, XtChainRight);
+    layer_viewform = XtCreateManagedWidget("layer_viewform",formWidgetClass,
+			layer_form, Args, ArgCount);     
+
+    /* label - "Front" */
+
+    FirstArg(XtNborderWidth, 0);
+    NextArg(XtNlabel, " Front ");
+    NextArg(XtNtop, XtChainTop);
+    NextArg(XtNbottom, XtChainTop);
+    NextArg(XtNleft, XtChainRight);
+    NextArg(XtNright, XtChainRight);
+    below = XtCreateManagedWidget("frontlabel", labelWidgetClass,
+				layer_viewform, Args, ArgCount);
+
+    /* viewport to be able to scroll the layer buttons */
 
-    /* viewport to be able to scroll the form with the layer buttons */
     FirstArg(XtNborderWidth, 1);
     NextArg(XtNwidth, LAYER_WD);
     NextArg(XtNallowVert, True);
@@ -184,42 +316,69 @@ init_depth_panel(parent)
     NextArg(XtNleft, XtChainLeft);
     NextArg(XtNright, XtChainRight);
     layer_viewp = XtCreateManagedWidget("layer_viewp", viewportWidgetClass, 
-			layer_form, Args, ArgCount);
+			layer_viewform, Args, ArgCount);
     /* popup when mouse passes over button */
-    XtAddEventHandler(layer_viewp, EnterWindowMask, (Boolean) 0,
-			  layer_balloon_trigger, (XtPointer) "Display or hide any depth");
-    XtAddEventHandler(layer_viewp, LeaveWindowMask, (Boolean) 0,
-			  layer_unballoon, (XtPointer) NULL);
-    XtAddEventHandler(layer_viewp, StructureNotifyMask, (Boolean) 0,
-			  update_layers, (XtPointer) NULL);
 
-    /* canvas (label, actually) to in which to create the buttons */
+    /* pass null pointer for trigger message so we'll use the two-line one */
+#ifndef XAW3D1_5E
+    XtAddEventHandler(layer_viewp, EnterWindowMask, False,
+			   (XtEventHandler)layer_balloon_trigger, (XtPointer) NULL);
+    XtAddEventHandler(layer_viewp, LeaveWindowMask, False,
+			   (XtEventHandler)layer_unballoon, (XtPointer) NULL);
+#endif /* XAW3D1_5E */
+    XtAddEventHandler(layer_viewp, StructureNotifyMask, False,
+			   (XtEventHandler)update_layers, (XtPointer) NULL);
+
+    /* canvas (label, actually) in which to create the buttons */
     /* we're not using real commandButtons because of the time to create
-       potentially hundreds of them depending on the layers in the figure */
+       potentially hundreds of them depending on the number of layers in the figure */
     FirstArg(XtNleft, XtChainLeft);
     NextArg(XtNright, XtChainRight);
     NextArg(XtNlabel, "");
-    layer_canvas = XtCreateManagedWidget("layer_canvas", labelWidgetClass, layer_viewp, 
-				Args, ArgCount);
+    layer_canvas = XtCreateManagedWidget("layer_canvas", labelWidgetClass,
+				layer_viewp, Args, ArgCount);
+    /* label - "Back" */
+
+    FirstArg(XtNborderWidth, 0);
+    NextArg(XtNfromVert, layer_viewp);
+    NextArg(XtNlabel, " Back  ");
+    NextArg(XtNtop, XtChainBottom);
+    NextArg(XtNbottom, XtChainBottom);
+    NextArg(XtNleft, XtChainRight);
+    NextArg(XtNright, XtChainRight);
+    below = XtCreateManagedWidget("backlabel", labelWidgetClass,
+				layer_viewform, Args, ArgCount);
+
     /* make actions/translations for user to click on a layer "button" and 
        to redraw buttons on expose */
     XtAppAddActions(tool_app, layer_actions, XtNumber(layer_actions));
     XtOverrideTranslations(layer_canvas,
 			   XtParseTranslationTable(layer_translations));
 
+#ifdef XAW3D1_5E
+    update_layerpanel();
+#endif /* XAW3D1_5E */
 }
 
 /* now that the mouse function and indicator panels have been sized
    correctly, resize our form */
 
 void
-setup_depth_panel()
+setup_depth_panel(void)
 {
-    Dimension	 ind_ht;
-    /* get height of indicator panel */
+    Dimension	 ind_ht, snap_ht=0;
+    /* get height of indicator and snap panels */
     FirstArg(XtNheight, &ind_ht);
     GetValues(ind_panel);
-    LAYER_HT = MOUSEFUN_HT + TOPRULER_HT + CANVAS_HT + ind_ht + INTERNAL_BW*4;
+    FirstArg(XtNheight, &snap_ht);
+    GetValues(snap_indicator_panel);
+    LAYER_HT = MOUSEFUN_HT + TOPRULER_HT + CANVAS_HT + ind_ht - snap_ht + INTERNAL_BW*4;
+
+    /* now that the bitmaps have been created, put the checkmark in the proper toggle */
+    FirstArg(XtNbitmap, (gray_layers? sm_check_pm : sm_null_check_pm));
+    SetValues(graytoggle);
+    FirstArg(XtNbitmap, (gray_layers? sm_null_check_pm : sm_check_pm));
+    SetValues(blanktoggle);
 
     XtUnmanageChild(layer_form);
     FirstArg(XtNheight, LAYER_HT);
@@ -227,7 +386,7 @@ setup_depth_panel()
     XtManageChild(layer_form);
 }
 
-#define B_WIDTH		40
+#define B_WIDTH		38
 #define B_BORDER	2
 #define C_SIZE		10
 #define TOT_B_HEIGHT	(C_SIZE+2*B_BORDER)
@@ -236,19 +395,21 @@ setup_depth_panel()
 /* get_min_max_depth() must have already been called */
 
 void
-update_layers()
+update_layers(void)
 {
     Window	 w = XtWindow(layer_canvas);
     int		 i, height;
 
-    if (preview_in_progress) return;  /* don't update depth panel when previewing */
+    /* don't update depth panel when previewing or reading in a Fig file */
+    if (preview_in_progress || defer_update_layers)
+	return;
 
     XClearWindow(tool_d, w);
     if (min_depth < 0) return;  /* if no objects, return */
 
     height = B_BORDER * 3;
     for (i = min_depth; i <= max_depth; i++) {
-      if (0 < object_depths[i]) height += TOT_B_HEIGHT;
+      if (object_depths[i] > 0) height += TOT_B_HEIGHT;
     }
     
     /* set height of canvas for all buttons */
@@ -259,7 +420,7 @@ update_layers()
 }
 
 static void
-draw_layer_buttons()
+draw_layer_buttons(void)
 {
     Window	 w = XtWindow(layer_canvas);
     int		 i;
@@ -268,7 +429,7 @@ draw_layer_buttons()
     if (min_depth < 0) return;  /* if no objects, return */
 
     for (i = min_depth; i <= max_depth; i++) {
-        if (0 < object_depths[i]) draw_layer_button(w, i);
+        if (object_depths[i] > 0) draw_layer_button(w, i);
     }
 }
 
@@ -277,9 +438,7 @@ draw_layer_buttons()
 	XDrawLine(tool_d, w, button_gc, x1, y1, x2, y2);
 
 static void
-draw_layer_button(win, but)
-    Window	 win;
-    int		 but;
+draw_layer_button(Window win, int but)
 {
     char	 str[20];
     int		 x, y, w, h, i;
@@ -320,17 +479,13 @@ draw_layer_button(win, but)
 
     /* now draw in the layer number for the button */
     sprintf(str, "%3d", but);
-    XDrawString(tool_d, win, button_gc, x+w+B_BORDER, y+w, str, strlen(str));
+    XDrawString(tool_d, win, button_gc, x+w+3*B_BORDER, y+w, str, strlen(str));
 }
 
 /* this is the callback to refresh the layer buttons that have been exposed */
 
-static XtActionProc
-layer_exposed(w, event, params, nparams)
-    Widget	    w;
-    XExposeEvent   *event;
-    String	   *params;
-    Cardinal	   *nparams;
+static void
+layer_exposed(Widget w, XExposeEvent *event, String *params, Cardinal *nparams)
 {
     int		    y, i;
 
@@ -338,7 +493,7 @@ layer_exposed(w, event, params, nparams)
 
     y = 0;
     for (i = min_depth; i <= max_depth; i++) {
-      if (0 < object_depths[i]) {
+      if (object_depths[i] > 0) {
 	y += TOT_B_HEIGHT;
 	if (event->y <= y && y - TOT_B_HEIGHT <= event->y + event->height)
 	  draw_layer_button(XtWindow(w), i);
@@ -351,14 +506,13 @@ layer_exposed(w, event, params, nparams)
 static int pressed_but = -1;
 
 static int
-calculate_pressed_depth(y)
-     int y;
+calculate_pressed_depth(int y)
 {
   int i, y1;
 
   y1 = TOT_B_HEIGHT;
   for (i = min_depth; i <= max_depth; i++) {
-    if (0 < object_depths[i]) {
+    if (object_depths[i] > 0) {
       if (y < y1) return i;
       y1 += TOT_B_HEIGHT;
     }
@@ -366,34 +520,58 @@ calculate_pressed_depth(y)
   return max_depth + 1;
 }
 
-static XtActionProc
-toggle_layer(w, event, params, nparams)
-    Widget	    w;
-    XButtonEvent   *event;
-    String	   *params;
-    Cardinal	   *nparams;
+/* user pressed right button on layer checkbox, set depth attribute to that layer */
+
+static void
+set_depth_to_layer(Widget w, XButtonEvent *event, String *params, Cardinal *nparams)
+{
+    int		    but;
+
+    but = calculate_pressed_depth(event->y);
+    if (but < min_depth || max_depth < but)
+	return;  /* no such button */
+    cur_depth = but;
+    show_depth(depth_button);
+}
+
+static void
+toggle_layer(Widget w, XButtonEvent *event, String *params, Cardinal *nparams)
 {
     Window	    win = XtWindow(layer_canvas);
-    int		    but;
+    int		    but, i;
+    Boolean	    obscure;
 
     if (min_depth < 0) return;  /* if no objects, return */
 
     but = calculate_pressed_depth(event->y);
-    if (but < min_depth || max_depth < but) return;  /* no such button */
+    if (but < min_depth || max_depth < but)
+	return;  /* no such button */
 
     /* yes, toggle visibility and redraw */
     active_layers[but] = !active_layers[but];
     draw_layer_button(win, but);
-    redisplay_canvas();
+    /* if user just turned on a layer, draw only it if there are no active layers on top */
+    if (active_layers[but]) {
+	obscure = False;
+	for (i=0; i<but; i++)
+	    if (active_layers[i]) {
+		obscure = True;
+		break;
+	    }
+	if (!obscure) {
+	    clearcounts();
+	    redisplay_compoundobject(&objects, but);
+	} else
+	    redisplay_canvas();
+    } else {
+	/* otherwise redraw whole canvas to get rid of that layer */
+	redisplay_canvas();
+    }
     pressed_but = but;
 }
 
-static XtActionProc
-sweep_layer(w, event, params, nparams)
-    Widget	    w;
-    XButtonEvent   *event;
-    String	   *params;
-    Cardinal	   *nparams;
+static void
+sweep_layer(Widget w, XButtonEvent *event, String *params, Cardinal *nparams)
 {
     Window	    win = XtWindow(layer_canvas);
     int		    but;
@@ -409,7 +587,7 @@ sweep_layer(w, event, params, nparams)
 
     if (pressed_but < but) {
       for (i = pressed_but + 1; i <= but; i++) {
-	if (0 < object_depths[i]
+	if (object_depths[i] > 0
 	    && active_layers[i] != active_layers[pressed_but]) {
 	  active_layers[i] = active_layers[pressed_but];
 	  draw_layer_button(win, i);
@@ -418,7 +596,7 @@ sweep_layer(w, event, params, nparams)
       }
     } else {
       for (i = pressed_but - 1; but <= i; i--) {
-	if (0 < object_depths[i]
+	if (object_depths[i] > 0
 	    && active_layers[i] != active_layers[pressed_but]) {
 	  active_layers[i] = active_layers[pressed_but];
 	  draw_layer_button(win, i);
@@ -431,20 +609,14 @@ sweep_layer(w, event, params, nparams)
 	redisplay_canvas();
 }
 
-static XtActionProc
-leave_layer(w, event, params, nparams)
-    Widget	    w;
-    XButtonEvent   *event;
-    String	   *params;
-    Cardinal	   *nparams;
+static void
+leave_layer(Widget w, XButtonEvent *event, String *params, Cardinal *nparams)
 {
     pressed_but = -1;
 }
 
-static XtCallbackProc
-all_active(w, closure, call_data)
-    Widget	    w;
-    XtPointer       closure, call_data;
+static void
+all_active(Widget w, XtPointer closure, XtPointer call_data)
 {
     int i;
     Boolean changed = False;
@@ -464,10 +636,8 @@ all_active(w, closure, call_data)
     }
 }
 
-static XtCallbackProc
-all_inactive(w, closure, call_data)
-    Widget	    w;
-    XtPointer       closure, call_data;
+static void
+all_inactive(Widget w, XtPointer closure, XtPointer call_data)
 {
     int i;
     Boolean changed = False;
@@ -487,10 +657,8 @@ all_inactive(w, closure, call_data)
     }
 }
 
-static XtCallbackProc
-toggle_all(w, closure, call_data)
-    Widget	    w;
-    XtPointer       closure, call_data;
+static void
+toggle_all(Widget w, XtPointer closure, XtPointer call_data)
 {
     int i;
 
@@ -503,18 +671,53 @@ toggle_all(w, closure, call_data)
     redisplay_canvas();
 }
 
-Boolean
-active_layer(layer)
-    int layer;
+/* when user toggles between gray-out and blank inactive layers */
+
+static void
+switch_layer_mode(Widget w, XtPointer closure, XtPointer call_data)
 {
-    return (active_layers[layer]);
+    Boolean	    state;
+    int		    which;
+
+    /* check state of the toggle and set/remove checkmark */
+    FirstArg(XtNstate, &state);
+    GetValues(w);
+    
+    if (state ) {
+	FirstArg(XtNbitmap, sm_check_pm);
+    } else {
+	FirstArg(XtNbitmap, sm_null_check_pm);
+    }
+    SetValues(w);
+
+    /* set the sensitivity of the toggle button to the opposite of its state
+       so that the user must press the other one now */
+    XtSetSensitive(w, !state);
+    /* and make the *other* button the opposite state */
+    if (w == graytoggle) {
+	XtSetSensitive(blanktoggle, state);
+    } else {
+	XtSetSensitive(graytoggle, state);
+    }
+    /* which button */
+    which = (int) XawToggleGetCurrent(w);
+    if (which == 0)		/* no buttons on, in transition so return now */
+	return;
+    if (which == 2)		/* "blank" button, invert state */
+	state = !state;
+
+    /* set global state */
+    gray_layers = state;
+
+    /* now simply redisplay everything */
+    clear_canvas();
+    redisplay_canvas();
 }
 
 /* return True if *any* object in the compound is in any active layer */
 
 Boolean
-any_active_in_compound(cmpnd)
-    F_compound	   *cmpnd;
+any_active_in_compound(F_compound *cmpnd)
 {
 	F_ellipse  *e;
 	F_arc	   *a;
@@ -544,21 +747,57 @@ any_active_in_compound(cmpnd)
 	return False;
 }
 
+#ifdef XAW3D1_5E
+void update_layerpanel()
+{
+    /*
+     * We must test for the widgets, as this is called by
+     * w_cmdpanel.c:refresh_view_menu().
+     */
+
+    if (all_active_but)
+	if (appres.showballoons)
+	    XawTipEnable(all_active_but, "Display all depths");
+	else
+	    XawTipDisable(all_active_but);
+    if (all_inactive_but)
+	if (appres.showballoons)
+	    XawTipEnable(all_inactive_but, "Hide all depths");
+	else
+	    XawTipDisable(all_inactive_but);
+    if (toggle_all_but)
+	if (appres.showballoons)
+	    XawTipEnable(toggle_all_but, "Toggle displayed/hidden depths");
+	else
+	    XawTipDisable(toggle_all_but);
+    if (layer_viewp)
+	if (appres.showballoons)
+	    XawTipEnable(layer_viewp, "Display or hide any depth");
+	else
+	    XawTipDisable(layer_viewp);
+    if (graylabel)
+	if (appres.showballoons)
+	    XawTipEnable(graylabel, "Display inactive layers in gray");
+	else
+	    XawTipDisable(graylabel);
+    if (blanklabel)
+	if (appres.showballoons)
+	    XawTipEnable(blanklabel, "Blank inactive layers");
+	else
+	    XawTipDisable(blanklabel);
+}
+#else
 static	Widget layer_balloon_popup = (Widget) 0;
 static	XtIntervalId balloon_id = (XtIntervalId) 0;
 static	Widget balloon_w;
 static	char *popmsg;
 
-static	void layer_balloon();
+static	void layer_balloon(Widget w, XtPointer closure, XtPointer call_data);
 
 /* come here when the mouse passes over a button in the depths panel */
 
 static void
-layer_balloon_trigger(widget, closure, event, continue_to_dispatch)
-    Widget        widget;
-    XtPointer	  closure;
-    XEvent*	  event;
-    Boolean*	  continue_to_dispatch;
+layer_balloon_trigger(Widget widget, XtPointer closure, XEvent *event, Boolean *continue_to_dispatch)
 {
 	if (!appres.showballoons)
 		return;
@@ -576,17 +815,14 @@ layer_balloon_trigger(widget, closure, event, continue_to_dispatch)
 /* come here when the timer times out (and the mouse is still over the button) */
 
 static void
-layer_balloon(w, closure, call_data)
-    Widget          w;
-    XtPointer	    closure;
-    XtPointer	    call_data;
+layer_balloon(Widget w, XtPointer closure, XtPointer call_data)
 {
 	Position  x, y;
 	Dimension wid, ht;
 	Widget box, balloon_label;
 	XtWidgetGeometry xtgeom,comp;
 
-	XtTranslateCoords(balloon_w, 0, 0, &x, &y);
+	XtTranslateCoords(w, 0, 0, &x, &y);
 	FirstArg(XtNx, x);
 	NextArg(XtNy, y);
 	layer_balloon_popup = XtCreatePopupShell("layer_balloon_popup",
@@ -598,11 +834,26 @@ layer_balloon(w, closure, call_data)
 	box = XtCreateManagedWidget("box", boxWidgetClass, 
 				layer_balloon_popup, Args, ArgCount);
 	
-	/* make label for message */
-	FirstArg(XtNborderWidth, 0);
-	NextArg(XtNlabel, popmsg);
-	balloon_label = XtCreateManagedWidget("label", labelWidgetClass,
-				    box, Args, ArgCount);
+	/* make label for mouse message */
+	/* if the message was passed via the callback */
+	if (popmsg) {
+		FirstArg(XtNborderWidth, 0);
+		NextArg(XtNlabel, popmsg);
+		balloon_label = XtCreateManagedWidget("l_label", labelWidgetClass,
+					    box, Args, ArgCount);
+	} else {
+		/* otherwise it is the two-line message with mouse indicators */
+		FirstArg(XtNborderWidth, 0);
+		NextArg(XtNleftBitmap, mouse_l);	/* bitmap of mouse with left button pushed */
+		NextArg(XtNlabel, "Display or hide depth    ");
+		balloon_label = XtCreateManagedWidget("l_label", labelWidgetClass,
+					    box, Args, ArgCount);
+		FirstArg(XtNborderWidth, 0);
+		NextArg(XtNleftBitmap, mouse_r);	/* bitmap of mouse with right button pushed */
+		NextArg(XtNlabel, "Set current depth to this");
+		(void) XtCreateManagedWidget("r_label", labelWidgetClass,
+					box, Args, ArgCount);
+	}
 
 	/* realize the popup so we can get its size */
 	XtRealizeWidget(layer_balloon_popup);
@@ -620,7 +871,7 @@ layer_balloon(w, closure, call_data)
 	    int wx, wy;	/* XTranslateCoordinates wants int, not Position */
 	    get_pointer_win_xy(&wx, &wy);
 	    xtgeom.request_mode |= CWY;
-	    xtgeom.y = y+wy - (int) ht;
+	    xtgeom.y = y+wy - (int) ht - 60;
 	}
 	(void) XtMakeGeometryRequest(layer_balloon_popup, &xtgeom, &comp);
 
@@ -632,11 +883,7 @@ layer_balloon(w, closure, call_data)
 /* come here when the mouse leaves a button in the layer panel */
 
 static void
-layer_unballoon(widget, closure, event, continue_to_dispatch)
-    Widget          widget;
-    XtPointer	    closure;
-    XEvent*	    event;
-    Boolean*	    continue_to_dispatch;
+layer_unballoon(Widget widget, XtPointer closure, XEvent *event, Boolean *continue_to_dispatch)
 {
     if (balloon_id) {
 	XtRemoveTimeOut(balloon_id);
@@ -647,15 +894,18 @@ layer_unballoon(widget, closure, event, continue_to_dispatch)
 	layer_balloon_popup = (Widget) 0;
     }
 }
+#endif /* XAW3D1_5E */
 
 /* map/unmap the depth manager on the right side */
 
 void
-toggle_show_depths()
+toggle_show_depths(void)
 {
     Dimension	wid, b;
 
     appres.showdepthmanager = !appres.showdepthmanager;
+    put_msg("%s depth manager",
+		appres.showdepthmanager? "Show": "Don't show");
     /* get actual width of layer form */
     FirstArg(XtNwidth, &wid);
     NextArg(XtNborderWidth, &b);
@@ -675,3 +925,69 @@ toggle_show_depths()
     /* and redraw the objects */
     redisplay_canvas();
 }
+
+void
+save_active_layers(void)
+{
+    int		 i;
+    for (i=0; i<=MAX_DEPTH; i++)
+	save_layers[i] = active_layers[i];
+}
+
+/* save depth counts and min/max depth */
+
+void
+save_depths(void)
+{
+    int		 i;
+    saved_min_depth = min_depth;
+    saved_max_depth = max_depth;
+    for (i=0; i<=MAX_DEPTH; i++)
+	saved_depths[i] = object_depths[i];
+}
+
+void
+save_counts(void)
+{
+    int		 i;
+    for (i=0; i<=MAX_DEPTH; i++)
+	saved_counts[i] = counts[i];
+}
+
+void
+save_counts_and_clear(void)
+{
+    save_counts();
+    clearallcounts();
+}
+
+void
+restore_active_layers(void)
+{
+    int		 i;
+    for (i=0; i<=MAX_DEPTH; i++)
+	active_layers[i] = save_layers[i];
+}
+
+/* restore depth counts and reset min/max depth */
+
+void
+restore_depths(void)
+{
+    int		 i;
+    min_depth = saved_min_depth;
+    max_depth = saved_max_depth;
+    for (i=0; i<=MAX_DEPTH; i++)
+	object_depths[i] = saved_depths[i];
+    /* refresh the layer manager */
+    update_layers();
+}
+
+void
+restore_counts(void)
+{
+    int		 i;
+    for (i=0; i<=MAX_DEPTH; i++)
+	 counts[i] = saved_counts[i];
+}
+
