@@ -1,142 +1,139 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 1989-2000 by Brian V. Smith
+ * Parts Copyright (c) 1989-2007 by Brian V. Smith
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
 #include "fig.h"
 #include "figx.h"
 #include "resources.h"
+#include "main.h"
 #include "mode.h"
 #include "object.h"
 #include "paintop.h"
+#include "d_arc.h"
+#include "d_arcbox.h"
+#include "d_box.h"
+#include "d_ellipse.h"
+#include "d_line.h"
+#include "d_picobj.h"
+#include "d_regpoly.h"
+#include "d_spline.h"
 #include "d_text.h"
+#include "e_chop.h"
 #include "e_flip.h"
 #include "e_rotate.h"
+#include "e_placelib.h"
+#include "e_addpt.h"
+#include "e_align.h"
+#include "e_arrow.h"
+#include "e_break.h"
+#include "e_compound.h"
+#include "e_convert.h"
+#include "e_copy.h"
+#include "e_delete.h"
+#include "e_deletept.h"
+#include "e_edit.h"
+#include "e_glue.h"
+#include "e_joinsplit.h"
+#include "e_measure.h"
+#include "e_move.h"
+#include "e_movept.h"
+#include "e_scale.h"
+#include "e_tangent.h"
+#include "e_update.h"
+#include "u_list.h"
+#include "u_markers.h"
 #include "u_search.h"
 #include "w_drawprim.h"
 #include "w_library.h"
 #include "w_indpanel.h"
-#include "w_util.h"
 #include "w_mousefun.h"
+#include "w_msgpanel.h"
+#include "w_modepanel.h"
 #include "w_setup.h"
+#include "w_util.h"
 #include "d_spline.h"
 
-/* it is easier to put these extern declarations here than to create 
-   all the d_*.h and e_*.h files to do it */
+/* EXPORTS */
 
-extern void	circlebyradius_drawing_selected();
-extern void	circlebydiameter_drawing_selected();
-extern void	ellipsebyradius_drawing_selected();
-extern void	ellipsebydiameter_drawing_selected();
-extern void	box_drawing_selected();
-extern void	arcbox_drawing_selected();
-extern void	line_drawing_selected();
-extern void	regpoly_drawing_selected();
-extern void	picobj_drawing_selected();
-extern void	text_drawing_selected();
-extern void	arc_drawing_selected();
-extern void	align_selected();
-extern void	compound_selected();
-extern void	open_compound();
-extern void	break_selected();
-extern void	join_split_selected();
-extern void	scale_selected();
-extern void	point_adding_selected();
-extern void	delete_point_selected();
-extern void	sel_place_lib_obj();
-extern void	move_selected();
-extern void	move_point_selected();
-extern void	delete_selected();
-extern void	copy_selected();
-extern void	rotate_cw_selected();
-extern void	rotate_ccw_selected();
-extern void	flip_ud_selected();
-extern void	flip_lr_selected();
-extern void	convert_selected();
-extern void	arrow_head_selected();
-extern void	edit_item_selected();
-extern void	update_selected();
+/* LOCALS */
 
-/* locals */
+/* drawing */
 
-static void	stub_circlebyradius_drawing_selected();
-static void	stub_circlebydiameter_drawing_selected();
-static void	stub_ellipsebyradius_drawing_selected();
-static void	stub_ellipsebydiameter_drawing_selected();
-static void	stub_box_drawing_selected();
-static void	stub_arcbox_drawing_selected();
-static void	stub_line_drawing_selected();
-static void	stub_poly_drawing_selected();
-static void	stub_regpoly_drawing_selected();
-static void	stub_picobj_drawing_selected();
-static void	stub_text_drawing_selected();
-static void	stub_arc_drawing_selected();
-static void	stub_spline_drawing_selected();
-static void	stub_cl_spline_drawing_selected();
-static void	stub_intspline_drawing_selected();
-static void	stub_cl_intspline_drawing_selected();
-static void	stub_align_selected();
-static void	stub_compound_selected();
-static void	stub_break_selected();
-static void	stub_open_compound_selected();
-static void	stub_join_split_selected();
-static void	stub_scale_selected();
-static void	stub_point_adding_selected();
-static void	stub_delete_point_selected();
-static void	stub_move_selected();
-static void	stub_popup_library();
-static void	stub_move_point_selected();
-static void	stub_delete_selected();
-static void	stub_copy_selected();
-static void	stub_rotate_cw_selected();
-static void	stub_rotate_ccw_selected();
-static void	stub_flip_ud_selected();
-static void	stub_flip_lr_selected();
-static void	stub_convert_selected();
-static void	stub_arrow_head_selected();
-static void	stub_edit_item_selected();
-static void	stub_update_selected();
-static void	stub_enter_mode_btn();
+static void	stub_circle_ellipse_byradius_drawing_selected(void);
+static void	stub_circle_ellipse_bydiameter_drawing_selected(void);
+static void	stub_box_drawing_selected(void);
+static void	stub_arcbox_drawing_selected(void);
+static void	stub_line_drawing_selected(void);
+static void	stub_poly_drawing_selected(void);
+static void	stub_regpoly_drawing_selected(void);
+static void	stub_picobj_drawing_selected(void);
+static void	stub_text_drawing_selected(void);
+static void	stub_arc_drawing_selected(void);
+static void	stub_spline_drawing_selected(void);
+static void	stub_cl_spline_drawing_selected(void);
+static void	stub_intspline_drawing_selected(void);
+static void	stub_cl_intspline_drawing_selected(void);
+
+/* editing */
+
+static void	stub_align_selected(void);
+static void	stub_compound_selected(void);
+static void	stub_break_selected(void);
+static void	stub_open_compound_selected(void);
+static void	stub_join_split_selected(void);
+static void	stub_chop_selected(void);
+static void	stub_scale_selected(void);
+static void	stub_point_adding_selected(void);
+static void	stub_delete_point_selected(void);
+static void	stub_move_selected(void);
+static void	stub_popup_library(void);
+static void	stub_move_point_selected(void);
+static void	stub_delete_selected(void);
+static void	stub_copy_selected(void);
+static void	stub_rotate_cw_selected(void);
+static void	stub_rotate_ccw_selected(void);
+static void	stub_flip_ud_selected(void);
+static void	stub_flip_lr_selected(void);
+static void	stub_convert_selected(void);
+static void	stub_chop_selected(void);
+static void	stub_arrow_head_selected(void);
+static void	stub_edit_item_selected(void);
+static void	stub_update_selected(void);
+static void	stub_enter_mode_btn(Widget widget, XEvent *event, String *params, Cardinal *num_params);
+static void	stub_anglemeas_selected(void);
+static void	stub_lenmeas_selected(void);
+static void	stub_areameas_selected(void);
+static void	stub_tangent_selected(void);
 
 /**************	    local variables and routines   **************/
 
-#define MAX_MODEMSG_LEN 80
-typedef struct mode_switch_struct {
-    icon_struct	   *icon;
-    int		    mode;
-    void	    (*setmode_func) ();
-    int		    objmask;
-    unsigned long   indmask;		/* mask to display indicators for this func */
-    char	    modemsg[MAX_MODEMSG_LEN];
-    Boolean	    popup;		/* true for commands that popup something */
-    Widget	    widget;
-    Pixmap	    pixmap, reversePM;
-}               mode_sw_info;
-
 DeclareStaticArgs(13);
 /* pointer to current mode switch */
 static mode_sw_info *current = NULL;
 
 /* button selection event handler */
-static void     sel_mode_but();
+static void     sel_mode_but(Widget widget, XtPointer closure, XEvent *event, Boolean *continue_to_dispatch);
 
+#ifndef XAW3D1_5E
 /* popup message over button when mouse enters it */
-static void     mode_balloon_trigger();
-static void     mode_unballoon();
+static void     mode_balloon_trigger(Widget widget, XtPointer closure, XEvent *event, Boolean *continue_to_dispatch);
+static void     mode_unballoon(Widget widget, XtPointer closure, XEvent *event, Boolean *continue_to_dispatch);
+#endif /* XAW3D1_5E */
 
 /* popdown message */
-static void     turn_on();
+static void     turn_on(mode_sw_info *msw);
 
 /* The M_XXX indicate which objects are selectable when the mode is on, and
    the I_XXX say which indicator buttons will appear on the indicator panel */
@@ -146,164 +143,140 @@ static void     turn_on();
    cases, the LeaveWindow event never happens on that button so the balloon popup
    would never be destroyed in that case.  */
 
-static mode_sw_info mode_switches[] = {
+mode_sw_info mode_switches[] = {
 
     /* DRAWING MODES */
 
-    {&cirrad_ic, F_CIRCLE_BY_RAD, circlebyradius_drawing_selected, M_NONE,
-       I_CIRCLE,
-       "CIRCLE drawing: specify RADIUS   (c)",
-       False},
-    {&cirdia_ic, F_CIRCLE_BY_DIA, circlebydiameter_drawing_selected, M_NONE,
-       I_CIRCLE,
-       "CIRCLE drawing: specify DIAMETER   (Shift-c)",
-       False},
-    {&ellrad_ic, F_ELLIPSE_BY_RAD, ellipsebyradius_drawing_selected, M_NONE,
+    {&ellrad_ic, F_CIRCLE_BY_RAD, circle_ellipse_byradius_drawing_selected, M_NONE,
        I_ELLIPSE,
-       "ELLIPSE drawing: specify RADII   (e)",
-       False},
-    {&elldia_ic, F_ELLIPSE_BY_DIA, ellipsebydiameter_drawing_selected, M_NONE,
+       "CIRCLE/ELLIPSE drawing: specify RADII   (c or e)", False},
+    {&elldia_ic, F_CIRCLE_BY_DIA, circle_ellipse_bydiameter_drawing_selected, M_NONE,
        I_ELLIPSE,
-       "ELLIPSE drawing: specify DIAMETERS   (Shift-e)",
-       False},
+       "CIRCLE/ELLIPSE drawing: specify DIAMETER(S)   (Shift-c or e)", False},
     {&c_spl_ic, F_CLOSED_APPROX_SPLINE, spline_drawing_selected, M_NONE,
        I_CLOSED,
-       "CLOSED APPROXIMATED SPLINE drawing: specify control points   (Shift-s)",
-       False},
+       "CLOSED APPROXIMATED SPLINE drawing: specify control points   (Shift-s)", False},
     {&spl_ic, F_APPROX_SPLINE, spline_drawing_selected, M_NONE,
        I_OPEN,
-       "APPROXIMATED SPLINE drawing: specify control points   (s)",
-       False},
+       "APPROXIMATED SPLINE drawing: specify control points   (s)", False},
     {&c_intspl_ic, F_CLOSED_INTERP_SPLINE, spline_drawing_selected, M_NONE,
        I_CLOSED,
-       "CLOSED INTERPOLATED SPLINE drawing: specify control points   (Shift-i)",
-       False},
+       "CLOSED INTERPOLATED SPLINE drawing: specify control points   (Shift-i)", False},
     {&intspl_ic, F_INTERP_SPLINE, spline_drawing_selected, M_NONE,
        I_OPEN,
-       "INTERPOLATED SPLINE drawing: specify control points   (i)",
-       False},
+       "INTERPOLATED SPLINE drawing: specify control points   (i)", False},
     {&polygon_ic, F_POLYGON, line_drawing_selected, M_NONE,
        I_CLOSED,
-       "POLYGON drawing   (p)",
-       False},
+       "POLYGON drawing   (p)", False},
     {&line_ic, F_POLYLINE, line_drawing_selected, M_NONE,
        I_LINE,
-       "POLYLINE drawing   (l)",
-       False},
+       "POLYLINE drawing   (l)", False},
     {&box_ic, F_BOX, box_drawing_selected, M_NONE,
        I_BOX,
-       "Rectangular BOX drawing   (b)",
-       False},
-    {&arc_box_ic, F_ARC_BOX, arcbox_drawing_selected, M_NONE,
+       "Rectangular BOX drawing   (b)", False},
+    {&arc_box_ic, F_ARCBOX, arcbox_drawing_selected, M_NONE,
        I_ARCBOX,
-       "Rectangular BOX drawing with ROUNDED CORNERS   (Shift-b)",
-       False},
+       "Rectangular BOX drawing with ROUNDED CORNERS   (Shift-b)", False},
     {&regpoly_ic, F_REGPOLY, regpoly_drawing_selected, M_NONE,
        I_REGPOLY,
-       "Regular Polygon   (Shift-p)",
-       False},
+       "Regular Polygon   (Shift-p)", False},
     {&arc_ic, F_CIRCULAR_ARC, arc_drawing_selected, M_NONE,
        I_ARC,
-       "ARC drawing: specify three points on the arc   (r)",
-       False},
+       "ARC drawing: specify three points on the arc   (r)", False},
     {&picobj_ic, F_PICOBJ, picobj_drawing_selected, M_NONE,
        I_PICOBJ,
-       "Picture Object   (Ctrl-p)",
-       True},		/* popups a panel */
+       "Picture Object   (Ctrl-p)", True},			/* popups a panel */
     {&text_ic, F_TEXT, text_drawing_selected, M_TEXT_NORMAL,
        I_TEXT,
-       "TEXT input from keyboard   (t)",
-       False},
+       "TEXT input from keyboard   (t)", False},
     {&library_ic, F_PLACE_LIB_OBJ, sel_place_lib_obj, M_NONE,
        I_MIN2,
-       "PLACE a library element   (Shift-l)",
-       True},		/* popups a panel */
+       "PLACE a library element   (Shift-l)", True},		/* popups a panel */
 
       /* EDITING MODES FOLLOW */
 
     {&glue_ic, F_GLUE, compound_selected, M_ALL,
        I_MIN2,
-       "GLUE objects into COMPOUND object   (g)",
-       False},
+       "GLUE objects into COMPOUND object   (g)", False},
     {&break_ic, F_BREAK, break_selected, M_COMPOUND,
        0,
-       "BREAK COMPOUND object   (Shift-g)",
-       False},
-    {&open_comp_ic, F_ENTER_COMP, open_compound, M_COMPOUND,
+       "BREAK COMPOUND object   (Shift-g)", False},
+    {&open_comp_ic, F_ENTER_COMP, open_compound_selected, M_COMPOUND,
        0,
-       "OPEN COMPOUND object   (o)",
-       False},
+       "OPEN COMPOUND object   (o)", False},
     {&join_split_ic, F_JOIN, join_split_selected, M_POLYLINE | M_SPLINE_O |M_SPLINE_C,
        0,
-       "Join or Split lines/splines   (j)",
-       False},
+       "Join or Split lines/splines   (j)", False},
+    {&chop_ic, F_CHOP, chop_selected, M_POLYLINE | M_ARC |M_ELLIPSE,  /* maybe spline someday */
+       I_CHOP,
+       "Chop objects   (x)", False},
     {&scale_ic, F_SCALE, scale_selected, M_NO_TEXT,
        I_MIN2,
-       "SCALE objects   (Ctrl-s)",
-       False},
+       "SCALE objects   (Ctrl-s)", False},
     {&align_ic, F_ALIGN, align_selected, M_COMPOUND,
        I_ALIGN,
-       "ALIGN objects within a COMPOUND or to CANVAS   (a)",
-       False},
+       "ALIGN objects within a COMPOUND or to CANVAS   (a)", False},
     {&movept_ic, F_MOVE_POINT, move_point_selected, M_NO_TEXT,
        I_ADDMOVPT,
-       "MOVE POINTs   (Shift-m)",
-       False},
+       "MOVE POINTs   (Shift-m)", False},
     {&move_ic, F_MOVE, move_selected, M_ALL,
        I_MIN3,
-       "MOVE objects   (m)",
-       False},
+       "MOVE objects   (m)", False},
     {&addpt_ic, F_ADD_POINT, point_adding_selected, M_VARPTS_OBJECT,
        I_ADDMOVPT,
-       "ADD POINTs to lines, polygons and splines   (Ctrl-a)",
-       False},
+       "ADD POINTs to lines, polygons and splines   (Ctrl-a)", False},
     {&copy_ic, F_COPY, copy_selected, M_ALL,
        I_COPY,
-       "COPY objects  (Ctrl-c)",
-       False},
+       "COPY objects  (Ctrl-c)", False},
     {&deletept_ic, F_DELETE_POINT, delete_point_selected, M_VARPTS_OBJECT,
        0,
-       "DELETE POINTs from lines, polygons and splines   (Shift-d)",
-       False},
+       "DELETE POINTs from lines, polygons and splines   (Shift-d)", False},
     {&delete_ic, F_DELETE, delete_selected, M_ALL,
        0,
-       "DELETE objects   (d)",
-       False},
+       "DELETE objects   (d)", False},
     {&update_ic, F_UPDATE, update_selected, M_ALL,
        I_OBJECT,
-       "UPDATE object <-> current settings   (u)",
-       False},
+       "UPDATE object <-> current settings   (u)", False},
     {&edit_ic, F_EDIT, edit_item_selected, M_ALL,
        0,
-       "CHANGE OBJECT via EDIT panel   (Ctrl-e)",
-       False},
+       "CHANGE OBJECT via EDIT panel   (Ctrl-e)", False},
     {&flip_y_ic, F_FLIP, flip_ud_selected, M_NO_TEXT,
        I_MIN2,
-       "FLIP objects up or down   (Shift-f)",
-       False},
+       "FLIP objects up or down   (f)", False},
     {&flip_x_ic, F_FLIP, flip_lr_selected, M_NO_TEXT,
        I_MIN2,
-       "FLIP objects left or right   (f)",
-       False},
+       "FLIP objects left or right   (Shift-f)", False},
     {&rotCW_ic, F_ROTATE, rotate_cw_selected, M_ALL,
        I_ROTATE,
-       "ROTATE objects clockwise   (Ctrl-r)",
-       False},
+       "ROTATE objects clockwise   (Ctrl-r)", False},
     {&rotCCW_ic, F_ROTATE, rotate_ccw_selected, M_ALL,
        I_ROTATE,
-       "ROTATE objects counter-clockwise   (Shift-r)",
-       False},
+       "ROTATE objects counter-clockwise   (Shift-r)", False},
     {&convert_ic, F_CONVERT, convert_selected, M_VARPTS_OBJECT | M_POLYLINE_BOX, 
        0,
-       "CONVERSION between lines, polygons and splines   (v)",
-       False},
+       "CONVERSION between lines, polygons and splines   (v)", False},
     {&autoarrow_ic, F_AUTOARROW, arrow_head_selected, M_OPEN_OBJECT,
        I_ADD_DEL_ARROW,
-       "ADD/DELETE ARROWHEADS   (Shift-a)",
-       False},
+       "ADD/DELETE ARROWHEADS   (Shift-a)", False},
+    {&tangent_ic, F_TANGENT, tangent_selected, M_TANGENT_OBJECT,
+       I_TANGENT,
+       "Add TANGENT/NORMAL to curve   (n)", False},
+    {&anglemeas_ic, F_ANGLEMEAS, anglemeas_selected, M_ANGLEMEAS_OBJECT,
+       I_MIN2,
+       "MEASURE angle (specify three points or select object)   (Ctrl-g)", False},
+    {&lenmeas_ic, F_LENMEAS, lenmeas_selected, M_LENMEAS_OBJECT,
+       I_MIN2,
+       "Measure LENGTH of polylines, arcs and ellipses   (Ctrl-n)", False},
+    {&areameas_ic, F_AREAMEAS, areameas_selected, M_AREAMEAS_OBJECT,
+       I_MIN2,
+       "Measure AREA of polygons, arcs and ellipses   (Ctrl-m)", False},
+
+    /* This must be last for create_mode_panel() (in w_canvas.c) */
+    { NULL, 0 }
+
 };
 
-int	NUM_MODE_SW = (sizeof(mode_switches) / sizeof(mode_sw_info));
+int	NUM_MODE_SW = (sizeof(mode_switches) / sizeof(mode_sw_info)) - 1;
 
 static Arg      button_args[] =
 {
@@ -319,12 +292,10 @@ static Arg      button_args[] =
      /* 6 */ {XtNbackgroundPixmap, (XtArgVal) NULL},
 };
 
+
+
 static void
-stub_enter_mode_btn(widget, event, params, num_params)
-    Widget	 widget;
-    XEvent	*event;
-    String	*params;
-    Cardinal	*num_params;
+stub_enter_mode_btn(Widget widget, XEvent *event, String *params, Cardinal *num_params)
 {
     draw_mousefun_mode();
 }
@@ -337,10 +308,8 @@ static XtActionsRec mode_actions[] =
     {"ReleaseMiddle", (XtActionProc) clear_middle},
     {"PressRight", (XtActionProc) notused_right},
     {"ReleaseRight", (XtActionProc) clear_right},
-    {"ModeCircleR", (XtActionProc) stub_circlebyradius_drawing_selected},
-    {"ModeCircleD", (XtActionProc) stub_circlebydiameter_drawing_selected},
-    {"ModeEllipseR", (XtActionProc) stub_ellipsebyradius_drawing_selected},
-    {"ModeEllipseD", (XtActionProc) stub_ellipsebydiameter_drawing_selected},
+    {"ModeEllipseR", (XtActionProc) stub_circle_ellipse_byradius_drawing_selected},
+    {"ModeEllipseD", (XtActionProc) stub_circle_ellipse_bydiameter_drawing_selected},
     {"ModeBox", (XtActionProc) stub_box_drawing_selected},
     {"ModeArcBox", (XtActionProc) stub_arcbox_drawing_selected},
     {"ModeLine", (XtActionProc) stub_line_drawing_selected},
@@ -371,9 +340,14 @@ static XtActionsRec mode_actions[] =
     {"ModeFlipObjectUD", (XtActionProc) stub_flip_ud_selected},
     {"ModeFlipObjectLR", (XtActionProc) stub_flip_lr_selected},
     {"ModeConvertObject", (XtActionProc) stub_convert_selected},
+    {"ModeChopObject", (XtActionProc) stub_chop_selected},
     {"ModeArrow", (XtActionProc) stub_arrow_head_selected},
     {"ModeEditObject", (XtActionProc) stub_edit_item_selected},
     {"ModeUpdateObject", (XtActionProc) stub_update_selected},
+    {"ModeTangent", (XtActionProc) stub_tangent_selected},
+    {"ModeAnglemeas", (XtActionProc) stub_anglemeas_selected},
+    {"ModeLenmeas", (XtActionProc) stub_lenmeas_selected},
+    {"ModeAreameas", (XtActionProc) stub_areameas_selected},
 };
 
 static String   mode_translations =
@@ -386,9 +360,8 @@ static String   mode_translations =
     <Btn3Up>:ReleaseRight()\n\
     <LeaveWindow>:LeaveModeSw()unhighlight()\n";
 
-int
-init_mode_panel(tool)
-    Widget           tool;
+void
+init_mode_panel(Widget tool)
 {
     register int    i;
     register mode_sw_info *sw;
@@ -419,14 +392,14 @@ init_mode_panel(tool)
 	    NextArg(XtNborderWidth, 0);
 	    NextArg(XtNresize, False);
 	    NextArg(XtNheight, (MODEPANEL_SPACE + 1) / 2);
-	    NextArg(XtNlabel, "Drawing\n modes");
+	    NextArg(XtNlabel, "Drawing");
 	    d_label = XtCreateManagedWidget("label", labelWidgetClass,
 					    mode_panel, Args, ArgCount);
 	} else if (sw->mode == FIRST_EDIT_MODE) {
 	    /* assume Args still set up from d_label */
 	    ArgCount -= 2;
 	    NextArg(XtNheight, (MODEPANEL_SPACE) / 2);
-	    NextArg(XtNlabel, "Editing\n modes");
+	    NextArg(XtNlabel, "Editing");
 	    e_label = XtCreateManagedWidget("label", labelWidgetClass,
 					    mode_panel, Args, ArgCount);
 	}
@@ -436,16 +409,23 @@ init_mode_panel(tool)
 			    mode_panel, button_args, XtNumber(button_args));
 
 	/* left button changes mode */
-	XtAddEventHandler(sw->widget, ButtonPressMask, (Boolean) 0,
+	XtAddEventHandler(sw->widget, ButtonPressMask, False,
 			  sel_mode_but, (XtPointer) sw);
+#ifndef XAW3D1_5E
 	/* popup when mouse passes over button */
-	XtAddEventHandler(sw->widget, EnterWindowMask, (Boolean) 0,
+	XtAddEventHandler(sw->widget, EnterWindowMask, False,
 			  mode_balloon_trigger, (XtPointer) sw);
-	XtAddEventHandler(sw->widget, LeaveWindowMask, (Boolean) 0,
+	XtAddEventHandler(sw->widget, LeaveWindowMask, False,
 			  mode_unballoon, (XtPointer) sw);
+#endif /* XAW3D1_5E */
 	XtOverrideTranslations(sw->widget,
 			       XtParseTranslationTable(mode_translations));
     }
+
+#ifdef XAW3D1_5E
+    update_modepanel();
+#endif /* XAW3D1_5E */
+
     return;
 }
 
@@ -453,7 +433,7 @@ init_mode_panel(tool)
  * after panel widget is realized (in main) put some bitmaps etc. in it
  */
 
-setup_mode_panel()
+void setup_mode_panel(void)
 {
     register int    i;
     register mode_sw_info *msw;
@@ -491,6 +471,28 @@ setup_mode_panel()
     SetValues(mode_panel);
 }
 
+#ifdef XAW3D1_5E
+void update_modepanel()
+{
+    register int    i;
+    register mode_sw_info *sw;
+
+    /*
+     * We must test for the widgets, as this is called by
+     * w_cmdpanel.c:refresh_view_menu().
+     */
+
+    for (i = 0; i < NUM_MODE_SW; ++i) {
+	sw = &mode_switches[i];
+	if (!sw->widget)
+	    continue;
+	if (appres.showballoons) {
+	    XawTipEnable(sw->widget, sw->modemsg);
+	} else
+	    XawTipDisable(sw->widget);
+    }
+}
+#else
 /* come here when the mouse passes over a button in the mode panel */
 
 static	Widget mode_balloon_popup = (Widget) 0;
@@ -498,14 +500,10 @@ static	XtIntervalId balloon_id = (XtIntervalId) 0;
 static	Widget balloon_w;
 static	XtPointer clos;
 
-static void mode_balloon();
+static void mode_balloon(void);
 
 static void
-mode_balloon_trigger(widget, closure, event, continue_to_dispatch)
-    Widget        widget;
-    XtPointer	  closure;
-    XEvent*	  event;
-    Boolean*	  continue_to_dispatch;
+mode_balloon_trigger(Widget widget, XtPointer closure, XEvent *event, Boolean *continue_to_dispatch)
 {
 	if (!appres.showballoons)
 		return;
@@ -520,7 +518,7 @@ mode_balloon_trigger(widget, closure, event, continue_to_dispatch)
 }
 
 static void
-mode_balloon()
+mode_balloon(void)
 {
 	Position  x, y;
 	mode_sw_info *msw = (mode_sw_info *) clos;
@@ -567,11 +565,7 @@ mode_balloon()
 /* come here when the mouse leaves a button in the mode panel */
 
 static void
-mode_unballoon(widget, closure, event, continue_to_dispatch)
-    Widget          widget;
-    XtPointer	    closure;
-    XEvent*	    event;
-    Boolean*	    continue_to_dispatch;
+mode_unballoon(Widget widget, XtPointer closure, XEvent *event, Boolean *continue_to_dispatch)
 {
     if (balloon_id) 
 	XtRemoveTimeOut(balloon_id);
@@ -581,15 +575,12 @@ mode_unballoon(widget, closure, event, continue_to_dispatch)
 	mode_balloon_popup = (Widget) 0;
     }
 }
+#endif /* XAW3D1_5E */
 
 /* come here when a button is pressed in the mode panel */
 
 static void
-sel_mode_but(widget, closure, event, continue_to_dispatch)
-    Widget          widget;
-    XtPointer	    closure;
-    XEvent*	    event;
-    Boolean*	    continue_to_dispatch;
+sel_mode_but(Widget widget, XtPointer closure, XEvent *event, Boolean *continue_to_dispatch)
 {
     XButtonEvent    xbutton;
     mode_sw_info    *msw = (mode_sw_info *) closure;
@@ -609,45 +600,46 @@ sel_mode_but(widget, closure, event, continue_to_dispatch)
     else if (highlighting)
 	erase_objecthighlight();
 
+#ifndef XAW3D1_5E
     /* if this command popups a window, destroy the balloon popup now. See the
        note above about this above the command panel definition. */
     if (msw->popup) {
 	mode_unballoon((Widget) 0, (XtPointer) 0, (XEvent*) 0, (Boolean*) 0);
     }
     app_flush();
+#endif /* XAW3D1_5E */
 
     if (xbutton.button == Button1) {	/* left button */
 	turn_off_current();
 	turn_on(msw);
-	if (msw->mode == F_UPDATE) {	/* map the set/clr/toggle button for update */
-	    if (cur_mode != F_UPDATE) {
+
 		update_indpanel(0);	/* first remove ind buttons */
 		XtUnmanageChild(ind_panel);
+	if (msw->mode != F_UPDATE)
+	    XtUnmanageChild(upd_ctrl);
+	FirstArg(XtNwidth, 1);	/* force a resize to update scrollbar */
+		SetValues(ind_panel);
+	if (msw->mode == F_UPDATE) {
+	    if (cur_mode != F_UPDATE) {
+		/* map the set/clr/toggle button for update */
 		XtManageChild(upd_ctrl);
-		/* get the width of the update control panel */
-		/* now put the ind_panel to our right */
-		FirstArg(XtNfromHoriz, upd_ctrl);
-		NextArg(XtNwidth, INDPANEL_WD-UPD_CTRL_WD-2*INTERNAL_BW); /* resize it */
-		SetValues(ind_panel);
-		XtManageChild(ind_panel);
-		update_indpanel(msw->indmask);	/* now manage the relevant buttons */
 	    }
-	} else { 	/* turn off the update boxes if not in update mode */
-	    if (cur_mode == F_UPDATE) {	/* if previous mode is update and current */
-		update_indpanel(0);	/* is not, first remove ind buttons */
+	    /* set the adjacent widget and width */
+	    FirstArg(XtNfromHoriz, upd_ctrl);
+	    NextArg(XtNwidth, INDPANEL_WD-UPD_CTRL_WD-2*INTERNAL_BW);
+	} else {
+	    if (cur_mode == F_UPDATE) {
+		/* unmap the set/clr/toggle button for update */
 		unmanage_update_buts();
-		XtUnmanageChild(ind_panel);
-		XtUnmanageChild(upd_ctrl);
-		/* now put the ind_panel to the right of the canvas */
+	    }
+	    /* clear the adjacent widget and set width */
 		FirstArg(XtNfromHoriz, NULL);
-		NextArg(XtNwidth, INDPANEL_WD);	/* resize it */
+	    NextArg(XtNwidth, INDPANEL_WD);
+	}
 		SetValues(ind_panel);
 		XtManageChild(ind_panel);
 		update_indpanel(msw->indmask);	/* now manage the relevant buttons */
-	    } else {
-		update_indpanel(msw->indmask);	/* just update indicator buttons */
-	    }
-	}
+
 	put_msg(msw->modemsg);
 	if ((cur_mode == F_GLUE || cur_mode == F_BREAK) &&
 	    msw->mode != F_GLUE && msw->mode != F_BREAK) {
@@ -658,56 +650,58 @@ sel_mode_but(widget, closure, event, continue_to_dispatch)
 	cur_mode = msw->mode;
 	anypointposn = !(msw->indmask & I_POINTPOSN);
 	new_objmask = msw->objmask;
-	if (cur_mode == F_ROTATE && cur_rotnangle != 90)
-	    new_objmask = M_ROTATE_ANGLE;
+	if ((cur_mode == F_ROTATE) && 
+	    fabs(cur_rotnangle) != 90.0 && fabs(cur_rotnangle) != 180.0)
+		new_objmask = M_ROTATE_ANGLE;
 	update_markers(new_objmask);
 	current = msw;
+	/* reset the "first length message" flag in case show_length is on */
+	first_lenmsg = True;
 	/* call the mode function */
 	msw->setmode_func();
     }
 }
 
 void
-force_positioning()
+force_positioning(void)
 {
     update_indpanel(current->indmask | I_POINTPOSN);
     anypointposn = 0;
 }
 
 void
-force_nopositioning()
+force_nopositioning(void)
 {
     update_indpanel(current->indmask & ~I_POINTPOSN);
     anypointposn = 1;
 }
 
 void
-force_anglegeom()
+force_anglegeom(void)
 {
     update_indpanel(current->indmask | I_ANGLEGEOM);
 }
 
 void
-force_noanglegeom()
+force_noanglegeom(void)
 {
     update_indpanel(current->indmask & ~I_ANGLEGEOM);
 }
 
 static void
-turn_on(msw)
-    mode_sw_info   *msw;
+turn_on(mode_sw_info *msw)
 {
     FirstArg(XtNbackgroundPixmap, msw->reversePM);
     SetValues(msw->widget);
 }
 
-turn_on_current()
+void turn_on_current(void)
 {
     if (current)
 	turn_on(current);
 }
 
-turn_off_current()
+void turn_off_current(void)
 {
     if (current) {
 	XtOverrideTranslations(current->widget,
@@ -717,8 +711,7 @@ turn_off_current()
     }
 }
 
-change_mode(icon)
-icon_struct *icon;
+void change_mode(icon_struct *icon)
 {
     int i;
     XButtonEvent ev; /* To fake an event with */
@@ -726,7 +719,7 @@ icon_struct *icon;
     ev.button = Button1;
     for (i = 0; i < NUM_MODE_SW; ++i)
 	if (mode_switches[i].icon == icon) {
-	    sel_mode_but(0,&mode_switches[i],&ev,0);
+	    sel_mode_but(0,(XtPointer)&mode_switches[i],(XEvent*)&ev,0);
 	    break;
 	}
     /* force update of mouse function window */
@@ -734,223 +727,244 @@ icon_struct *icon;
 }
 
 static void
-stub_circlebyradius_drawing_selected()
-{
-	change_mode(&cirrad_ic);
-}
-
-static void
-stub_circlebydiameter_drawing_selected()
-{
-	change_mode(&cirdia_ic);
-}
-
-static void
-stub_ellipsebyradius_drawing_selected()
+stub_circle_ellipse_byradius_drawing_selected(void)
 {
 	change_mode(&ellrad_ic);
 }
 
 static void
-stub_ellipsebydiameter_drawing_selected()
+stub_circle_ellipse_bydiameter_drawing_selected(void)
 {
 	change_mode(&elldia_ic);
 }
 
 static void
-stub_box_drawing_selected()
+stub_box_drawing_selected(void)
 {
 	change_mode(&box_ic);
 }
 
 static void
-stub_arcbox_drawing_selected()
+stub_arcbox_drawing_selected(void)
 {
 	change_mode(&arc_box_ic);
 }
 
 static void
-stub_poly_drawing_selected()
+stub_poly_drawing_selected(void)
 {
 	change_mode(&polygon_ic);
 }
 
 static void
-stub_line_drawing_selected()
+stub_line_drawing_selected(void)
 {
 	change_mode(&line_ic);
 }
 
 static void
-stub_regpoly_drawing_selected()
+stub_regpoly_drawing_selected(void)
 {
 	change_mode(&regpoly_ic);
 }
 
 static void
-stub_picobj_drawing_selected()
+stub_picobj_drawing_selected(void)
 {
 	change_mode(&picobj_ic);
 }
 
 static void
-stub_text_drawing_selected()
+stub_text_drawing_selected(void)
 {
 	change_mode(&text_ic);
 }
 
 static void
-stub_arc_drawing_selected()
+stub_arc_drawing_selected(void)
 {
 	change_mode(&arc_ic);
 }
 
 static void
-stub_cl_spline_drawing_selected()
+stub_cl_spline_drawing_selected(void)
 {
 	change_mode(&c_spl_ic);
 }
 
 static void
-stub_spline_drawing_selected()
+stub_spline_drawing_selected(void)
 {
 	change_mode(&spl_ic);
 }
 
 static void
-stub_cl_intspline_drawing_selected()
+stub_cl_intspline_drawing_selected(void)
 {
 	change_mode(&c_intspl_ic);
 }
 
 static void
-stub_intspline_drawing_selected()
+stub_intspline_drawing_selected(void)
 {
 	change_mode(&intspl_ic);
 }
 
 static void
-stub_align_selected()
+stub_align_selected(void)
 {
 	change_mode(&align_ic);
 }
 
 static void
-stub_compound_selected()
+stub_compound_selected(void)
 {
 	change_mode(&glue_ic);
 }
 
 static void
-stub_break_selected()
+stub_break_selected(void)
 {
 	change_mode(&break_ic);
 }
 
 static void
-stub_join_split_selected()
+stub_join_split_selected(void)
 {
 	change_mode(&join_split_ic);
 }
 
 static void
-stub_open_compound_selected()
+stub_open_compound_selected(void)
 {
 	change_mode(&open_comp_ic);
 }
 
 static void
-stub_scale_selected()
+stub_scale_selected(void)
 {
 	change_mode(&scale_ic);
 }
 
 static void
-stub_point_adding_selected()
+stub_point_adding_selected(void)
 {
 	change_mode(&addpt_ic);
 }
 
 static void
-stub_delete_point_selected()
+stub_delete_point_selected(void)
 {
 	change_mode(&deletept_ic);
 }
 
 static void
-stub_move_selected()
+stub_move_selected(void)
 {
 	change_mode(&move_ic);
 }
 static void
-stub_popup_library()
+stub_popup_library(void)
 {
 	change_mode(&library_ic);
 }
 
 
 static void
-stub_move_point_selected()
+stub_move_point_selected(void)
 {
 	change_mode(&movept_ic);
 }
 
 static void
-stub_delete_selected()
+stub_delete_selected(void)
 {
 	change_mode(&delete_ic);
 }
 
 static void
-stub_copy_selected()
+stub_copy_selected(void)
 {
 	change_mode(&copy_ic);
 }
 
 static void
-stub_rotate_cw_selected()
+stub_rotate_cw_selected(void)
 {
 	change_mode(&rotCW_ic);
 }
 
 static void
-stub_rotate_ccw_selected()
+stub_rotate_ccw_selected(void)
 {
 	change_mode(&rotCCW_ic);
 }
 
 static void
-stub_flip_ud_selected()
+stub_flip_ud_selected(void)
 {
 	change_mode(&flip_y_ic);
 }
 
 static void
-stub_flip_lr_selected()
+stub_flip_lr_selected(void)
 {
 	change_mode(&flip_x_ic);
 }
 
 static void
-stub_convert_selected()
+stub_convert_selected(void)
 {
 	change_mode(&convert_ic);
 }
 
 static void
-stub_arrow_head_selected()
+stub_chop_selected(void)
+{
+	change_mode(&chop_ic);
+}
+
+static void
+stub_arrow_head_selected(void)
 {
 	change_mode(&autoarrow_ic);
 }
 
 static void
-stub_edit_item_selected()
+stub_edit_item_selected(void)
 {
 	change_mode(&edit_ic);
 }
 
 static void
-stub_update_selected()
+stub_update_selected(void)
 {
 	change_mode(&update_ic);
 }
+
+static void
+stub_tangent_selected(void)
+{
+	change_mode(&tangent_ic);
+}
+
+static void
+stub_anglemeas_selected(void)
+{
+	change_mode(&anglemeas_ic);
+}
+
+static void
+stub_lenmeas_selected(void)
+{
+	change_mode(&lenmeas_ic);
+}
+
+static void
+stub_areameas_selected(void)
+{
+	change_mode(&areameas_ic);
+}
+
+
+
