@@ -1,16 +1,16 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1997 by T. Sato
- * Parts Copyright (c) 1997-2000 by Brian V. Smith
+ * Parts Copyright (c) 1997-2007 by Brian V. Smith
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish distribute, sublicense and/or sell copies of
+ * the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
@@ -45,15 +45,24 @@ There is currently no way to undo replace/update operations.
 #include "resources.h"
 #include "object.h"
 #include "d_text.h"
-#include "e_edit.h"
 #include "e_update.h"
 #include "f_util.h"
+#include "w_drawprim.h"
+#include "w_indpanel.h"
+#include "w_listwidget.h"
+#include "w_msgpanel.h"
+#include "w_srchrepl.h"
 #include "w_setup.h"
 #include "w_util.h"
-#include "w_drawprim.h"
-#include "w_listwidget.h"
-#include "w_srchrepl.h"
 #include "u_create.h"
+
+#include "mode.h"
+#include "u_bound.h"
+#include "u_fonts.h"
+#include "u_redraw.h"
+#include "w_canvas.h"
+#include "w_color.h"
+
 #include <stdarg.h>
 
 #define MAX_MISSPELLED_WORDS	200
@@ -61,29 +70,33 @@ There is currently no way to undo replace/update operations.
 
 static String search_panel_translations =
         "<Message>WM_PROTOCOLS: QuitSearchPanel()\n";
-static String found_text_panel_translations =
-        "<Message>WM_PROTOCOLS: QuitFoundTextPanel()\n";
 static String spell_panel_translations =
         "<Message>WM_PROTOCOLS: QuitSpellPanel()\n";
+
 static String spell_text_translations =
-	"<Key>Return: Correct()\n";
+	"<Key>Return: Correct()\n\
+	Meta<Key>Q: QuitSpellPanel()\n\
+	<Key>Escape: QuitSpellPanel()\n";
 String  search_text_translations =
 	"<Key>Return: SearchText()";
 String  replace_text_translations =
 	"<Key>Return: ReplaceText()";
+String	search_results_translations = 
+	"<Btn4Down>:	scroll-one-line-up()\n\
+	<Btn5Down>:	scroll-one-line-down()";
 
-static void	search_panel_dismiss();
-static void	search_and_replace_text();
-static Boolean	search_text_in_compound();
-static Boolean	replace_text_in_compound();
-static void	found_text_panel_dismiss();
-static void	do_replace();
+static void	search_panel_dismiss(Widget widget, XtPointer closure, XtPointer call_data);
+static void	search_and_replace_text(Widget widget, XtPointer closure, XtPointer call_data);
+static Boolean	search_text_in_compound(F_compound *com, char *pattern, void (*proc) (/* ??? */));
+static Boolean	replace_text_in_compound(F_compound *com, char *pattern, char *dst);
+static void	found_text_panel_dismiss(void);
+static void	do_replace(Widget widget, XtPointer closure, XtPointer call_data);
 static void	show_search_result(char *format, ...);
 static void	show_search_msg(char *format, ...);
 
-static void	spell_panel_dismiss();
-static void	spell_select_word();
-static void	spell_correct_word();
+static void	spell_panel_dismiss(Widget widget, XtPointer closure, XtPointer call_data);
+static void	spell_select_word(Widget widget, XtPointer closure, XtPointer call_data);
+static void	spell_correct_word(Widget widget, XtPointer closure, XtPointer call_data);
 static void	show_spell_msg(char *format, ...);
 
 static XtActionsRec search_actions[] =
@@ -94,11 +107,6 @@ static XtActionsRec search_actions[] =
     {"QuitFoundTextPanel", (XtActionProc) found_text_panel_dismiss},
 };
 
-static XtActionsRec found_text_actions[] =
-{
-    {"QuitFoundTextPanel", (XtActionProc)found_text_panel_dismiss},
-};
-
 static XtActionsRec spell_actions[] =
 {
     {"QuitSpellPanel", (XtActionProc)spell_panel_dismiss},
@@ -133,9 +141,8 @@ static Boolean	 do_replace_called;
 
 DeclareStaticArgs(14);
 
-static Boolean compare_string(str, pattern)
-     char *str;
-     char *pattern;
+
+static Boolean compare_string(char *str, char *pattern)
 {
   if (case_sensitive) {
     return strncmp(str, pattern, strlen(pattern)) == 0;
@@ -145,10 +152,7 @@ static Boolean compare_string(str, pattern)
 }
 
 static void
-do_replace(widget, closure, call_data)
-    Widget	    widget;
-    XtPointer	    closure;
-    XtPointer	    call_data;
+do_replace(Widget widget, XtPointer closure, XtPointer call_data)
 {
   int cnt;
 
@@ -161,7 +165,7 @@ do_replace(widget, closure, call_data)
       return;
     }
     replace_text_in_compound(&objects, panel_get_value(search_text_widget),
-                             panel_get_value(replace_text_widget));
+				panel_get_value(replace_text_widget));
     found_text_panel_dismiss();
     redisplay_canvas();
     set_modifiedflag();
@@ -172,10 +176,7 @@ do_replace(widget, closure, call_data)
 }
 
 static Boolean 
-replace_text_in_compound(com, pattern, dst)
-     F_compound *com;
-     char *pattern;
-     char *dst;
+replace_text_in_compound(F_compound *com, char *pattern, char *dst)
 {
   F_compound	*c;
   F_text	*t;
@@ -219,46 +220,28 @@ replace_text_in_compound(com, pattern, dst)
         }
       }
       if (replaced) {  /* replace the text object */
-#ifdef notdef  /* this code can get into infinite loop
-		  when replacing "a" to "A", for example */
-	new_t = copy_text(t);
-	free(new_t->cstring);
-	/* put in new string */
-	new_t->cstring = strdup(str);
-        size = textsize(lookfont(x_fontnum(psfont_text(t), new_t->font),
-                                 new_t->size), strlen(new_t->cstring), new_t->cstring);
-        new_t->ascent = size.ascent;
-        new_t->descent = size.descent;
-        new_t->length = size.length;
-	/* replace old with new in the list */
-	change_text(t, new_t);
-#else  /* here is code from older xfig */
         if (strlen(t->cstring) != strlen(str)) {
           free(t->cstring);
           t->cstring = new_string(strlen(str));
         }
         strcpy(t->cstring, str);
         size = textsize(lookfont(x_fontnum(psfont_text(t), t->font),
-                                 t->size), strlen(t->cstring), t->cstring);
+				t->size), strlen(t->cstring), t->cstring);
         t->ascent = size.ascent;
         t->descent = size.descent;
         t->length = size.length;
-#endif
         processed = True;
       }
     }
   }
   if (processed)
     compound_bound(com, &com->nwcorner.x, &com->nwcorner.y,
-                   &com->secorner.x, &com->secorner.y);
+			&com->secorner.x, &com->secorner.y);
   return processed;
 }
 
 static void 
-do_update(widget, closure, call_data)
-    Widget	    widget;
-    XtPointer	    closure;
-    XtPointer	    call_data;
+do_update(Widget widget, XtPointer closure, XtPointer call_data)
 {
   if (found_text_cnt > 0) {
     search_text_in_compound(&objects,
@@ -272,10 +255,7 @@ do_update(widget, closure, call_data)
 }
 
 static void 
-found_text_panel_dismiss(widget, closure, call_data)
-    Widget	    widget;
-    XtPointer	    closure;
-    XtPointer	    call_data;
+found_text_panel_dismiss(void)
 {
   if (found_text_panel != None) 
 	XtDestroyWidget(found_text_panel);
@@ -321,7 +301,6 @@ static void
 show_search_msg(char *format,...)
 {
   va_list ap;
-  XawTextBlock block;
   static char tmpstr[300];
 
   va_start(ap, format);
@@ -333,8 +312,7 @@ show_search_msg(char *format,...)
 }
 
 static void 
-show_text_object(t)
-     F_text *t;
+show_text_object(F_text *t)
 {
   float x, y;
   char *unit;
@@ -352,10 +330,7 @@ show_text_object(t)
 }
 
 static void 
-search_and_replace_text(widget, closure, call_data)
-    Widget	    widget;
-    XtPointer	    closure;
-    XtPointer	    call_data;
+search_and_replace_text(Widget widget, XtPointer closure, XtPointer call_data)
 {
   char	*string;
 
@@ -390,10 +365,7 @@ search_and_replace_text(widget, closure, call_data)
 }
 
 static Boolean 
-search_text_in_compound(com, pattern, proc)
-     F_compound *com;
-     char *pattern;
-     void (*proc)();
+search_text_in_compound(F_compound *com, char *pattern, void (*proc) (/* ??? */))
 {
   F_compound *c;
   F_text *t;
@@ -423,15 +395,12 @@ search_text_in_compound(com, pattern, proc)
   }
   if (processed)
     compound_bound(com, &com->nwcorner.x, &com->nwcorner.y,
-                   &com->secorner.x, &com->secorner.y);
+			&com->secorner.x, &com->secorner.y);
   return processed;
 }
 
 static void 
-search_panel_dismiss(widget, closure, call_data)
-    Widget	    widget;
-    XtPointer	    closure;
-    XtPointer	    call_data;
+search_panel_dismiss(Widget widget, XtPointer closure, XtPointer call_data)
 {
   found_text_panel_dismiss();
   if (search_panel != None) 
@@ -442,13 +411,16 @@ search_panel_dismiss(widget, closure, call_data)
 /* create and popup the search panel */
 
 void 
-popup_search_panel()
+popup_search_panel(void)
 {
     static Boolean actions_added = False;
     Widget below = None;
     Widget form, label, dismiss_button;
     int rx,ry;
 
+    /* turn off Compose key LED */
+    setCompLED(0);
+
     /* don't paste if in the middle of drawing/editing */
     if (check_action_on())
 	return;
@@ -468,10 +440,10 @@ popup_search_panel()
     NextArg(XtNtitle, "Xfig: Search & Replace");
     
     search_panel = XtCreatePopupShell("search_panel",
-                                      transientShellWidgetClass, tool,
-                                      Args, ArgCount);
+				transientShellWidgetClass, tool,
+				Args, ArgCount);
     XtOverrideTranslations(search_panel,
-                   XtParseTranslationTable(search_panel_translations));
+		XtParseTranslationTable(search_panel_translations));
     if (!actions_added) {
 	XtAppAddActions(tool_app, search_actions, XtNumber(search_actions));
 	actions_added = True;
@@ -481,63 +453,90 @@ popup_search_panel()
     
     FirstArg(XtNlabel, "  Search for:");
     NextArg(XtNborderWidth, 0);
+    NextArg(XtNtop, XtChainTop);
+    NextArg(XtNbottom, XtChainTop);
+    NextArg(XtNleft, XtChainLeft);
+    NextArg(XtNright, XtChainLeft);
     label = XtCreateManagedWidget("search_lab", labelWidgetClass,
-                                  form, Args, ArgCount);
+				form, Args, ArgCount);
     
     FirstArg(XtNfromHoriz, label);
     NextArg(XtNeditType, XawtextEdit);
     NextArg(XtNwidth, 200);
+    NextArg(XtNtop, XtChainTop);
+    NextArg(XtNbottom, XtChainTop);
+    NextArg(XtNleft, XtChainLeft);
+    NextArg(XtNright, XtChainLeft);
     search_text_widget = XtCreateManagedWidget("search_text", asciiTextWidgetClass,
-                                               form, Args, ArgCount);
+				form, Args, ArgCount);
     XtOverrideTranslations(search_text_widget,
-                           XtParseTranslationTable(search_text_translations));
+			XtParseTranslationTable(search_text_translations));
 
     /* search button */
 
     FirstArg(XtNlabel, "Search ");
     NextArg(XtNfromHoriz, search_text_widget);
+    NextArg(XtNtop, XtChainTop);
+    NextArg(XtNbottom, XtChainTop);
+    NextArg(XtNleft, XtChainLeft);
+    NextArg(XtNright, XtChainLeft);
     search_button = XtCreateManagedWidget("search", commandWidgetClass,
-                                          form, Args, ArgCount);
+				form, Args, ArgCount);
     XtAddCallback(search_button, XtNcallback,
                 (XtCallbackProc) search_and_replace_text, (XtPointer) NULL);
   
     (void) CreateCheckbutton("Case sensitive", "case_sensitive", 
-			form, NULL, search_button, True, False, &case_sensitive, 0, 0);
-
+			form, NULL, search_button, MANAGE, SMALL_CHK, &case_sensitive, 0, 0);
     below = label;
 
     FirstArg(XtNfromVert, below);
     NextArg(XtNvertDistance, 6);
     NextArg(XtNborderWidth, 0);
     NextArg(XtNlabel, "Replace with:");
+    NextArg(XtNtop, XtChainTop);
+    NextArg(XtNbottom, XtChainTop);
+    NextArg(XtNleft, XtChainLeft);
+    NextArg(XtNright, XtChainLeft);
     replace_text_label = XtCreateManagedWidget("replace_lab", labelWidgetClass,
-                                  form, Args, ArgCount);
+				form, Args, ArgCount);
 
     FirstArg(XtNfromVert, below);
     NextArg(XtNvertDistance, 6);
     NextArg(XtNfromHoriz, replace_text_label);
     NextArg(XtNeditType, XawtextEdit);
     NextArg(XtNwidth, 200);
+    NextArg(XtNtop, XtChainTop);
+    NextArg(XtNbottom, XtChainTop);
+    NextArg(XtNleft, XtChainLeft);
+    NextArg(XtNright, XtChainLeft);
     replace_text_widget = XtCreateManagedWidget("replace_text", asciiTextWidgetClass,
-                                               form, Args, ArgCount);
+				form, Args, ArgCount);
     XtOverrideTranslations(replace_text_widget,
-                           XtParseTranslationTable(replace_text_translations));
+				XtParseTranslationTable(replace_text_translations));
     
     FirstArg(XtNfromVert, below);
     NextArg(XtNfromHoriz, replace_text_widget);
     NextArg(XtNlabel, "Replace");
+    NextArg(XtNtop, XtChainTop);
+    NextArg(XtNbottom, XtChainTop);
+    NextArg(XtNleft, XtChainLeft);
+    NextArg(XtNright, XtChainLeft);
     do_replace_button = XtCreateManagedWidget("do_replace", commandWidgetClass,
-                                                   form, Args, ArgCount);
+				form, Args, ArgCount);
     XtAddCallback(do_replace_button, XtNcallback,
-                      (XtCallbackProc) do_replace, (XtPointer) NULL);
+			(XtCallbackProc) do_replace, (XtPointer) NULL);
 
     FirstArg(XtNfromVert, below);
     NextArg(XtNfromHoriz, do_replace_button);
     NextArg(XtNlabel, "UPDATE  settings");
+    NextArg(XtNtop, XtChainTop);
+    NextArg(XtNbottom, XtChainTop);
+    NextArg(XtNleft, XtChainLeft);
+    NextArg(XtNright, XtChainLeft);
     do_update_button = XtCreateManagedWidget("dismiss", commandWidgetClass,
-                                        form, Args, ArgCount);
+				form, Args, ArgCount);
     XtAddCallback(do_update_button, XtNcallback,
-                  (XtCallbackProc) do_update, (XtPointer) NULL);
+			(XtCallbackProc) do_update, (XtPointer) NULL);
 
     below = replace_text_widget;
 
@@ -548,7 +547,12 @@ popup_search_panel()
     NextArg(XtNjustify, XtJustifyLeft);
     NextArg(XtNwidth, SEARCH_WIDTH);
     NextArg(XtNheight, 20);
-    search_msg_win = XtCreateManagedWidget("search_msg_win", labelWidgetClass,                                       form, Args, ArgCount);
+    NextArg(XtNtop, XtChainTop);
+    NextArg(XtNbottom, XtChainTop);
+    NextArg(XtNleft, XtChainLeft);
+    NextArg(XtNright, XtChainLeft);
+    search_msg_win = XtCreateManagedWidget("search_msg_win", labelWidgetClass, 
+		    form, Args, ArgCount);
     
     below = search_msg_win;
 
@@ -561,8 +565,14 @@ popup_search_panel()
     NextArg(XtNdisplayCaret, False);
     NextArg(XtNscrollHorizontal, XawtextScrollWhenNeeded);
     NextArg(XtNscrollVertical, XawtextScrollAlways);
+    NextArg(XtNtop, XtChainTop);
+    NextArg(XtNbottom, XtChainBottom);
+    NextArg(XtNleft, XtChainLeft);
+    NextArg(XtNright, XtChainLeft);
     search_results_win = XtCreateManagedWidget("search_results_win", asciiTextWidgetClass,
 			form, Args, ArgCount);
+    XtOverrideTranslations(search_results_win,
+				XtParseTranslationTable(search_results_translations));
     
     below = search_results_win;
 
@@ -570,25 +580,24 @@ popup_search_panel()
 
     FirstArg(XtNfromVert, below);
     NextArg(XtNlabel, "Dismiss");
+    NextArg(XtNtop, XtChainBottom);
+    NextArg(XtNbottom, XtChainBottom);
+    NextArg(XtNleft, XtChainLeft);
+    NextArg(XtNright, XtChainLeft);
     dismiss_button = XtCreateManagedWidget("dismiss", commandWidgetClass,
-                                    form, Args, ArgCount);
+				form, Args, ArgCount);
     XtAddCallback(dismiss_button, XtNcallback,
-                  (XtCallbackProc) search_panel_dismiss, (XtPointer) NULL);
+			(XtCallbackProc) search_panel_dismiss, (XtPointer) NULL);
 
     /* make update/replace buttons insensitive to start */
     XtSetSensitive(replace_text_label, False);
     XtSetSensitive(do_replace_button, False);
     XtSetSensitive(do_update_button, False);
 
-    /* install accelerators for the following functions */
-    XtInstallAccelerators(form, dismiss_button);
-
     XtPopup(search_panel, XtGrabNone);
+
     XSetWMProtocols(tool_d, XtWindow(search_panel), &wm_delete_window, 1);
     set_cmap(XtWindow(search_panel));
-
-    /* install accelerators for the following functions */
-    XtInstallAccelerators(form, dismiss_button);
 }
 
 /***********************/
@@ -596,9 +605,7 @@ popup_search_panel()
 /***********************/
 
 void 
-popup_spell_check_panel(list, nitems)
-    char   **list;
-    int	     nitems;
+popup_spell_check_panel(char **list, int nitems)
 {
   static Boolean actions_added = False;
   Widget form, dismiss_button, below, label;
@@ -616,17 +623,17 @@ popup_spell_check_panel(list, nitems)
     NextArg(XtNcolormap, tool_cm);
     NextArg(XtNtitle, "Xfig: Misspelled words");
     spell_check_panel = XtCreatePopupShell("spell_check_panel",
-                                          transientShellWidgetClass,
-                                          tool, Args, ArgCount);
+				transientShellWidgetClass,
+				tool, Args, ArgCount);
     XtOverrideTranslations(spell_check_panel,
-                   XtParseTranslationTable(spell_panel_translations));
+			XtParseTranslationTable(spell_panel_translations));
     if (!actions_added) {
 	XtAppAddActions(tool_app, spell_actions, XtNumber(spell_actions));
 	actions_added = True;
     }
     
     form = XtCreateManagedWidget("form", formWidgetClass,
-                                 spell_check_panel, NULL, ZERO);
+				spell_check_panel, NULL, ZERO);
 
     /* make a label to report either "No misspelled words" or "Misspelled words:" */
 
@@ -638,7 +645,8 @@ popup_spell_check_panel(list, nitems)
     NextArg(XtNbottom, XtChainTop);
     NextArg(XtNleft, XtChainLeft);
     NextArg(XtNright, XtChainRight);
-    spell_msg_win = XtCreateManagedWidget("spell_msg_win", labelWidgetClass,                                       form, Args, ArgCount);
+    spell_msg_win = XtCreateManagedWidget("spell_msg_win", labelWidgetClass, 
+				form, Args, ArgCount);
     
     /* labels for list and correct word entry */
     FirstArg(XtNlabel, "Misspelled words    ");
@@ -648,7 +656,7 @@ popup_spell_check_panel(list, nitems)
     NextArg(XtNleft, XtChainLeft);
     NextArg(XtNright, XtChainRight);
     label = XtCreateManagedWidget("misspelled_label", labelWidgetClass,
-                                  form, Args, ArgCount);
+				form, Args, ArgCount);
     below = label;
 
     FirstArg(XtNlabel, "Correction");
@@ -659,7 +667,7 @@ popup_spell_check_panel(list, nitems)
     NextArg(XtNleft, XtChainLeft);
     NextArg(XtNright, XtChainRight);
     label = XtCreateManagedWidget("correction_label", labelWidgetClass,
-                                  form, Args, ArgCount);
+				form, Args, ArgCount);
 
     /* make a viewport to hold the list widget containing the misspelled words */
 
@@ -678,8 +686,8 @@ popup_spell_check_panel(list, nitems)
     /* now make the list widget */
     FirstArg(XtNlist, list);
     NextArg(XtNnumberStrings, nitems);
-    NextArg(XtNforceColumns, True);	/* force to one column */
-    NextArg(XtNdefaultColumns, 1);	/* " */
+    NextArg(XtNforceColumns, True);		/* force to one column */
+    NextArg(XtNdefaultColumns, 1);		/* ditto */
     NextArg(XtNwidth, 150);
     NextArg(XtNheight, 200);
     word_list = XtCreateManagedWidget("word_list", figListWidgetClass,
@@ -687,7 +695,8 @@ popup_spell_check_panel(list, nitems)
     XtAddCallback(word_list, XtNcallback, spell_select_word, (XtPointer) NULL);
 
     /* now an ascii widget to put the correct word into */
-    FirstArg(XtNeditType, XawtextEdit);
+    FirstArg(XtNeditType, XawtextRead);		/* make uneditable until user selects a misspelled word */
+    NextArg(XtNsensitive, False);		/* start insensitive */
     NextArg(XtNfromVert, below);
     NextArg(XtNvertDistance, 1);
     NextArg(XtNfromHoriz, spell_viewport);	/* to the right of the viewport */
@@ -695,27 +704,31 @@ popup_spell_check_panel(list, nitems)
     NextArg(XtNtop, XtChainTop);
     NextArg(XtNbottom, XtChainTop);
     NextArg(XtNleft, XtChainLeft);
-    NextArg(XtNright, XtChainRight);
+    NextArg(XtNright, XtChainLeft);
     correct_word = XtCreateManagedWidget("correct_word", asciiTextWidgetClass,
 				     form, Args, ArgCount);
+
     /* "Return" corrects word */
     XtOverrideTranslations(correct_word,
-                   XtParseTranslationTable(spell_text_translations));
+		XtParseTranslationTable(spell_text_translations));
+
+    /* focus keyboard on text widget */
+    XtSetKeyboardFocus(form, correct_word);
 
     /* now "Correct" button to the right */
     FirstArg(XtNlabel, "Correct");
-    NextArg(XtNsensitive, False);	/* start insensitive */
+    NextArg(XtNsensitive, False);		/* start insensitive */
     NextArg(XtNfromVert, below);
     NextArg(XtNvertDistance, 1);
     NextArg(XtNfromHoriz, correct_word);
     NextArg(XtNtop, XtChainTop);
     NextArg(XtNbottom, XtChainTop);
-    NextArg(XtNleft, XtChainRight);
-    NextArg(XtNright, XtChainRight);
+    NextArg(XtNleft, XtChainLeft);
+    NextArg(XtNright, XtChainLeft);
     correct_button = XtCreateManagedWidget("correct", commandWidgetClass,
-                                           form, Args, ArgCount);
+				form, Args, ArgCount);
     XtAddCallback(correct_button, XtNcallback,
-                  (XtCallbackProc) spell_correct_word, (XtPointer) NULL);
+		(XtCallbackProc) spell_correct_word, (XtPointer) NULL);
 
     /* make a re-check spelling button at bottom of whole panel */
 
@@ -727,9 +740,9 @@ popup_spell_check_panel(list, nitems)
     NextArg(XtNleft, XtChainLeft);
     NextArg(XtNright, XtChainLeft);
     recheck_button = XtCreateManagedWidget("recheck", commandWidgetClass,
-                                    form, Args, ArgCount);
+				form, Args, ArgCount);
     XtAddCallback(recheck_button, XtNcallback,
-                  (XtCallbackProc) spell_check, (XtPointer) NULL);
+		(XtCallbackProc) spell_check, (XtPointer) NULL);
 
     /* make dismiss button to the right of the recheck button */
 
@@ -741,12 +754,13 @@ popup_spell_check_panel(list, nitems)
     NextArg(XtNleft, XtChainLeft);
     NextArg(XtNright, XtChainLeft);
     dismiss_button = XtCreateManagedWidget("dismiss", commandWidgetClass,
-                                           form, Args, ArgCount);
+				form, Args, ArgCount);
     XtAddCallback(dismiss_button, XtNcallback,
-                  (XtCallbackProc) spell_panel_dismiss, (XtPointer) NULL);
+		(XtCallbackProc) spell_panel_dismiss, (XtPointer) NULL);
 
     /* install accelerators for the dismiss function */
     XtInstallAccelerators(form, dismiss_button);
+    XtInstallAccelerators(word_list, dismiss_button);
   }
 
   XtPopup(spell_check_panel, XtGrabExclusive);
@@ -758,20 +772,17 @@ popup_spell_check_panel(list, nitems)
 }
 
 static void 
-spell_panel_dismiss(widget, closure, call_data)
-    Widget	    widget;
-    XtPointer	    closure;
-    XtPointer	    call_data;
+spell_panel_dismiss(Widget widget, XtPointer closure, XtPointer call_data)
 {
   if (spell_check_panel != None) 
 	XtDestroyWidget(spell_check_panel);
   spell_check_panel = None;
 }
 
-static void write_text_from_compound();
+static void write_text_from_compound(FILE *fp, F_compound *com);
 
 void 
-spell_check()
+spell_check(void)
 {
   char	  filename[PATH_MAX];
   char	 *cmd;
@@ -781,6 +792,9 @@ spell_check()
   Boolean done = FALSE;
   static int lines = 0;
 
+  /* turn off Compose key LED */
+  setCompLED(0);
+
   put_msg("Spell checking...");
 
   /* free any strings from the previous spelling */
@@ -793,7 +807,7 @@ spell_check()
   sprintf(filename, "%s/xfig-spell.%d", TMPDIR, (int)getpid());
   fp = fopen(filename, "w");
   if (fp == NULL) {
-    file_msg("Can't open temporary file: %s: %s\n", filename, sys_errlist[errno]);
+    file_msg("Can't open temporary file: %s: %s\n", filename, strerror(errno));
   } else {
     /* locate all text objects and write them to file fp */
     write_text_from_compound(fp, &objects);
@@ -809,7 +823,7 @@ spell_check()
         if (str[len - 1] == '\n') 
 	    str[len - 1] = '\0';
 	/* save the word in the list */
-        miss_word_list[lines] = strdup(str);
+        miss_word_list[lines] = my_strdup(str);
         lines++;
 	if (lines >= MAX_MISSPELLED_WORDS)
 	    break;
@@ -823,7 +837,7 @@ spell_check()
     popup_spell_check_panel(miss_word_list, lines);
 
     if (!done) 
-	show_spell_msg("Can't exec \"%s\": %s", cmd, sys_errlist[errno]);
+	show_spell_msg("Can't exec \"%s\": %s", cmd, strerror(errno));
     else if (lines == 0) 
 	show_spell_msg("No misspelled words found");
     else if (lines >= MAX_MISSPELLED_WORDS)
@@ -847,9 +861,7 @@ spell_check()
 /* locate all text objects and write them to file fp */
 
 static void 
-write_text_from_compound(fp, com)
-     FILE *fp;
-     F_compound *com;
+write_text_from_compound(FILE *fp, F_compound *com)
 {
     F_compound *c;
     F_text *t;
@@ -864,30 +876,27 @@ write_text_from_compound(fp, com)
 /* user has selected a word from the list */
 
 static void
-spell_select_word(widget, closure, call_data)
-    Widget	    widget;
-    XtPointer	    closure;
-    XtPointer	    call_data;
+spell_select_word(Widget widget, XtPointer closure, XtPointer call_data)
 {
     XawListReturnStruct *ret_struct = (XawListReturnStruct *) call_data;
 
+    /* make correct button and correction entry sensitive */
+    XtSetSensitive(correct_button, True);
+    XtSetSensitive(correct_word, True);
+
     /* save the selected word */
     strcpy(selected_word, ret_struct->string);
     /* copy the word to the correct_word ascii widget */
     FirstArg(XtNstring, ret_struct->string);
+    NextArg(XtNeditType, XawtextEdit);		/* make editable now */
+    NextArg(XtNsensitive, True);		/* and sensitive */
     SetValues(correct_word);
-
-    /* make correct button sensitive */
-    XtSetSensitive(correct_button, True);
 }
 
 /* correct word that user has selected */
 
 static void
-spell_correct_word(widget, closure, call_data)
-    Widget	    widget;
-    XtPointer	    closure;
-    XtPointer	    call_data;
+spell_correct_word(Widget widget, XtPointer closure, XtPointer call_data)
 {
     char	   *corrected_word;
 
@@ -904,7 +913,6 @@ static void
 show_spell_msg(char *format,...)
 {
   va_list ap;
-  XawTextBlock block;
   static char tmpstr[300];
 
   va_start(ap, format);
